	PROGRAM EXECSYMB
C
C	EXECSYMB -- Executive Server Symbiont
C
C	Version V02.27			(modify PARAMETER statement as well!)
C
C	Version 1 written 21-Jan-1986 by John Osudar
C	Version 2 written 10-Feb-1986 by John Osudar
C
C	Modification history:
C
C	1.00 21-Jan-1986 Created initial version (CMDSYMB) from test program
C	2.00 10-Feb-1986 Created version 2 (EXECSYMB) from CMDSYMB V1.05
C	2.01 11-Feb-1986 Modified to specify spool directory, do ENTER/REMOVE
C	2.02 12-Feb-1986 Fixed ASCII stream item edit code
C	2.03 12-Feb-1986 Added subprocess termination recognition
C	2.04 14-Feb-1986 Modified for general item selection, better wakeup code
C	2.05 14-Feb-1986 Added PARSEMSG to end of code, removed ITEM_CONV
C	2.06 26-Feb-1986 Fixed PARSE_ITEMS code
C	2.07 28-Feb-1986 Added CHECKPOINT/NOCHECKPOINT, COPY=(FIRST,LAST,ALL)
C	2.08 04-Mar-1986 Added code to set queue JOB_RESET_MODULES to reflect
C			 non-default options that were actually selected
C	2.09 05-Mar-1986 Fixed multi-stream status return code, abort code
C	2.10 05-Mar-1986 Fixed subprocess exit code to reset queue
C	2.11 07-Mar-1986 Worked on bug in task abort code that hangs stream
C	2.12 24-Mar-1986 Documented code better; changed channel number to word
C	2.13 25-Mar-1986 Added debugging code activated through status mailbox;
C			 made adjustments to messages logged to operator
C	2.14 01-Apr-1986 Worked on "48042" bug (which causes symbiont to attempt
C			 to abort nonexistent job at queue reset/stop time);
C			 fixed process name length restriction checking and 
C			 related logical name definition
C	2.15 06-Nov-1986 Added USER=username for subprocess username setting;
C			 made queue processors detached processes instead of
C			 subprocesses
C	2.16 07-Nov-1986 Completed conversion to detached processes by 
C			 changing logical names of command and status mailboxes
C	2.17 07-Nov-1986 Added creation of QUEUE_NAME_xxxxxxxx logical to pass
C			 queue name informatio to processor; changed Fortran
C			 I/O to $FAO and other system calls
C	2.18 13-Feb-1987 Added DYNAMIC=dddd hh:mm:ss.cc parameter for dynamic
C			 processor start/stop
C	2.19 18-Feb-1987 Modified operator logging to go to OPER12
C	2.20 10-Jun-1987 Fixed problem with hangup in WRITE_MBX code for ASCII
C			 queue processors
C	2.21 11-Jun-1987 Made further adjustments to WRITE_MBX code to avoid
C			 hanging the symbiont if a processor hangs
C	2.22 15-Jun-1987 Fixed bug with process completion synchronization
C	2.23 16-Jun-1987 Fixed race condition with dynamic process termination
C	2.24 16-Jun-1987 Cleaned up 2.23 fix; converted LOGICAL*1 flags to bit
C			 flags in FLAGS word
C	2.25 15-Jul-1987 Updated to allow 32 streams per process
C	2.26 19-Aug-1987 Worked on mysterious processor exit problem
C	2.27 02-Dec-1987 Added permanent processor code; moved logs to 
C			 SYS_EXECSYMB (if that is defined)
C
C	This program is a VAX/VMS server symbiont that feeds queue entry 
C	information to processes that are designed to execute specific
C	operations.  It is multi-threaded (it can handle up to 32 queues
C	at the same time); it requires one process and one mailbox for each 
C	queue, plus one mailbox for status information.
C
C==============================================================================
C
C	Make implicit declaration and specify all INCLUDE files
C
	IMPLICIT INTEGER*4 (A-Z)
	INCLUDE '($PRVDEF)'
	INCLUDE '($SJCDEF)'
	INCLUDE '($DVIDEF)'
	INCLUDE '($JPIDEF)'
	INCLUDE '($LNMDEF)'
	INCLUDE 'EXECSYMB.INC'
	INCLUDE 'ITEMDEF.INC'
C
C	Version number
C
	CHARACTER*6 VERSION_NUMBER
	PARAMETER (VERSION_NUMBER='V02.27')
C
C	AST routine for message receipt
C
	EXTERNAL MSG_AST
C
C	Create-mailbox definitions
C
	INTEGER*4 SYS$CREMBX,LIB$GETDVI,SYS$CRELNM
C
C	QIO definitions
C
C
C	System services and other general routines called as functions
C
	INTEGER*4 SYS$SETPRN,SYS$HIBER,SYS$SETPRV,SYS$SNDJBCW
	INTEGER*4 SMB$INITIALIZE,SMB$SEND_TO_JOBCTL,LIB$GETJPI
C
C	Our own routines, called as functions
C
	INTEGER*4 DO_REMOVE
C
C	Vectors used with system and symbiont services
C
	INTEGER*4 PRIVILEGES(2),STATUS_VECTOR(2)
C
C	Item list for defining logical name
C
	RECORD /ITEM/ LNM_LIST(2)
	CHARACTER*9 LNMGROUP
	DATA LNMGROUP/'LNM$GROUP'/
C
C	Miscellaneous local variables
C
	INTEGER*2 ACTIVE_STREAMS
	INTEGER*4 IOSB(2)
	LOGICAL*1 STOPPING
	INTEGER*4 ITEM_CODE
	CHARACTER*256 EDITBUF
	CHARACTER*12 STRN
C
C	Process name (used for generating logfile and error messages)
C
	CHARACTER*15 PROCESS_NAME
	COMMON /PRCNAM/ PROCESS_NAME
C
C	Requeue stuff
C
	RECORD /ITEM/ ITEMLIST(6)
	CHARACTER*32 SJC_QUEUE
	INTEGER*4 SJC_ENTRY_NUMBER
	INTEGER*4 SJC_AFTER_TIME(2)
C
C	Item conversion information
C
C	Each item has an item name, the length of the name, and an item
C	conversion type, used for converting the item value to ASCII for
C	streams requiring ASCII-format item values.  The following table
C	lists the possible item types:
C	      type	meaning
C	      ----	-------
C		1	No translation (already ASCII, or item not easily
C			translated to ASCII, so left in original form)
C		2	Binary longword, converted to hexadecimal in the
C			format "%Xnnnnnnnn"
C		3	Date and time, converted to dd-mmm-yyyy hh:mm:ss.cc
C			format (with one space between date and time)
C		4	UIC, converted to [gggggg,mmmmmm] format with octal
C			group and member numbers containing leading zeroes.
C		5	Bitstring, converted to a list of which bits are
C			set.  Up to 16 bytes of data can be included in the
C			item (since the CHARACTERISTICS item is 16 bytes long.)
C			The list is a series of decimal numbers separated by
C			commas (e.g. hex 641 becomes "0,6,9,10")
C
	CHARACTER*21 SMBITEM(SMBMSG$K_MAX_ITEM_CODE-1)
	BYTE SMBITEMLEN(SMBMSG$K_MAX_ITEM_CODE-1)
	BYTE SMBITEMTYPE(SMBMSG$K_MAX_ITEM_CODE-1)
	COMMON /SMBITEMINFO/ SMBITEM,SMBITEMLEN,SMBITEMTYPE
C
	DATA SMBITEM(SMBMSG$K_ACCOUNTING_DATA) /'ACCOUNTING_DATA'/
	DATA SMBITEMLEN(SMBMSG$K_ACCOUNTING_DATA) /15/
	DATA SMBITEMTYPE(SMBMSG$K_ACCOUNTING_DATA) /1/
	DATA SMBITEM(SMBMSG$K_ACCOUNT_NAME) /'ACCOUNT_NAME'/
	DATA SMBITEMLEN(SMBMSG$K_ACCOUNT_NAME) /12/
	DATA SMBITEMTYPE(SMBMSG$K_ACCOUNT_NAME) /1/
	DATA SMBITEM(SMBMSG$K_AFTER_TIME) /'AFTER_TIME'/
	DATA SMBITEMLEN(SMBMSG$K_AFTER_TIME) /10/
	DATA SMBITEMTYPE(SMBMSG$K_AFTER_TIME) /3/
	DATA SMBITEM(SMBMSG$K_ALIGNMENT_PAGES) /'ALIGNMENT_PAGES'/
	DATA SMBITEMLEN(SMBMSG$K_ALIGNMENT_PAGES) /15/
	DATA SMBITEMTYPE(SMBMSG$K_ALIGNMENT_PAGES) /2/
	DATA SMBITEM(SMBMSG$K_BOTTOM_MARGIN) /'BOTTOM_MARGIN'/
	DATA SMBITEMLEN(SMBMSG$K_BOTTOM_MARGIN) /13/
	DATA SMBITEMTYPE(SMBMSG$K_BOTTOM_MARGIN) /2/
	DATA SMBITEM(SMBMSG$K_CHARACTERISTICS) /'CHARACTERISTICS'/
	DATA SMBITEMLEN(SMBMSG$K_CHARACTERISTICS) /15/
	DATA SMBITEMTYPE(SMBMSG$K_CHARACTERISTICS) /5/
	DATA SMBITEM(SMBMSG$K_CHECKPOINT_DATA) /'CHECKPOINT_DATA'/
	DATA SMBITEMLEN(SMBMSG$K_CHECKPOINT_DATA) /15/
	DATA SMBITEMTYPE(SMBMSG$K_CHECKPOINT_DATA) /1/
	DATA SMBITEM(SMBMSG$K_CONDITION_VECTOR) /'CONDITION_VECTOR'/
	DATA SMBITEMLEN(SMBMSG$K_CONDITION_VECTOR) /16/
	DATA SMBITEMTYPE(SMBMSG$K_CONDITION_VECTOR) /1/
	DATA SMBITEM(SMBMSG$K_DEVICE_NAME) /'DEVICE_NAME'/
	DATA SMBITEMLEN(SMBMSG$K_DEVICE_NAME) /11/
	DATA SMBITEMTYPE(SMBMSG$K_DEVICE_NAME) /1/
	DATA SMBITEM(SMBMSG$K_DEVICE_STATUS) /'DEVICE_STATUS'/
	DATA SMBITEMLEN(SMBMSG$K_DEVICE_STATUS) /13/
	DATA SMBITEMTYPE(SMBMSG$K_DEVICE_STATUS) /5/
	DATA SMBITEM(SMBMSG$K_ENTRY_NUMBER) /'ENTRY_NUMBER'/
	DATA SMBITEMLEN(SMBMSG$K_ENTRY_NUMBER) /12/
	DATA SMBITEMTYPE(SMBMSG$K_ENTRY_NUMBER) /2/
	DATA SMBITEM(SMBMSG$K_EXECUTOR_QUEUE) /'EXECUTOR_QUEUE'/
	DATA SMBITEMLEN(SMBMSG$K_EXECUTOR_QUEUE) /14/
	DATA SMBITEMTYPE(SMBMSG$K_EXECUTOR_QUEUE) /1/
	DATA SMBITEM(SMBMSG$K_FILE_COPIES) /'FILE_COPIES'/
	DATA SMBITEMLEN(SMBMSG$K_FILE_COPIES) /11/
	DATA SMBITEMTYPE(SMBMSG$K_FILE_COPIES) /2/
	DATA SMBITEM(SMBMSG$K_FILE_COUNT) /'FILE_COUNT'/
	DATA SMBITEMLEN(SMBMSG$K_FILE_COUNT) /10/
	DATA SMBITEMTYPE(SMBMSG$K_FILE_COUNT) /2/
	DATA SMBITEM(SMBMSG$K_FILE_SETUP_MODULES) /'FILE_SETUP_MODULES'/
	DATA SMBITEMLEN(SMBMSG$K_FILE_SETUP_MODULES) /18/
	DATA SMBITEMTYPE(SMBMSG$K_FILE_SETUP_MODULES) /1/
	DATA SMBITEM(SMBMSG$K_FIRST_PAGE) /'FIRST_PAGE'/
	DATA SMBITEMLEN(SMBMSG$K_FIRST_PAGE) /10/
	DATA SMBITEMTYPE(SMBMSG$K_FIRST_PAGE) /2/
	DATA SMBITEM(SMBMSG$K_FORM_LENGTH) /'FORM_LENGTH'/
	DATA SMBITEMLEN(SMBMSG$K_FORM_LENGTH) /11/
	DATA SMBITEMTYPE(SMBMSG$K_FORM_LENGTH) /2/
	DATA SMBITEM(SMBMSG$K_FORM_NAME) /'FORM_NAME'/
	DATA SMBITEMLEN(SMBMSG$K_FORM_NAME) /9/
	DATA SMBITEMTYPE(SMBMSG$K_FORM_NAME) /1/
	DATA SMBITEM(SMBMSG$K_FORM_SETUP_MODULES) /'FORM_SETUP_MODULES'/
	DATA SMBITEMLEN(SMBMSG$K_FORM_SETUP_MODULES) /18/
	DATA SMBITEMTYPE(SMBMSG$K_FORM_SETUP_MODULES) /1/
	DATA SMBITEM(SMBMSG$K_FORM_WIDTH) /'FORM_WIDTH'/
	DATA SMBITEMLEN(SMBMSG$K_FORM_WIDTH) /10/
	DATA SMBITEMTYPE(SMBMSG$K_FORM_WIDTH) /2/
	DATA SMBITEM(SMBMSG$K_FILE_IDENTIFICATION) /'FILE_IDENTIFICATION'/
	DATA SMBITEMLEN(SMBMSG$K_FILE_IDENTIFICATION) /19/
	DATA SMBITEMTYPE(SMBMSG$K_FILE_IDENTIFICATION) /1/
	DATA SMBITEM(SMBMSG$K_FILE_SPECIFICATION) /'FILE_SPECIFICATION'/
	DATA SMBITEMLEN(SMBMSG$K_FILE_SPECIFICATION) /18/
	DATA SMBITEMTYPE(SMBMSG$K_FILE_SPECIFICATION) /1/
	DATA SMBITEM(SMBMSG$K_JOB_COPIES) /'JOB_COPIES'/
	DATA SMBITEMLEN(SMBMSG$K_JOB_COPIES) /10/
	DATA SMBITEMTYPE(SMBMSG$K_JOB_COPIES) /2/
	DATA SMBITEM(SMBMSG$K_JOB_COUNT) /'JOB_COUNT'/
	DATA SMBITEMLEN(SMBMSG$K_JOB_COUNT) /9/
	DATA SMBITEMTYPE(SMBMSG$K_JOB_COUNT) /2/
	DATA SMBITEM(SMBMSG$K_JOB_NAME) /'JOB_NAME'/
	DATA SMBITEMLEN(SMBMSG$K_JOB_NAME) /8/
	DATA SMBITEMTYPE(SMBMSG$K_JOB_NAME) /1/
	DATA SMBITEM(SMBMSG$K_JOB_RESET_MODULES) /'JOB_RESET_MODULES'/
	DATA SMBITEMLEN(SMBMSG$K_JOB_RESET_MODULES) /17/
	DATA SMBITEMTYPE(SMBMSG$K_JOB_RESET_MODULES) /1/
	DATA SMBITEM(SMBMSG$K_LAST_PAGE) /'LAST_PAGE'/
	DATA SMBITEMLEN(SMBMSG$K_LAST_PAGE) /9/
	DATA SMBITEMTYPE(SMBMSG$K_LAST_PAGE) /2/
	DATA SMBITEM(SMBMSG$K_LEFT_MARGIN) /'LEFT_MARGIN'/
	DATA SMBITEMLEN(SMBMSG$K_LEFT_MARGIN) /11/
	DATA SMBITEMTYPE(SMBMSG$K_LEFT_MARGIN) /2/
	DATA SMBITEM(SMBMSG$K_LIBRARY_SPECIFICATION) /'LIBRARY_SPECIFICATION'/
	DATA SMBITEMLEN(SMBMSG$K_LIBRARY_SPECIFICATION) /21/
	DATA SMBITEMTYPE(SMBMSG$K_LIBRARY_SPECIFICATION) /1/
	DATA SMBITEM(SMBMSG$K_MAXIMUM_STREAMS) /'MAXIMUM_STREAMS'/
	DATA SMBITEMLEN(SMBMSG$K_MAXIMUM_STREAMS) /15/
	DATA SMBITEMTYPE(SMBMSG$K_MAXIMUM_STREAMS) /2/
	DATA SMBITEM(SMBMSG$K_MESSAGE_VECTOR) /'MESSAGE_VECTOR'/
	DATA SMBITEMLEN(SMBMSG$K_MESSAGE_VECTOR) /14/
	DATA SMBITEMTYPE(SMBMSG$K_MESSAGE_VECTOR) /1/
	DATA SMBITEM(SMBMSG$K_NOTE) /'NOTE'/
	DATA SMBITEMLEN(SMBMSG$K_NOTE) /4/
	DATA SMBITEMTYPE(SMBMSG$K_NOTE) /1/
	DATA SMBITEM(SMBMSG$K_PAGE_SETUP_MODULES) /'PAGE_SETUP_MODULES'/
	DATA SMBITEMLEN(SMBMSG$K_PAGE_SETUP_MODULES) /18/
	DATA SMBITEMTYPE(SMBMSG$K_PAGE_SETUP_MODULES) /1/
	DATA SMBITEM(SMBMSG$K_PARAMETER_1) /'PARAMETER_1'/
	DATA SMBITEMLEN(SMBMSG$K_PARAMETER_1) /11/
	DATA SMBITEMTYPE(SMBMSG$K_PARAMETER_1) /1/
	DATA SMBITEM(SMBMSG$K_PARAMETER_2) /'PARAMETER_2'/
	DATA SMBITEMLEN(SMBMSG$K_PARAMETER_2) /11/
	DATA SMBITEMTYPE(SMBMSG$K_PARAMETER_2) /1/
	DATA SMBITEM(SMBMSG$K_PARAMETER_3) /'PARAMETER_3'/
	DATA SMBITEMLEN(SMBMSG$K_PARAMETER_3) /11/
	DATA SMBITEMTYPE(SMBMSG$K_PARAMETER_3) /1/
	DATA SMBITEM(SMBMSG$K_PARAMETER_4) /'PARAMETER_4'/
	DATA SMBITEMLEN(SMBMSG$K_PARAMETER_4) /11/
	DATA SMBITEMTYPE(SMBMSG$K_PARAMETER_4) /1/
	DATA SMBITEM(SMBMSG$K_PARAMETER_5) /'PARAMETER_5'/
	DATA SMBITEMLEN(SMBMSG$K_PARAMETER_5) /11/
	DATA SMBITEMTYPE(SMBMSG$K_PARAMETER_5) /1/
	DATA SMBITEM(SMBMSG$K_PARAMETER_6) /'PARAMETER_6'/
	DATA SMBITEMLEN(SMBMSG$K_PARAMETER_6) /11/
	DATA SMBITEMTYPE(SMBMSG$K_PARAMETER_6) /1/
	DATA SMBITEM(SMBMSG$K_PARAMETER_7) /'PARAMETER_7'/
	DATA SMBITEMLEN(SMBMSG$K_PARAMETER_7) /11/
	DATA SMBITEMTYPE(SMBMSG$K_PARAMETER_7) /1/
	DATA SMBITEM(SMBMSG$K_PARAMETER_8) /'PARAMETER_8'/
	DATA SMBITEMLEN(SMBMSG$K_PARAMETER_8) /11/
	DATA SMBITEMTYPE(SMBMSG$K_PARAMETER_8) /1/
	DATA SMBITEM(SMBMSG$K_PRINT_CONTROL) /'PRINT_CONTROL'/
	DATA SMBITEMLEN(SMBMSG$K_PRINT_CONTROL) /13/
	DATA SMBITEMTYPE(SMBMSG$K_PRINT_CONTROL) /5/
	DATA SMBITEM(SMBMSG$K_PRIORITY) /'PRIORITY'/
	DATA SMBITEMLEN(SMBMSG$K_PRIORITY) /8/
	DATA SMBITEMTYPE(SMBMSG$K_PRIORITY) /2/
	DATA SMBITEM(SMBMSG$K_QUEUE) /'QUEUE'/
	DATA SMBITEMLEN(SMBMSG$K_QUEUE) /5/
	DATA SMBITEMTYPE(SMBMSG$K_QUEUE) /1/
	DATA SMBITEM(SMBMSG$K_REFUSE_REASON) /'REFUSE_REASON'/
	DATA SMBITEMLEN(SMBMSG$K_REFUSE_REASON) /13/
	DATA SMBITEMTYPE(SMBMSG$K_REFUSE_REASON) /1/
	DATA SMBITEM(SMBMSG$K_RELATIVE_PAGE) /'RELATIVE_PAGE'/
	DATA SMBITEMLEN(SMBMSG$K_RELATIVE_PAGE) /13/
	DATA SMBITEMTYPE(SMBMSG$K_RELATIVE_PAGE) /2/
	DATA SMBITEM(SMBMSG$K_REQUEST_CONTROL) /'REQUEST_CONTROL'/
	DATA SMBITEMLEN(SMBMSG$K_REQUEST_CONTROL) /15/
	DATA SMBITEMTYPE(SMBMSG$K_REQUEST_CONTROL) /5/
	DATA SMBITEM(SMBMSG$K_REQUEST_RESPONSE) /'REQUEST_RESPONSE'/
	DATA SMBITEMLEN(SMBMSG$K_REQUEST_RESPONSE) /16/
	DATA SMBITEMTYPE(SMBMSG$K_REQUEST_RESPONSE) /2/
	DATA SMBITEM(SMBMSG$K_RIGHT_MARGIN) /'RIGHT_MARGIN'/
	DATA SMBITEMLEN(SMBMSG$K_RIGHT_MARGIN) /12/
	DATA SMBITEMTYPE(SMBMSG$K_RIGHT_MARGIN) /2/
	DATA SMBITEM(SMBMSG$K_SEARCH_STRING) /'SEARCH_STRING'/
	DATA SMBITEMLEN(SMBMSG$K_SEARCH_STRING) /13/
	DATA SMBITEMTYPE(SMBMSG$K_SEARCH_STRING) /1/
	DATA SMBITEM(SMBMSG$K_SEPARATION_CONTROL) /'SEPARATION_CONTROL'/
	DATA SMBITEMLEN(SMBMSG$K_SEPARATION_CONTROL) /18/
	DATA SMBITEMTYPE(SMBMSG$K_SEPARATION_CONTROL) /5/
	DATA SMBITEM(SMBMSG$K_STOP_CONDITION) /'STOP_CONDITION'/
	DATA SMBITEMLEN(SMBMSG$K_STOP_CONDITION) /14/
	DATA SMBITEMTYPE(SMBMSG$K_STOP_CONDITION) /2/
	DATA SMBITEM(SMBMSG$K_TIME_QUEUED) /'TIME_QUEUED'/
	DATA SMBITEMLEN(SMBMSG$K_TIME_QUEUED) /11/
	DATA SMBITEMTYPE(SMBMSG$K_TIME_QUEUED) /3/
	DATA SMBITEM(SMBMSG$K_TOP_MARGIN) /'TOP_MARGIN'/
	DATA SMBITEMLEN(SMBMSG$K_TOP_MARGIN) /10/
	DATA SMBITEMTYPE(SMBMSG$K_TOP_MARGIN) /2/
	DATA SMBITEM(SMBMSG$K_UIC) /'UIC'/
	DATA SMBITEMLEN(SMBMSG$K_UIC) /3/
	DATA SMBITEMTYPE(SMBMSG$K_UIC) /4/
	DATA SMBITEM(SMBMSG$K_USER_NAME) /'USER_NAME'/
	DATA SMBITEMLEN(SMBMSG$K_USER_NAME) /9/
	DATA SMBITEMTYPE(SMBMSG$K_USER_NAME) /1/
C
C	Miscellaneous data statements
C
	INTEGER*4 DESIRED_PRIVS
	PARAMETER (DESIRED_PRIVS=PRV$M_TMPMBX+PRV$M_CMKRNL+PRV$M_DETACH+
	1 PRV$M_SYSPRV)
	DATA PRIVILEGES/DESIRED_PRIVS,0/,STATUS_VECTOR/1,0/
	DATA PROCESS_NAME/'SYMBIONT_EXEC_0'/
C==============================================================================

C==============================================================================
C
C	Start of code
C
C	Set our process name to "SYMBIONT_EXEC_n", n=1,...,9,A,...,Z -- 
C	whichever is the next one not in use.
C
500	IF(PROCESS_NAME(15:15).EQ.'9')THEN
	   I=ICHAR('A')
	ELSE
	   I=ICHAR(PROCESS_NAME(15:15))+1
	ENDIF
	PROCESS_NAME(15:15)=CHAR(I)
	ISTAT=SYS$SETPRN(%DESCR(PROCESS_NAME))
	IF((ISTAT.AND.1).EQ.0)GOTO500
C
C	Inform operator that process is starting, and set up log file
C
	CALL TELL_OPER('EXECSYMB '//VERSION_NUMBER//' process starting',3)
	LOGDIR(1:12)='SYS_EXECSYMB'
	L_LD=12
	OPEN(UNIT=1,NAME=LOGDIR(1:L_LD)//':EXECSYMB'//PROCESS_NAME(15:15)
	1 //'.LOG',TYPE='NEW',CARRIAGECONTROL='LIST',ERR=501)
	GOTO502
501	LOGDIR(1:11)='SYS$MANAGER'
	L_LD=11
	OPEN(UNIT=1,NAME=LOGDIR(1:L_LD)//':EXECSYMB'//PROCESS_NAME(15:15)
	1 //'.LOG',TYPE='NEW',CARRIAGECONTROL='LIST',ERR=99)
502	CLOSE(UNIT=1)
C
C	Set needed privileges and determine base priority (to propagate to
C	processes)
C
	ISTAT=SYS$SETPRV(%VAL(1),%REF(PRIVILEGES),%VAL(1),)
	CALL LOGGIT('$SETPRV status = ',ISTAT)
	BASE_PRIO=4
	ISTAT=LIB$GETJPI(%REF(JPI$_PRIB),,,BASE_PRIO)
	CALL LOGGIT('LIB$GETJPI status = ',ISTAT)
C
C	Do the equivalent of:
C
C	DEFINE/TABLE=LNM$PROCESS_DIRECTORY LNM$TEMPORARY_MAILBOX LNM$GROUP
C
C	telling the system to put the logical names of temporary mailboxes
C	into the group logical name table (which will be LNM$GROUP_000001)
C
	LNM_LIST(1).BUFLEN=9
	LNM_LIST(1).ITEMCODE=LNM$_STRING
	LNM_LIST(1).BUFADR=%LOC(LNMGROUP)
	LNM_LIST(1).RETLENADR=0
	LNM_LIST(2).TERMINATOR=0
	ISTAT=SYS$CRELNM(,'LNM$PROCESS_DIRECTORY','LNM$TEMPORARY_MAILBOX',
	1 ,LNM_LIST)
	CALL LOGGIT('SYS$CRELNM status = ',ISTAT)
C
C	Create mailboxes for status return and process exit detection,
C	and do initial reads to them (with AST on completion)
C	Effective with V2.16, the status mailbox name is STAT_W_MBX_x
C	where "x" is the last character of our process name, SYMBIONT_EXEC_x.
C	This is necessary now that these logicals are in the group table,
C	since more than one copy of the symbiont may be active.
C
C	Similarly, the subprocess exit mailbox is called SUBPROC_EXIT_MBX_x.
C
	ISTAT=SYS$CREMBX(,%REF(STAT_W_MBX_CHAN),%VAL(12),,,%VAL('F000'X),
	1 %DESCR('STAT_W_MBX'//PROCESS_NAME(14:15)))
	CALL LOGGIT('SYS$CREMBX STAT_W_MBX status = ',ISTAT)
	CALL READ_STAT_MBX
	ISTAT=SYS$CREMBX(,%REF(EXIT_MBX_CHAN),%VAL(132),,,%VAL('F000'X),
	1 %DESCR('SUBPROC_EXIT_MBX'//PROCESS_NAME(14:15)))
	CALL LOGGIT('SYS$CREMBX EXIT_MBX status = ',ISTAT)
	CALL READ_EXIT_MBX
	ISTAT=LIB$GETDVI(%REF(DVI$_UNIT),%REF(EXIT_MBX_CHAN),,
	1 %REF(EXIT_MBX_UNIT))
	CALL LOGGIT('LIB$GETDVI status = ',ISTAT)
C
C	Initialize SCB's to indicate all streams inactive
C
	DO I=0,MAX_STREAMS-1
	   STREAM(I).PID=0
	   STREAM(I).MBX_CHAN=-1
	   STREAM(I).REQUEST=SMBMSG$K_STOP_STREAM
	ENDDO
C
C	Set up itemlist for modifying aborted job for requeue
C
	ITEMLIST(1).ITEMCODE=SJC$_QUEUE
	ITEMLIST(1).BUFADR=%LOC(SJC_QUEUE)
	ITEMLIST(1).RETLENADR=0
	ITEMLIST(2).ITEMCODE=SJC$_ENTRY_NUMBER
	ITEMLIST(2).BUFLEN=4
	ITEMLIST(2).BUFADR=%LOC(SJC_ENTRY_NUMBER)
	ITEMLIST(2).RETLENADR=0
	ITEMLIST(3).ITEMCODE=SJC$_NO_HOLD
	ITEMLIST(3).BUFLEN=0
	ITEMLIST(3).BUFADR=0
	ITEMLIST(3).RETLENADR=0
	ITEMLIST(4).ITEMCODE=SJC$_AFTER_TIME
	ITEMLIST(4).BUFLEN=8
	ITEMLIST(4).BUFADR=%LOC(SJC_AFTER_TIME(1))
	ITEMLIST(4).RETLENADR=0
	ITEMLIST(5).ITEMCODE=SJC$_NO_CHECKPOINT_DATA
	ITEMLIST(5).BUFLEN=0
	ITEMLIST(5).BUFADR=0
	ITEMLIST(5).RETLENADR=0
	ITEMLIST(6).TERMINATOR=0
C
C	Now, tell the job controller that we are really a symbiont, and
C	let it know how many streams we can handle
C
	ISTAT=SMB$INITIALIZE(%REF(SMBMSG$K_STRUCTURE_LEVEL),
	1 %REF(MSG_AST),%REF(MAX_STREAMS))
	CALL LOGGIT('SMB$INITIALIZE status = ',ISTAT)
C==============================================================================

C==============================================================================
C
C	Main processing loop
C
C	For each pass, set hibernation flag and initialize active stream count
C
1000	DO_HIBER=.TRUE.
	ACTIVE_STREAMS=0
C
C	Repeat loop for each stream
C
	DO I=0,MAX_STREAMS-1
C
C	START_STREAM indicates that stream is started but idle
C
	   IF(STREAM(I).REQUEST.EQ.SMBMSG$K_START_STREAM)THEN
	      ACTIVE_STREAMS=ACTIVE_STREAMS+1
C
C	START_TASK indicates that task is being processed
C	RESET_STREAM is present only if task was active when stream was 
C	reset; handle task abort, then change request to STOP_STREAM
C
	   ELSE IF(STREAM(I).REQUEST.EQ.SMBMSG$K_START_TASK.OR.
	1          STREAM(I).REQUEST.EQ.SMBMSG$K_RESET_STREAM)THEN
	      ACTIVE_STREAMS=ACTIVE_STREAMS+1
C
C	Active task can be in one of five states, given by INDEX:
C
C		1	Task being started
C		2	Task processing (sending items to process)
C		3	Task waiting for status to be returned
C		4	Task completing
C		5	Task being aborted
C
	      GOTO(1010,1020,1030,1040,1050),STREAM(I).INDEX
	      CALL SYS$FAO(' !2UB=!4XW',,STRN,%VAL(I),%VAL(STREAM(I).INDEX))
	      CALL LOGGIT('Invalid index value for stream '//STRN,)
	      GOTO1040
C
C	Task being started -- verify that old processor is not exiting; if so,
C				initialize item index and go on; otherwise
C				retain this item index and skip this stream
C				(allowing hibernation) until old processor exits
C
1010	      IF(BTEST(STREAM(I).FLAGS,FLG_V_EXITING))GOTO189
	      STREAM(I).ITEMNO=0
	      STREAM(I).INDEX=STREAM(I).INDEX+1
	      GOTO1021
C
C	Task processing -- if last item was read, send next item and go on
C
1020	      IF(STREAM(I).IOSB(1).EQ.0)GOTO189
1021	      STREAM(I).ITEMNO=STREAM(I).ITEMNO+1
	      ITEM_CODE=STREAM(I).ORDER(STREAM(I).ITEMNO)
C
C	Zero item code indicates end of items, and requires command "item"
C	to be sent; otherwise, send item as requested
C
	      IF(ITEM_CODE.EQ.0)GOTO1025
	      L=STREAM(I).SIZE(ITEM_CODE)
C
C	If item is null and stream specified NONULL, skip item
C
	      IF((L.EQ.0).AND.(.NOT.BTEST(STREAM(I).FLAGS,FLG_V_NULL)))GOTO1021
	      IP=STREAM(I).POS(ITEM_CODE)
	      IF(IP.EQ.0)GOTO1021
C
C	Send item in binary form or ASCII form, as specified for stream
C
	      IF(BTEST(STREAM(I).FLAGS,FLG_V_BINARY))THEN
	         CALL WRITE_MBX_BINARY(I,ITEM_CODE,STREAM(I).MSG(IP:IP+L-1))
	      ELSE
	         IT=SMBITEMTYPE(ITEM_CODE)
	         IF(IT.EQ.1)THEN
	            CALL WRITE_MBX_ASCII(I,SMBITEM(ITEM_CODE)(1:
	1            SMBITEMLEN(ITEM_CODE)),STREAM(I).MSG(IP:IP+L-1))
	         ELSE
	            CALL ITEMEDIT(EDITBUF,STREAM(I).MSG(IP:IP+L-1),LE,L,
	1            SMBITEMTYPE(ITEM_CODE))
	            CALL WRITE_MBX_ASCII(I,SMBITEM(ITEM_CODE)(1:
	1            SMBITEMLEN(ITEM_CODE)),EDITBUF(1:LE))
	         ENDIF
	      ENDIF
	      GOTO1020
C
C	After all items sent, initialize status to zero (indicating status
C	not yet received) and write the "EXECUTE" command pseudo-item to the 
C	process; then, enter wait-for-status state
C
1025	      STREAM(I).STATUS=0
	      IF(BTEST(STREAM(I).FLAGS,FLG_V_BINARY))THEN
	         CALL WRITE_MBX_BINARY(I,%REF(0),'EXECUTE')
	      ELSE
	         CALL WRITE_MBX_ASCII(I,'EXEC_STEP','EXECUTE')
	      ENDIF
	      GOTO188
C
C	Waiting for status -- once status is nonzero, go on
C
1030	      IF(STREAM(I).STATUS.EQ.0)GOTO189
	      STREAM(I).INDEX=STREAM(I).INDEX+1
C
C	Task completion state -- if reached normally, clear abort flag
C
1040	      STOPPING=.FALSE.
C
C	Merge point from abort code; get status and do completion processing
C
1041	      STATUS_VECTOR(2)=STREAM(I).STATUS
C
C	If file was a spool file entered by the AST routine, we must remove
C	its directory entry (since the job controller will delete the file,
C	but won't remove the entry)
C
	      IF(BTEST(STREAM(I).FLAGS,FLG_V_ENTERED))THEN
	         CALL SETAST(%REF(0))
	         STREAM(I).FLAGS=STREAM(I).FLAGS.AND..NOT.FLG_M_ENTERED
	         IX=INDEX(STREAM(I).FILESPEC(1:STREAM(I).FILESPEC_LEN),']')
	         ISTAT=DO_REMOVE(
	1         STREAM(I).DEVICE_NAME(1:STREAM(I).DEVICE_NAME_LEN)//':'//
	2         STREAM(I).SPOOLDIR(1:STREAM(I).SPOOLDIR_LEN),STREAM(I).FID,
	3         STREAM(I).FILESPEC(IX+1:STREAM(I).FILESPEC_LEN))
	         CALL SETAST(%REF(1))
	         CALL LOGGIT('DO_REMOVE status = ',ISTAT)
	      ENDIF
C
C	Reset stream state to idle and set dynamic exit timer (if dynamic)
C
	      STREAM(I).REQUEST=SMBMSG$K_START_STREAM
	      CALL SET_TIMER(I)
C
C	If task was aborted, determine whether we did it and proceed accordingly
C
	      IF(STOPPING)THEN
C
C	If we aborted the task for requeuing, requeue it (and set checkpoint
C	indicator according to stream specifications)
C
	         IF(BTEST(STREAM(I).FLAGS,FLG_V_ABORT))THEN
	            SJC_QUEUE=STREAM(I).QUEUENAME(1:STREAM(I).QUEUENAME_LEN)
	            ITEMLIST(1).BUFLEN=STREAM(I).QUEUENAME_LEN
	            SJC_ENTRY_NUMBER=STREAM(I).ENTRY
	            SJC_AFTER_TIME(1)=STREAM(I).REQUEUE_TIME(1)
	            SJC_AFTER_TIME(2)=STREAM(I).REQUEUE_TIME(2)
	            IF(BTEST(STREAM(I).FLAGS,FLG_V_CHECKPOINT))THEN
	                ITEMLIST(5).TERMINATOR=0
	            ELSE
	               	ITEMLIST(5).ITEMCODE=SJC$_NO_CHECKPOINT_DATA
	                ITEMLIST(6).TERMINATOR=0
	            ENDIF
	         ENDIF
C
C	Since we can't requeue an active task, and this task got a STOP_TASK
C	request, complete the request by returning a status to the job 
C	controller.  This will put the job back into the queue, where we can 
C	manipulate it.
C
	         ISTAT=SMB$SEND_TO_JOBCTL(%REF(I),
	1         %REF(SMBMSG$K_STOP_TASK),,,,%REF(STATUS_VECTOR))
	         CALL LOGGIT('SEND_TO_JOBCTL STOP status = ',ISTAT)
C
C	If we are requeuing the job, modify it to specify the AFTER-time, 
C	and release the HOLD on it.  (NOTE:  Prior to VMS V4.2, this didn't
C	work correctly in one step, i.e. you couldn't both RELEASE the job
C	and set its AFTER attribute in one operation.  The roundabout 
C	solution, used by Jan and Don Provan in their MFENET code, was to 
C	alter the held job to have characteristic 127, release the job, and 
C	finally modify the released job to both clear characteristic 127 and 
C	set the AFTER-time, which worked.  I am assuming that this will run 
C	under VMS V4.2 or later, so that method isn't necessary.)
C
	         IF(BTEST(STREAM(I).FLAGS,FLG_V_ABORT))THEN
	            ISTAT=SYS$SNDJBCW(,%VAL(SJC$_ALTER_JOB),,%REF(ITEMLIST),
	1            IOSB,,)
	            CALL LOGGIT('$SNDJBCW ALTER status = ',ISTAT)
	            CALL LOGGIT('$SNDJBCW ALTER IOSB = ',IOSB(1))
	         ENDIF
	      ELSE
C
C	If we are not killing the job, send a TASK_COMPLETE status to the 
C	job controller, completing the task and making the stream ready for
C	another job.
C
	         ISTAT=SMB$SEND_TO_JOBCTL(%REF(I),
	1         %REF(SMBMSG$K_TASK_COMPLETE),,,,%REF(STATUS_VECTOR))
	         CALL LOGGIT('SEND_TO_JOBCTL COMPLETE status = ',ISTAT)
	      ENDIF
	      GOTO189
C
C	Aborting the task -- set the STOPPING flag, and determine which of 
C	several scenarios is occurring.
C
1050	      STOPPING=.TRUE.
C
C	If stream is being reset, we need to kill the job, tell the 
C	process to exit, and set the stream status to STOP_STREAM
C
	      IF(STREAM(I).REQUEST.EQ.SMBMSG$K_RESET_STREAM)THEN
	         CALL SETAST(%REF(0))
	         IF(BTEST(STREAM(I).FLAGS,FLG_V_DYNAMIC).AND.
	1         BTEST(STREAM(I).FLAGS,FLG_V_NO_PROCESSOR))THEN
	            CALL FAKE_DYNAMIC_EXIT(I,1)
	         ELSE
	            CALL SEND_EXIT_MSG(I)
	         ENDIF
	         STREAM(I).REQUEST=SMBMSG$K_STOP_STREAM
	         ACTIVE_STREAMS=ACTIVE_STREAMS-1
	         CALL SETAST(%REF(1))
	         GOTO189
	      ELSE
C
C	If the stream wasn't being reset (stopped), tell the process to 
C	reset itself to its initial state, and go on.
C
	         IF(BTEST(STREAM(I).FLAGS,FLG_V_BINARY))THEN
	            CALL WRITE_MBX_BIN_X(I,0,'RESET')
	         ELSE
	            CALL WRITE_MBX_ASC_X(I,'EXEC_STEP','RESET')
	         ENDIF
	      ENDIF
	      GOTO1041
C
C	After completing as much as can be done in the present task state,
C	come here.  If the task state completed its work, advance to the
C	next state and clear the hibernation flag.  If the state is waiting 
C	for something (e.g. a status, or completion of a QIO), let the
C	hibernate flag remain as-is, and keep the same task state number.
C
188	      IF(STREAM(I).INDEX.LT.4)STREAM(I).INDEX=STREAM(I).INDEX+1
	      DO_HIBER=.FALSE.
189	      CONTINUE
C
C	PAUSE_TASK indicates that task is paused -- stream is active, but
C	there's no reason not to hibernate
C
	   ELSE IF(STREAM(I).REQUEST.EQ.SMBMSG$K_PAUSE_TASK)THEN
	      ACTIVE_STREAMS=ACTIVE_STREAMS+1
C
C	STOP_STREAM indicates that stream is inactive (or becoming inactive, 
C	if process hasn't yet exited) -- count stream as "active" if 
C	process is alive, or else we might exit before process does!
C
	   ELSE IF(STREAM(I).REQUEST.EQ.SMBMSG$K_STOP_STREAM)THEN
	      IF(STREAM(I).PID.NE.0)ACTIVE_STREAMS=ACTIVE_STREAMS+1
	   ELSE
C
C	Should never reach here -- REQUEST (stream status) value is illegal
C
	      CALL SYS$FAO(' !2UB=!8XL',,STRN,%VAL(I),%VAL(STREAM(I).REQUEST))
	      CALL LOGGIT('Invalid request for stream'//STRN,)
	   ENDIF
C
C	End of main processing loop, executed once per stream
C
	ENDDO
C
C	Assume we have to do a wakeup to get ourselves moving again; this
C	prevents race condition between hibernation code in non-AST state
C	and wakeup code in AST state.
C
	DO_WAKE=.TRUE.
C
C	If no stream had a reason to keep us from hibernating, we hibernate.
C	If there were no active streams, though, we have no reason to keep
C	running, so we must exit.  (Job controller will NOT recognize us as 
C	a symbiont if we stay active, so we have to exit when no more 
C	streams are active.)
C
	IF(DO_HIBER)THEN
	   IF(ACTIVE_STREAMS.EQ.0)THEN
	      CALL TELL_OPER('All streams now inactive; EXECSYMB '//
	1      VERSION_NUMBER//' process exiting.',3)
	      CALL SYS$EXIT(%VAL(1))
	   ENDIF
C
C	The following message will never appear unless LOGGIT is modified
C	to ignore the status value (which can be done for debugging purposes).
C
	   CALL LOGGIT('Hibernating at end of loop...',1)
C
C	Hibernate.  If an AST routine spotted the fact that the DO_WAKE flag
C	was set, and posted a SYS$WAKE already, this SYS$HIBER will do 
C	nothing more than cancel the pending wakeup.
C
	   ISTAT=SYS$HIBER()
C
C	(Note that the following message, if it appears, will be printed 
C	after wakeup!)
C
	   CALL LOGGIT('End-of-main-loop SYS$HIBER status = ',ISTAT)
	ENDIF
C
C	Once we are past the hibernate code, clear the wakeup flag (thus,
C	we cannot enter the state where the non-AST code hibernates but the
C	AST-level code thinks it doesn't need to do a wakeup.)
C
	DO_WAKE=.FALSE.
C
C	Done with one pass through all the streams, so go back through the 
C	main loop
C
	GOTO1000
C
C	If we can't create the log file, exit with ABORT status.
C
99	CALL TELL_OPER('Exiting due to create failure on log file',4)
	CALL SYS$EXIT(%VAL('2C'X))
	END
C
C	READ_EXIT_MBX -- Post a read QIO with AST on the process-exit 
C	mailbox channel
C
	SUBROUTINE READ_EXIT_MBX
	IMPLICIT INTEGER*4 (A-Z)
	INCLUDE 'EXECSYMB.INC'
	INCLUDE '($IODEF)'
	EXTERNAL EXIT_AST
	INTEGER*4 SYS$QIO
	ISTAT=SYS$QIO(%VAL(31),%VAL(EXIT_MBX_CHAN),
	1 %VAL(IO$_READVBLK),,%REF(EXIT_AST),,
	2 %REF(EXITBUF),%VAL(12),,,,)
	CALL LOGGIT('READ_EXIT_MBX $QIO status = ',ISTAT)
	RETURN
	END
C
C	READ_STAT_MBX -- post a read QIO with AST on the task status-return 
C	mailbox channel
C
	SUBROUTINE READ_STAT_MBX
	IMPLICIT INTEGER*4 (A-Z)
	INCLUDE 'EXECSYMB.INC'
	INCLUDE '($IODEF)'
	EXTERNAL IO_DONE_AST
	INTEGER*4 SYS$QIO
	ISTAT=SYS$QIO(%VAL(31),%VAL(STAT_W_MBX_CHAN),
	1 %VAL(IO$_READVBLK),%REF(STATIOSB),%REF(IO_DONE_AST),,
	2 %REF(STATBUF),%VAL(12),,,,)
	CALL LOGGIT('READ_STAT_MBX $QIO status = ',ISTAT)
	RETURN
	END
C
C	MSG_AST -- routine called by the shareable symbiont code when a new
C	request message is received from the job controller.  This routine 
C	does all of the initialization in setting up a new request.
C
	SUBROUTINE MSG_AST
C
C	First, make implicit declaration and specify INCLUDE files
C
	IMPLICIT INTEGER*4 (A-Z)
	INCLUDE 'EXECSYMB.INC'
	INCLUDE '($SJCDEF)'
	INCLUDE '($PRCDEF)'
	INCLUDE 'ITEMDEF.INC'
C
C	System services called as functions
C
	INTEGER*4 SYS$CREPRC,SYS$CREMBX,SYS$WAKE,SYS$CANTIM
	INTEGER*4 SYS$BINTIM,SYS$ASCTIM,SYS$QIO,SYS$SNDJBCW
C
C	Process creation flags for processors
C
	INTEGER*4 PRC_FLAGS
	PARAMETER (PRC_FLAGS=PRC$M_NOUAF+PRC$M_DETACH+PRC$M_HIBER)
C
C	Item list for setting JOB_RESET_MODULES
C
	RECORD /ITEM/ SJCLIST(3)
C
C	Other routines called as functions
C
	INTEGER*4 SMB$READ_MESSAGE,DO_ENTER
C
C	Local storage for holding current message, and item position/size
C	arrays for messages that aren't inserted into stream SCB
C
	CHARACTER*1024 MESSAGE
	INTEGER*2 POS(SMBMSG$K_MAX_ITEM_CODE-1),SIZE(SMBMSG$K_MAX_ITEM_CODE-1)
C
C	Miscellaneous local storage
C
	INTEGER*4 ISTAT,STATUS_VECTOR(2)
	INTEGER*4 NSTREAM,NEWREQUEST
	CHARACTER*256 NEW_JOB_RESET_MODULES
	INTEGER*2 LEN_NJRM
	INTEGER*2 DID(3)
	BYTE BTEMP
	BYTE BYTE
	INTEGER*2 WORD
	INTEGER*4 LONG
	LOGICAL*1 REALLY_START_TASK
C
C	Process name (need last character only)
C
	CHARACTER*14 FILLER
	CHARACTER*1 SYMB_INDEX
	COMMON /PRCNAM/ FILLER,SYMB_INDEX
	CHARACTER*15 PROCESSOR_NAME
	INTEGER*2 LPN
C
C	Data declarations for local variables
C
	DATA STATUS_VECTOR/1,1/
C
C	Start of MSG_AST code
C
C	Call process wakeup routine first -- getting a new message is reason
C	to wake up non-AST code
C
	CALL WAKEUP
C
C	Read the message from the job controller via shareable symbiont code
C
	ISTAT=SMB$READ_MESSAGE(%REF(NSTREAM),%DESCR(MESSAGE),%REF(NEWREQUEST))
	CALL LOGGIT('SMB$READ_MESSAGE status = ',ISTAT)
C
C	If this is a dynamic processor and the processor exists, cancel its
C	timer request.
C
	IF(BTEST(STREAM(NSTREAM).FLAGS,FLG_V_DYNAMIC).AND..NOT.
	1 BTEST(STREAM(NSTREAM).FLAGS,FLG_V_NO_PROCESSOR))THEN
	   ISTAT=SYS$CANTIM(%VAL(NSTREAM+1),)
	   CALL LOGGIT('SYS$CANTIM status = ',ISTAT)
	ENDIF
C
C	If START_TASK request, parse message items into stream SCB's item 
C	position and size arrays, and initialize directory fileid, device
C	name length, and task abort flag
C
	IF(NEWREQUEST.EQ.SMBMSG$K_START_TASK)THEN
	   CALL PARSEMSG(MESSAGE,STREAM(NSTREAM).POS,STREAM(NSTREAM).SIZE,L)
	   DID(1)=0
	   DID(2)=0
	   DID(3)=0
	   STREAM(NSTREAM).DEVICE_NAME_LEN=0
	   STREAM(NSTREAM).FLAGS=STREAM(NSTREAM).FLAGS.AND..NOT.FLG_M_ABORT
	ELSE
C
C	For all other requests, parse items into local position and size 
C	arrays
C
	   CALL PARSEMSG(MESSAGE,POS,SIZE,L)
	ENDIF
C
C	If request was RESET_STREAM, determine current stream state and 
C	handle accordingly
C
	IF(NEWREQUEST.EQ.SMBMSG$K_RESET_STREAM)THEN
C
C	If stream being reset is idle (START_STREAM) or stopped 
C	(STOP_STREAM) then change request to a simple STOP_STREAM
C
	   IF(STREAM(NSTREAM).REQUEST.EQ.SMBMSG$K_START_STREAM.OR.
	1   STREAM(NSTREAM).REQUEST.EQ.SMBMSG$K_STOP_STREAM)THEN
	      NEWREQUEST=SMBMSG$K_STOP_STREAM
	   ELSE
C
C	Otherwise (task active, stream in START_TASK or PAUSE_TASK or
C	STOP_TASK state), set stream task state index to 5 (abort task)
C
	      STREAM(NSTREAM).INDEX=5
	   ENDIF
	ENDIF
C
C	If request is START_STREAM, process stream initialization information
C	specified in various items
C
	IF(NEWREQUEST.EQ.SMBMSG$K_START_STREAM)THEN
C
C	Get EXECUTOR_QUEUE item and use it to determine OURNAME (which gives
C	process its name)
C
	   IX=SMBMSG$K_EXECUTOR_QUEUE
	   L=SIZE(IX)
	   STREAM(NSTREAM).OURNAME(1:L)=MESSAGE(POS(IX):POS(IX)+L-1)
	   STREAM(NSTREAM).OURNAME_LEN=L
C
C	Get LIBRARY_SPECIFICATION item, strip off "SYS$LIBRARY:" that was 
C	put on its front, and ".TLB" that was put on its end, and use the
C	resulting string as the name of the stream's command procedure file.
C
	   IX=SMBMSG$K_LIBRARY_SPECIFICATION
	   L=SIZE(IX)-16
	   STREAM(NSTREAM).COM_FILE_LEN=L
	   STREAM(NSTREAM).COM_FILE(1:L)=MESSAGE(POS(IX)+12:POS(IX)+L+11)
C
C	Get JOB_RESET_MODULES item.  (This is a character string that is not
C	interpreted by the job controller.)  Use the item as a means for
C	passing stream parameters.
C
C	Values recognized in this string (separated by commas):
C		TIME="d hh:mm:ss.cc"	(requeue time)
C		ITEMS="itemlist"	(list of NN or MM:NN item numbers)
C		SPOOL=[spooldir]	(directory for entering spool files)
C		ASCII or BINary		(how values are passed)
C		PRINTer or SERVER	(type of queue)
C		NONULl or NULL		(whether null items are passed)
C		CHECKpoint or NOCHECKpoint
C					(whether jobs restart from beginning)
C		COPY={ALL, First, or Last}
C					(which copy of job/file to process)
C		USER=username		(username under which to run process)
C		DYN="d hh:mm:ss.cc"	(timeout for dynamic start/stop)
C		PERM=logical-name-suffix
C					(permanent processor)
C
C	Defaults:	no requeue, no ITEMS, no SPOOL directory,
C			ASCII, SERVER, NULL, CHECKpoint, COPY=ALL, 
C			USER=SYSTEM, no permanent processor
C
	   IP=POS(SMBMSG$K_JOB_RESET_MODULES)
	   L=SIZE(SMBMSG$K_JOB_RESET_MODULES)
C
C	First, initialize all values to defaults
C
	   LEN_NJRM=0
	   STREAM(NSTREAM).FLAGS=FLG_M_NULL+FLG_M_CHECKPOINT+FLG_M_COPYALL
	   STREAM(NSTREAM).SPOOLDIR_LEN=0
	   STREAM(NSTREAM).USERNAME='SYSTEM'
C
C	Parse for requeue time parameter
C
	   IX=INDEX(MESSAGE(IP:IP+L-1),'TIME=')
C
C	From there to next comma (or end of string) is requeue time
C
	   IF(IX.NE.0)THEN
	      IY=INDEX(MESSAGE(IP+IX:IP+L-1),',')+IX
	      IF(IY.EQ.IX)IY=L+1
	      ISTAT=SYS$BINTIM(MESSAGE(IP+IX+4:IP+IY-2),
	1      STREAM(NSTREAM).REQUEUE_TIME(1))
	      CALL LOGGIT('$BINTIM status = ',ISTAT)
C
C	If $BINTIM converts requeue time successfully, indicate that stream
C	is to requeue aborted jobs
C
	      IF((ISTAT.AND.1).NE.0)STREAM(NSTREAM).FLAGS=
	1      STREAM(NSTREAM).FLAGS.OR.FLG_M_DO_REQUEUE
C
C	Blank out part of string that contained requeue time parameter,
C	and add edited requeue time to actual parameters string.
C
	      MESSAGE(IP+IX-1:IP+IY-2)=' '
	      NEW_JOB_RESET_MODULES(LEN_NJRM+1:LEN_NJRM+6)='TIME='
	      LEN_NJRM=LEN_NJRM+6
	      ISTAT=SYS$ASCTIM(IY,NEW_JOB_RESET_MODULES(LEN_NJRM:LEN_NJRM+15),
	1      STREAM(NSTREAM).REQUEUE_TIME(1),)
	      CALL LOGGIT('$ASCTIM status = ',ISTAT)
C
C	Adjust converted requeue time to eliminate extra spaces, zeroes
C
	      CALL FIXTIME(NEW_JOB_RESET_MODULES(LEN_NJRM:LEN_NJRM+15),IY)
	      LEN_NJRM=LEN_NJRM+IY-1
	   ENDIF
C
C	Parse for dynamic timeout parameter
C
	   IX=INDEX(MESSAGE(IP:IP+L-1),'DYN=')
C
C	From there to next comma (or end of string) is dynamic timeout time
C                                            
	   IF(IX.NE.0)THEN
	      IY=INDEX(MESSAGE(IP+IX:IP+L-1),',')+IX
	      IF(IY.EQ.IX)IY=L+1
	      ISTAT=SYS$BINTIM(MESSAGE(IP+IX+3:IP+IY-2),
	1      STREAM(NSTREAM).DYNAMIC_TIME(1))
	      CALL LOGGIT('$BINTIM status = ',ISTAT)
C
C	If $BINTIM converts time successfully, indicate that stream processor 
C	is to be started and stopped dynamically, independent of queue.
C
	      IF((ISTAT.AND.1).NE.0)STREAM(NSTREAM).FLAGS=
	1      STREAM(NSTREAM).FLAGS.OR.FLG_M_DYNAMIC
C
C	Blank out part of string that contained time parameter,
C	and add edited time to actual parameters string.
C
	      MESSAGE(IP+IX-1:IP+IY-2)=' '
	      IF(LEN_NJRM.NE.0)THEN
	         LEN_NJRM=LEN_NJRM+1
	         NEW_JOB_RESET_MODULES(LEN_NJRM:LEN_NJRM)=','
	      ENDIF
	      NEW_JOB_RESET_MODULES(LEN_NJRM+1:LEN_NJRM+5)='DYN='
	      LEN_NJRM=LEN_NJRM+5
	      ISTAT=SYS$ASCTIM(IY,NEW_JOB_RESET_MODULES(LEN_NJRM:LEN_NJRM+15),
	1      STREAM(NSTREAM).DYNAMIC_TIME(1),)
	      CALL LOGGIT('$ASCTIM status = ',ISTAT)
C
C	Adjust converted time to eliminate extra spaces, zeroes
C
	      CALL FIXTIME(NEW_JOB_RESET_MODULES(LEN_NJRM:LEN_NJRM+15),IY)
	      LEN_NJRM=LEN_NJRM+IY-1
	   ENDIF
C
C	Look for list of item codes to be passed to subroutine, and use
C	PARSE_ITEMS routine to set up the ORDER array with specified codes
C
	   IX=INDEX(MESSAGE(IP:IP+L-1),'ITEMS=')
	   CALL PARSE_ITEMS(MESSAGE(IP+IX+5:IP+L-1),STREAM(NSTREAM).ORDER)
C
C	Then, edit ORDER array into new JOB_RESET_MODULES (stream parameters)
C	string
C
	   CALL EDIT_NJRM(STREAM(NSTREAM).ORDER,NEW_JOB_RESET_MODULES,LEN_NJRM)
C
C	Look for spool directory name
C
	   IX=INDEX(MESSAGE(IP:IP+L-1),'SPOOL=[')
C
C	Find terminator for name, and add entry to new stream parameters string
C
	   IF(IX.NE.0)THEN
	      IY=INDEX(MESSAGE(IP+IX:IP+L-1),']')+IX
	      IF(IY.EQ.IX)GOTO1118
	      IL=IY-IX-5
	      IF(LEN_NJRM.NE.0)THEN
	         LEN_NJRM=LEN_NJRM+1
	         NEW_JOB_RESET_MODULES(LEN_NJRM:LEN_NJRM)=','
	      ENDIF
	      NEW_JOB_RESET_MODULES(LEN_NJRM+1:LEN_NJRM+IL+6)=
	1      MESSAGE(IP+IX-1:IP+IY-1)
	      LEN_NJRM=LEN_NJRM+IL+6
C
C	Turn spool directory name into name of file that represents that
C	directory, so we can get its fileid.
C
	      STREAM(NSTREAM).SPOOLDIR_LEN=IL
	      STREAM(NSTREAM).SPOOLDIR=MESSAGE(IP+IX+5:IP+IY-1)
	      MESSAGE(IP+IX-1:IP+IY-1)=' '
	      IX=INDEX(STREAM(NSTREAM).SPOOLDIR(1:IL),'.')
C
C	If name contains no subdirectory, change it to a file in [000000].
C	e.g., change [SPOOL] into [000000]SPOOL.DIR;0
C
	      IF(IX.EQ.0)THEN
	         STREAM(NSTREAM).SPOOLDIR(2:IL+12)='000000]'//
	1         STREAM(NSTREAM).SPOOLDIR(2:IL-1)//'.DIR;0'
	         STREAM(NSTREAM).SPOOLDIR_LEN=IL+12
	      ELSE
C
C	If name contains a subdirectory, change it to a file in the next
C	higher directory.
C	e.g., change [SPOOL.FOO.BAR] into [SPOOL.FOO]BAR.DIR;0
C
	         DO IY=IL-2,IX+1,-1
	            IF(STREAM(NSTREAM).SPOOLDIR(IY:IY).EQ.'.')GOTO1117
	         ENDDO
	         IY=IX
1117	         STREAM(NSTREAM).SPOOLDIR(IY:IY)=']'
	         STREAM(NSTREAM).SPOOLDIR(IL:IL+5)='.DIR;0'
	         STREAM(NSTREAM).SPOOLDIR_LEN=IL+5
	      ENDIF
	   ENDIF
C
C	Check for USER=username
C
1118	   IX=INDEX(MESSAGE(IP:IP+L-1),'USER=')
C
C	Find terminator for name, and add entry to new stream parameters string
C
	   IF(IX.NE.0)THEN
	      IY=INDEX(MESSAGE(IP+IX:IP+L-1),',')+IX-1
	      IF(IY.EQ.IX-1)IY=L
	      STREAM(NSTREAM).USERNAME=MESSAGE(IP+IX+4:IP+IY-1)
	      IF(STREAM(NSTREAM).USERNAME.NE.'SYSTEM')THEN
	         IF(LEN_NJRM.GT.0)THEN
	            LEN_NJRM=LEN_NJRM+1
	            NEW_JOB_RESET_MODULES(LEN_NJRM:LEN_NJRM)=','
	         ENDIF
	         NEW_JOB_RESET_MODULES(LEN_NJRM+1:LEN_NJRM+5)='USER='
	         NEW_JOB_RESET_MODULES(LEN_NJRM+6:LEN_NJRM+IY-IX+1)=
	1         MESSAGE(IP+IX+4:IP+IY-1)
	         LEN_NJRM=LEN_NJRM+IY-IX+1
	      ENDIF
	      MESSAGE(IP+IX-1:IP+IY-1)=' '
	   ENDIF
C
C	Check for PERM=logical-name-suffix
C
	   IX=INDEX(MESSAGE(IP:IP+L-1),'PERM=')
C
C	Find terminator for name, and add entry to new stream parameters string
C
	   IF(IX.NE.0)THEN
	      IY=INDEX(MESSAGE(IP+IX:IP+L-1),',')+IX-1
	      IF(IY.EQ.IX-1)IY=L
	      STREAM(NSTREAM).PERMNAME_LEN=MIN(20,IY-IX-4)
	      STREAM(NSTREAM).PERMNAME(1:STREAM(NSTREAM).PERMNAME_LEN)=
	1      MESSAGE(IP+IX+4:IP+IY-1)
	      STREAM(NSTREAM).FLAGS=STREAM(NSTREAM).FLAGS.OR.FLG_M_PERMANENT
	      IF(LEN_NJRM.GT.0)THEN
	         LEN_NJRM=LEN_NJRM+1
	         NEW_JOB_RESET_MODULES(LEN_NJRM:LEN_NJRM)=','
	      ENDIF
	      NEW_JOB_RESET_MODULES(LEN_NJRM+1:LEN_NJRM+5)='PERM='
	      NEW_JOB_RESET_MODULES(LEN_NJRM+6:LEN_NJRM+IY-IX+1)=
	1      MESSAGE(IP+IX+4:IP+IY-1)
	      LEN_NJRM=LEN_NJRM+IY-IX+1
	      MESSAGE(IP+IX-1:IP+IY-1)=' '
	   ENDIF
C
C	Check for BINARY flag (if not present, assume ASCII)
C
1119	   IX=INDEX(MESSAGE(IP:IP+L-1),'BIN')
	   IF(IX.NE.0)THEN
	      STREAM(NSTREAM).FLAGS=STREAM(NSTREAM).FLAGS.OR.FLG_M_BINARY
	      IF(LEN_NJRM.NE.0)THEN
	         LEN_NJRM=LEN_NJRM+1
	         NEW_JOB_RESET_MODULES(LEN_NJRM:LEN_NJRM)=','
	      ENDIF
	      NEW_JOB_RESET_MODULES(LEN_NJRM+1:LEN_NJRM+3)='BIN'
	      LEN_NJRM=LEN_NJRM+3
	   ENDIF
C
C	Check for PRINTER flag (if not present, assume SERVER)
C
	   IX=INDEX(MESSAGE(IP:IP+L-1),'PRINT')
	   IF(IX.NE.0)THEN
	      STREAM(NSTREAM).FLAGS=STREAM(NSTREAM).FLAGS.OR.FLG_M_PRINTQUEUE
	      IF(LEN_NJRM.NE.0)THEN
	         LEN_NJRM=LEN_NJRM+1
	         NEW_JOB_RESET_MODULES(LEN_NJRM:LEN_NJRM)=','
	      ENDIF
	      NEW_JOB_RESET_MODULES(LEN_NJRM+1:LEN_NJRM+5)='PRINT'
	      LEN_NJRM=LEN_NJRM+5
	   ENDIF
C
C	Check for NONULL flag (if not present, assume NULL)
C
	   IX=INDEX(MESSAGE(IP:IP+L-1),'NONUL')
	   IF(IX.NE.0)THEN
	      STREAM(NSTREAM).FLAGS=STREAM(NSTREAM).FLAGS.AND..NOT.FLG_M_NULL
	      IF(LEN_NJRM.NE.0)THEN
	         LEN_NJRM=LEN_NJRM+1
	         NEW_JOB_RESET_MODULES(LEN_NJRM:LEN_NJRM)=','
	      ENDIF
	      NEW_JOB_RESET_MODULES(LEN_NJRM+1:LEN_NJRM+5)='NONUL'
	      LEN_NJRM=LEN_NJRM+5
	   ENDIF
C
C	Check for NOCHECKpoint flag (if not present, assume CHECKpoint)
C
	   IX=INDEX(MESSAGE(IP:IP+L-1),'NOCHECK')
	   IF(IX.NE.0)THEN
	      STREAM(NSTREAM).FLAGS=STREAM(NSTREAM).FLAGS.AND.
	1      .NOT.FLG_M_CHECKPOINT
	      IF(LEN_NJRM.NE.0)THEN
	         LEN_NJRM=LEN_NJRM+1
	         NEW_JOB_RESET_MODULES(LEN_NJRM:LEN_NJRM)=','
	      ENDIF
	      NEW_JOB_RESET_MODULES(LEN_NJRM+1:LEN_NJRM+7)='NOCHECK'
	      LEN_NJRM=LEN_NJRM+7
	   ENDIF
C
C	Check COPY parameter
C
	   IX=INDEX(MESSAGE(IP:IP+L-1),'COPY=F')
C
C	If COPY=FIRST, indicate stream doesn't want all copies, and wants
C	first copy of each job or file
C
	   IF(IX.NE.0)THEN
	      STREAM(NSTREAM).FLAGS=STREAM(NSTREAM).FLAGS.AND.
	1      .NOT.FLG_M_COPYALL
	      STREAM(NSTREAM).FLAGS=STREAM(NSTREAM).FLAGS.OR.FLG_M_COPYFIRST
	      IF(LEN_NJRM.NE.0)THEN
	         LEN_NJRM=LEN_NJRM+1
	         NEW_JOB_RESET_MODULES(LEN_NJRM:LEN_NJRM)=','
	      ENDIF
	      NEW_JOB_RESET_MODULES(LEN_NJRM+1:LEN_NJRM+6)='COPY=F'
	      LEN_NJRM=LEN_NJRM+6
	   ENDIF
	   IX=INDEX(MESSAGE(IP:IP+L-1),'COPY=L')
C
C	If COPY=LAST specified, indicate stream doesn't want all copies, and
C	doesn't want first copy.  (If neither COPY=FIRST or COPY=LAST 
C	specified, COPY=ALL is assumed.)
C
	   IF(IX.NE.0)THEN
	      STREAM(NSTREAM).FLAGS=STREAM(NSTREAM).FLAGS.AND.
	1      .NOT.FLG_M_COPYALL
	      IF(LEN_NJRM.NE.0)THEN
	         LEN_NJRM=LEN_NJRM+1
	         NEW_JOB_RESET_MODULES(LEN_NJRM:LEN_NJRM)=','
	      ENDIF
	      NEW_JOB_RESET_MODULES(LEN_NJRM+1:LEN_NJRM+6)='COPY=L'
	      LEN_NJRM=LEN_NJRM+6
	   ENDIF
C
C	Set up itemlist for modifying queue's JOB_RESET_MODULES to reflect
C	non-default parameters that were actually set
C
	   SJCLIST(1).BUFLEN=STREAM(NSTREAM).OURNAME_LEN
	   SJCLIST(1).ITEMCODE=SJC$_QUEUE
	   SJCLIST(1).BUFADR=%LOC(STREAM(NSTREAM).OURNAME)
	   SJCLIST(1).RETLENADR=0
	   SJCLIST(2).BUFLEN=LEN_NJRM
	   SJCLIST(2).ITEMCODE=SJC$_JOB_RESET_MODULES
	   SJCLIST(2).BUFADR=%LOC(NEW_JOB_RESET_MODULES)
	   SJCLIST(2).RETLENADR=0
	   SJCLIST(3).TERMINATOR=0
	   ISTAT=SYS$SNDJBCW(,%VAL(SJC$_ALTER_QUEUE),,SJCLIST,,,)
	   CALL LOGGIT('MSG_AST $SNDJBCW status = ',ISTAT)
C
C	If request was START_TASK, determine whether we really want to start
C	a task (based upon which copy of the file and job we are being 
C	given, and which copy -- FIRST, LAST, or ALL -- the stream requires)
C
	ELSE IF(NEWREQUEST.EQ.SMBMSG$K_START_TASK)THEN
	   REALLY_START_TASK=.TRUE.
C
C	If all copies are passed to stream, we must really start the task
C
	   IF(BTEST(STREAM(NSTREAM).FLAGS,FLG_V_COPYALL))THEN
	      CONTINUE
C
C	If the first copy is passed to the stream, check the job count and
C	file count parameters; both must be one in order to start this task
C
	   ELSE IF(BTEST(STREAM(NSTREAM).FLAGS,FLG_V_COPYFIRST))THEN
	      IP=STREAM(NSTREAM).POS(SMBMSG$K_JOB_COUNT)
	      JC1=LONG(MESSAGE(IP:IP+3))
	      IF(JC1.NE.1)THEN
	         REALLY_START_TASK=.FALSE.
	      ELSE
	         IP=STREAM(NSTREAM).POS(SMBMSG$K_FILE_COUNT)
	         JC2=LONG(MESSAGE(IP:IP+3))
	         IF(JC2.NE.1)REALLY_START_TASK=.FALSE.
	      ENDIF
C
C	If the last copy is passed to the stream, get the JOB_COPIES and 
C	JOB_COUNT items.  They must be equal, or the task is not really
C	started.
C
	   ELSE
	      IP=STREAM(NSTREAM).POS(SMBMSG$K_JOB_COPIES)
	      JC1=LONG(MESSAGE(IP:IP+3))
	      IP=STREAM(NSTREAM).POS(SMBMSG$K_JOB_COUNT)
	      JC2=LONG(MESSAGE(IP:IP+3))
	      IF(JC1.NE.JC2)THEN
	         REALLY_START_TASK=.FALSE.
C
C	If last copy of the job, check FILE_COPIES against FILE_COUNT.
C	They must also be equal in order to really start the task.
C
	      ELSE
	         IP=STREAM(NSTREAM).POS(SMBMSG$K_FILE_COPIES)
	         JC1=LONG(MESSAGE(IP:IP+3))
	         IP=STREAM(NSTREAM).POS(SMBMSG$K_FILE_COUNT)
	         JC2=LONG(MESSAGE(IP:IP+3))
	         IF(JC1.NE.JC2)REALLY_START_TASK=.FALSE.
	      ENDIF
	   ENDIF
C
C	If we are really starting the task, copy the symbiont message
C	to the stream's SCB, and perform task initialization code.
C
	   IF(REALLY_START_TASK)THEN
	      STREAM(NSTREAM).MESSAGE(1:L)=MESSAGE(1:L)
C
C	Determine which queue (our own or a generic queue) originated this 
C	job, and save the name in case we need to manipulate the entry.
C
	      L=STREAM(NSTREAM).SIZE(SMBMSG$K_QUEUE)
	      IP=STREAM(NSTREAM).POS(SMBMSG$K_QUEUE)
	      STREAM(NSTREAM).QUEUENAME_LEN=L
	      STREAM(NSTREAM).QUEUENAME(1:L)=MESSAGE(IP:IP+L-1)
C
C	Determine the entry number in case we need it.
C
	      IP=STREAM(NSTREAM).POS(SMBMSG$K_ENTRY_NUMBER)
C
C	Look at the identification of the file being processed.  Strip off the
C	device name, and get the fileid and directory id values.
C
	      STREAM(NSTREAM).ENTRY=LONG(MESSAGE(IP:IP+3))
	      IP=STREAM(NSTREAM).POS(SMBMSG$K_FILE_IDENTIFICATION)
	      STREAM(NSTREAM).DEVICE_NAME_LEN=BYTE(MESSAGE(IP:IP))
	      STREAM(NSTREAM).DEVICE_NAME=MESSAGE(IP+1:IP+15)
	      STREAM(NSTREAM).FID(1)=WORD(MESSAGE(IP+16:IP+17))
	      STREAM(NSTREAM).FID(2)=WORD(MESSAGE(IP+18:IP+19))
	      STREAM(NSTREAM).FID(3)=WORD(MESSAGE(IP+20:IP+21))
	      DID(1)=WORD(MESSAGE(IP+22:IP+23))
	      DID(2)=WORD(MESSAGE(IP+24:IP+25))
	      DID(3)=WORD(MESSAGE(IP+26:IP+27))
C
C	Get the filespec (at least in its original form)
C
	      IP=STREAM(NSTREAM).POS(SMBMSG$K_FILE_SPECIFICATION)
	      L=STREAM(NSTREAM).SIZE(SMBMSG$K_FILE_SPECIFICATION)
	      STREAM(NSTREAM).FILESPEC_LEN=L
	      STREAM(NSTREAM).FILESPEC(1:L)=MESSAGE(IP:IP+L-1)
	   ENDIF
C
C	If request was STOP_TASK, pick up the stop condition as the stream
C	status (or use %X0000002C = "abort")
C
	ELSE IF(NEWREQUEST.EQ.SMBMSG$K_STOP_TASK)THEN
	   IP=STREAM(NSTREAM).POS(SMBMSG$K_STOP_CONDITION)
	   STREAM(NSTREAM).STATUS=LONG(MESSAGE(IP:IP+3))
	   IF(STREAM(NSTREAM).STATUS.EQ.0)STREAM(NSTREAM).STATUS='2C'X
	ENDIF
C
C	Save the current stream status ("request" value) and tentatively
C	set up the new request.
C
	ITEMP=STREAM(NSTREAM).REQUEST
	STREAM(NSTREAM).REQUEST=NEWREQUEST
C
C	If it's START_STREAM, create the command mailbox for the process,
C	then create the process.  The command mailbox has a 260-byte 
C	limit on record size (4-byte item code + 256-byte item, for BINARY
C	streams) and has the logical name CMD_MBX_pid to enable the
C	process to find it.
C
C	Effective with V2.16, the process name is no longer based entirely 
C	upon the queue name, but rather is constructed from the symbiont name's
C	last character and the stream number, as well as the queue name.
C	Also, the logical names for the mailboxes are now based upon the PID 
C	of the queue-processor process, since the queue name may not be unique
C	in fifteen characters.  To implement the latter, the process is
C	created in hibernate state, and is awakened after the mailbox has
C	been set up.
C
	IF(NEWREQUEST.EQ.SMBMSG$K_START_STREAM)THEN
	   IF(BTEST(STREAM(NSTREAM).FLAGS,FLG_V_PERMANENT))THEN
	      CALL TELL_OPER('Queue '//
	1      STREAM(NSTREAM).OURNAME(1:STREAM(NSTREAM).OURNAME_LEN)//
	2      'starting with permanent processor ID '//
	3      STREAM(NSTREAM).PERMNAME(1:STREAM(NSTREAM).PERMNAME_LEN)//
	4      '.',3)
	      CALL START_PROCESSOR(NSTREAM,ISTAT_CREPRC)
	   ELSE IF(BTEST(STREAM(NSTREAM).FLAGS,FLG_V_DYNAMIC))THEN
	      CALL TELL_OPER('Queue '//
	1      STREAM(NSTREAM).OURNAME(1:STREAM(NSTREAM).OURNAME_LEN)//
	2      ' starting; processor will start dynamically.',3)
	      STREAM(NSTREAM).FLAGS=STREAM(NSTREAM).FLAGS.OR.FLG_M_NO_PROCESSOR
	      ISTAT_CREPRC=1
	   ELSE
	      CALL START_PROCESSOR(NSTREAM,ISTAT_CREPRC)
	   ENDIF
C
C	If it's START_TASK, determine whether we are really starting the 
C	task (i.e. if it's the copy of the job and file that the stream wants 
C	to process)
C
	ELSE IF(NEWREQUEST.EQ.SMBMSG$K_START_TASK)THEN
	   IF(REALLY_START_TASK)THEN
C
C	If it's the right copy, set up the stream to process the task:
C
C	First, determine whether the file is a spool file.  Spool files 
C	(files created by the job controller from data copied directly to a 
C	spooled device) have no directory entry.  Most applications of this
C	symbiont require a valid filespec for each file to be processed; VMS
C	expects spool files to be opened by fileid and not by name.  We get
C	around this problem, if necessary, by entering the file into the
C	"spool directory" specified for this stream.  A "spool directory" of
C	the same name must exist on each disk used as an intermediate device
C	for spooled devices processed by this stream.
C
	      STREAM(NSTREAM).FLAGS=STREAM(NSTREAM).FLAGS.AND.
	1      .NOT.FLG_M_ENTERED
C
C	If the file is a spool file, its directory's fileid will be zero.
C	If the device name is also zero, it's an even more special case:
C	it is a nonexistent file that resulted from failing to create a 
C	batch log file.  The latter case is determined from the device name
C	length, and if it is that special case, the filespec is modified to
C	include the null device (NLA0:); the process should be able to 
C	open or copy the "file" by name, getting an empty file as the result.
C
	      IF(DID(1).EQ.0.AND.DID(2).EQ.0.AND.DID(3).EQ.0)THEN
	         IF(STREAM(NSTREAM).DEVICE_NAME_LEN.EQ.0)THEN
	            IL=STREAM(NSTREAM).FILESPEC_LEN
	            IX=INDEX(STREAM(NSTREAM).FILESPEC(1:IL),':')
	            IY=IL+5-IX
	            STREAM(NSTREAM).FILESPEC(1:IY)='NLA0:'//
	1            STREAM(NSTREAM).FILESPEC(IX+1:IL)
	            STREAM(NSTREAM).FILESPEC_LEN=IY
C
C	If the filespec is modified, it is changed in the SCB field called
C	FILE_SPECIFICATION, and not in the message itself.  This SCB field
C	follows the field containing the message; by setting the position
C	to one greater than the size of the message buffer, normal processing
C	will pick up the modified name.
C
	            STREAM(NSTREAM).POS(SMBMSG$K_FILE_SPECIFICATION)=1025
	            STREAM(NSTREAM).SIZE(SMBMSG$K_FILE_SPECIFICATION)=IY
	            GOTO215
	         ENDIF
C
C	If there is no directory, but there is a device, the file is a spool
C	file.  Enter it in the spool directory, but give it the next available
C	version number, in case a file with the same name and version number
C	already exists in the spool directory.
C
	         STREAM(NSTREAM).FLAGS=STREAM(NSTREAM).FLAGS.OR.FLG_M_ENTERED
	         IX=INDEX(STREAM(NSTREAM).FILESPEC(1:STREAM(NSTREAM).
	1         FILESPEC_LEN),']')
	         IY=INDEX(STREAM(NSTREAM).FILESPEC(1:STREAM(NSTREAM).
	1         FILESPEC_LEN),';')
	         ISTAT=DO_ENTER(
	1         STREAM(NSTREAM).DEVICE_NAME(1:STREAM(NSTREAM).
	2         DEVICE_NAME_LEN)//':'//STREAM(NSTREAM).SPOOLDIR(1:
	3         STREAM(NSTREAM).SPOOLDIR_LEN),STREAM(NSTREAM).FID,
	4         STREAM(NSTREAM).FILESPEC(IX+1:IY-1)//';0')
	         CALL LOGGIT('DO_ENTER status = ',ISTAT)
C
C	Convert the fileid to a filespec.  This routine (LIB$FID_TO_NAME) was
C	stolen from some other software (compressed raster symbiont???) 
C	because it works... it does have one peculiarity, though; it returns
C	a blank-padded filespec, which we have to trim to get the length.
C
	         CALL LIB$FID_TO_NAME(
	1         STREAM(NSTREAM).DEVICE_NAME(1:STREAM(NSTREAM).
	2         DEVICE_NAME_LEN),STREAM(NSTREAM).FID,STREAM(NSTREAM).
	3         FILESPEC,,,)
	         CALL STR$TRIM(STREAM(NSTREAM).FILESPEC,STREAM(NSTREAM).
	1         FILESPEC,STREAM(NSTREAM).FILESPEC_LEN)
C
C	Again, we point the FILE_SPECIFICATION item to the revised filespec.
C
	         STREAM(NSTREAM).POS(SMBMSG$K_FILE_SPECIFICATION)=1025
	         STREAM(NSTREAM).SIZE(SMBMSG$K_FILE_SPECIFICATION)=
	1         STREAM(NSTREAM).FILESPEC_LEN
215	         CONTINUE
	      ENDIF
C
C	Finally, to start the task up right, we initialize the item number 
C	index and set the task status index to the initial state.
C
	      STREAM(NSTREAM).ITEMNO=0
	      STREAM(NSTREAM).INDEX=1
C
C	If we didn't really want to start the task (i.e. it was the wrong
C	copy of the job or file), we do nothing here.
C
	   ENDIF
C
C	PAUSE_TASK requires no processing, as it simply changes the stream
C	state to prevent the task from doing any further processing (once
C	the main loop gets back to checking the stream's state)
C
	ELSE IF(NEWREQUEST.EQ.SMBMSG$K_PAUSE_TASK)THEN
	   CONTINUE
C
C	Similarly, RESUME_TASK requires no processing, other than to restore
C	the state of the stream to START_TASK, which means "task in progress"
C
	ELSE IF(NEWREQUEST.EQ.SMBMSG$K_RESUME_TASK)THEN
	   STREAM(NSTREAM).REQUEST=SMBMSG$K_START_TASK
C
C	STOP_TASK requires that the task be processable, so the stream's state
C	must be set to START_TASK (clearing any previous PAUSE_TASK request),
C	and the task state index is set to the abort-task value.
C
	ELSE IF(NEWREQUEST.EQ.SMBMSG$K_STOP_TASK)THEN
	   STREAM(NSTREAM).REQUEST=SMBMSG$K_START_TASK
	   STREAM(NSTREAM).INDEX=5
C
C	A RESET_STREAM request is not processed here.  (If it got this far,
C	it means that an active task on the stream needs to be aborted.
C	The main loop at non-AST level will take care of converting this
C	request to a STOP_STREAM once that is done.)
C
	ELSE IF(NEWREQUEST.EQ.SMBMSG$K_RESET_STREAM)THEN
	   CONTINUE
C
C	STOP_STREAM requires that the process be asked to exit.  The
C	stream is not considered completely stopped until the process 
C	exits (so that if it's the last active stream, we won't accidentally
C	abort the process prematurely by exiting ourselves.)
C
	ELSE IF(NEWREQUEST.EQ.SMBMSG$K_STOP_STREAM)THEN
	   IF(BTEST(STREAM(NSTREAM).FLAGS,FLG_V_DYNAMIC).AND.
	1   BTEST(STREAM(NSTREAM).FLAGS,FLG_V_NO_PROCESSOR))THEN
	      CALL FAKE_DYNAMIC_EXIT(NSTREAM,1)
	   ELSE
	      CALL SEND_EXIT_MSG(NSTREAM)
	   ENDIF
C
C	Should never get here -- an invalid request code was found.
C
	ELSE
	   CALL LOGGIT('Invalid request type in AST routine = ',NEWREQUEST)
	   STREAM(NSTREAM).REQUEST=ITEMP
	ENDIF
C
C	One more time:  if it's a START_STREAM request, reply to the job
C	controller with the proper attribute, SERVER or not.
C
	IF(NEWREQUEST.EQ.SMBMSG$K_START_STREAM)THEN
	   IF(BTEST(STREAM(NSTREAM).FLAGS,FLG_V_PRINTQUEUE))THEN
	      ISTAT=SMB$SEND_TO_JOBCTL(%REF(NSTREAM),%REF(NEWREQUEST))
	   ELSE
	      ISTAT=SMB$SEND_TO_JOBCTL(%REF(NSTREAM),%REF(NEWREQUEST),,,
	1      %REF(SMBMSG$M_SERVER))
	   ENDIF
	   CALL LOGGIT('SEND_TO_JOBCTL START_STREAM status = ',ISTAT)
C
C	Then, if the $CREPRC for the stream processor failed, pretend
C	that the process actually exited abnormally, to clean up
C	the SCB.
C
	   IF((ISTAT_CREPRC.AND.7).NE.1)THEN
	      CALL CREPRC_FAILED(NSTREAM,ISTAT_CREPRC)
	      STREAM(NSTREAM).REQUEST=SMBMSG$K_STOP_STREAM
	   ENDIF
C
C	If it's not a STOP_TASK, and it's not a STOP_STREAM for a stream
C	whose process still exists, and it's not a RESET_STREAM, then
C	we reply to the job controller that the request has been completed.
C	(Note that replying to a START_TASK with the START_TASK status means
C	that the task has been started, and not that the task is completed!)
C	We do not want to reply to STOP_TASK, STOP_STREAM with existing
C	process, or RESET_STREAM because these requests need to be 
C	completed at non-AST level; completing them here would allow the 
C	next task for the queue to wipe out the current one.
C
	ELSE IF(NEWREQUEST.NE.SMBMSG$K_STOP_TASK.AND.
	1 (NEWREQUEST.NE.SMBMSG$K_STOP_STREAM.OR.STREAM(NSTREAM).MBX_CHAN
	1 .EQ.-1).AND.NEWREQUEST.NE.SMBMSG$K_RESET_STREAM)THEN
	   ISTAT=SMB$SEND_TO_JOBCTL(%REF(NSTREAM),%REF(NEWREQUEST))
	   CALL LOGGIT('SEND_TO_JOBCTL non-STOP_TASK status = ',ISTAT)
	ENDIF
C
C	Another special case:  if it was a START_TASK request, and we didn't
C	really start the task (because it was the wrong copy of the job or
C	file), we simply respond with TASK_COMPLETE.  This gets us quickly
C	through all unwanted copies of the job and file(s).
C
	IF((NEWREQUEST.EQ.SMBMSG$K_START_TASK).AND.(.NOT.REALLY_START_TASK))
	1 THEN
	   ISTAT=SMB$SEND_TO_JOBCTL(%REF(NSTREAM),
	1   %REF(SMBMSG$K_TASK_COMPLETE),,,,%REF(STATUS_VECTOR))
	   CALL LOGGIT('SEND_TO_JOBCTL TASK_COMPLETE in MSG_AST status = ',
	1   ISTAT)
C
C	Make sure the stream state is idle after task "completion"
C
	   STREAM(NSTREAM).REQUEST=SMBMSG$K_START_STREAM
	ENDIF
C
C	Done.  Let non-AST code take it from here.
C
	RETURN
	END
C
C	EXIT_AST -- AST routine for completion of read QIO on process 
C	exit mailbox
C                            
	SUBROUTINE EXIT_AST
C
C	Make implicit declaration and specify INCLUDE files
C
	IMPLICIT INTEGER*4 (A-Z)
	INCLUDE 'EXECSYMB.INC'
	INCLUDE '($SJCDEF)'
	INCLUDE 'ITEMDEF.INC'
C
C	Declare itemlist for sending request to job controller
C
	RECORD /ITEM/ SJCLIST(2)
C
C	System and SMBSRVSHR routines called as functions
C
	INTEGER*4 SMB$SEND_TO_JOBCTL,SYS$DASSGN,SYS$SNDJBCW,SYS$DELLNM
C
C	Miscellaneous local storage
C
	INTEGER*4 EXITPID,PIDSTATUS
	CHARACTER*8 PROCPID
	INTEGER*4 NSTREAM
	CHARACTER*17 EXITSTAT
	LOGICAL*1 ALT_ENTRY
C
C	Start of code:  process exit is an event that should awaken 
C	mainline non-AST level code
C
	CALL WAKEUP
C
C	Indicate entry at EXIT_AST and not at CREPRC_FAILED (below)
C
	ALT_ENTRY=.FALSE.
C
C	Read process status and PID from mailbox message
C
	CALL LIB$MOVC3(%REF(4),%REF(EXITBUF(5:8)),%REF(PIDSTATUS))
	CALL LIB$MOVC3(%REF(4),%REF(EXITBUF(9:12)),%REF(EXITPID))
C
C	Scan stream SCB's for one with this PID
C
4	DO NSTREAM=0,MAX_STREAMS-1
	   IF(STREAM(NSTREAM).PID.EQ.EXITPID)THEN
	      CALL TELL_OPER('Processor for '//STREAM(NSTREAM).OURNAME
	1      (1:STREAM(NSTREAM).OURNAME_LEN)//' has exited.',3)
	      CALL SYS$FAO('!8XL',,PROCPID,%VAL(EXITPID))
	      ISTAT=SYS$DELLNM('LNM$GROUP','QUEUE_NAME_'//PROCPID,)
	      CALL LOGGIT('SYS$DELLNM QUEUE_NAME status =',ISTAT)
	      ISTAT=SYS$DELLNM('LNM$GROUP','STAT_W_MBX_'//PROCPID,)
	      CALL LOGGIT('SYS$DELLNM STAT_W_MBX status =',ISTAT)
	      GOTO7
	   ENDIF
	ENDDO
C
C	If we couldn't find the PID anywhere, it's probably a processor that
C	didn't exit in time, and was replaced by a new one.  Log it.
C
	CALL SYS$FAO('!8XL=!8XL',,EXITSTAT,%VAL(EXITPID),%VAL(PIDSTATUS))
	CALL LOGGIT('Exit status for unknown PID '//EXITSTAT//' ignored',)
	GOTO99
C
C	FAKE_DYNAMIC_EXIT is used with streams that have dynamic processors.
C	When one of these streams is supposed to be stopped, this entry gets
C	called to fake the processor exit if there is no active processor.
C	It works like CREPRC_FAILED but doesn't print the error message.
C
	ENTRY FAKE_DYNAMIC_EXIT(INSTREAM,IPIDSTATUS)
	CALL LOGGIT('Faking exit of inactive dynamic processor...',
	1 INSTREAM*256+3)
	GOTO69
C
C	CREPRC_FAILED enters with NSTREAM and PIDSTATUS already specified,
C	and just handles the rest of the work.
C
	ENTRY CREPRC_FAILED(INSTREAM,IPIDSTATUS)
	CALL LOGGIT('Processing $CREPRC failure as if process exited...',
	1 INSTREAM*256+3)
69	NSTREAM=INSTREAM
	PIDSTATUS=IPIDSTATUS
	ALT_ENTRY=.TRUE.
C
C	Format a string for subsequent messages
C
7	CALL SYS$FAO('!2UB = %X!8XL  ',,EXITSTAT,%VAL(NSTREAM),
	1 %VAL(PIDSTATUS))
C
C	If the stream was already "stopped", this is an expected process
C	exit, so send the STOP_STREAM status back to the job controller.
C
8	IF(STREAM(NSTREAM).REQUEST.EQ.SMBMSG$K_STOP_STREAM)THEN
	   ISTAT=SMB$SEND_TO_JOBCTL(%REF(NSTREAM),%REF(SMBMSG$K_STOP_STREAM))
	   CALL LOGGIT('EXIT_AST SEND_TO_JOBCTL STOP_STREAM status = ',ISTAT)
C
C	If the stream state was other than STOP_STREAM, check to see if this
C	is a dynamic processor that timed out.  (Cannot check the EXITING bit
C	here since EXIT_TIMER_AST may have cleared it, so use DYNAMIC and 
C	NO_PROCESSOR.  Make sure EXITING is cleared in any case.)
C
	ELSE IF(BTEST(STREAM(NSTREAM).FLAGS,FLG_V_DYNAMIC).AND.
	1 BTEST(STREAM(NSTREAM).FLAGS,FLG_V_NO_PROCESSOR))THEN
	   STREAM(NSTREAM).FLAGS=STREAM(NSTREAM).FLAGS.AND..NOT.FLG_M_EXITING
C
C	Otherwise, it's an unexpected process exit.  We need to do some 
C	cleanup in that case, so that the job controller, our code, and the 
C	stream SCB all reflect the fact that the stream has stopped on its own.
C
	ELSE
	   CALL LOGGIT('Process exit status for stream '//EXITSTAT,)
C
C	Set up and submit a RESET_QUEUE request for the stream.  This forces
C	the job controller to put the stream in the "stopped" state.
C
	   SJCLIST(1).BUFLEN=STREAM(NSTREAM).OURNAME_LEN
	   SJCLIST(1).ITEMCODE=SJC$_QUEUE
	   SJCLIST(1).BUFADR=%LOC(STREAM(NSTREAM).OURNAME)
	   SJCLIST(1).RETLENADR=0
	   SJCLIST(2).TERMINATOR=0
	   ISTAT=SYS$SNDJBCW(,%VAL(SJC$_RESET_QUEUE),,%REF(SJCLIST),,,)
	   CALL LOGGIT('EXIT_AST $SNDJBCW RESET_QUEUE status = ',ISTAT)
	ENDIF
C
C	Since the process exited, clear the stream's PID, and if the 
C	mailbox for the stream still has a channel assigned, deassign it.
C
	STREAM(NSTREAM).PID=0
	IF(STREAM(NSTREAM).MBX_CHAN.NE.-1)THEN
	   ISTAT=SYS$DASSGN(%VAL(STREAM(NSTREAM).MBX_CHAN))
	   CALL LOGGIT('$DASSGN status = ',ISTAT)
	   STREAM(NSTREAM).MBX_CHAN=-1
	ENDIF
C
C	Finally, post another read to the process exit mailbox, so we'll
C	get called again when another process exits (but only if we came
C	in at EXIT_AST!)
C
99	IF(.NOT.ALT_ENTRY)CALL READ_EXIT_MBX
	RETURN         
	END
C
C	IO_DONE_AST -- AST routine for completion of read QIO on the status
C	return mailbox (used by processes to report completion status of
C	a task)
C                      
	SUBROUTINE IO_DONE_AST
C
C	Make implicit declaration and specify required INCLUDE files
C
	IMPLICIT INTEGER*4 (A-Z)
	INCLUDE '($SJCDEF)'
	INCLUDE 'EXECSYMB.INC'
	INCLUDE 'ITEMDEF.INC'
C
C	Declare system routines called as functions
C
	INTEGER*4 SYS$SNDJBC,SYS$SNDJBCW
C
C	Declare itemlist for requesting requeue and hold on current job
C
	RECORD /ITEM/ ITEMLIST(4)
C
C	Miscellaneous local storage
C
	CHARACTER*32 SJC_OUR_QUEUE
	LOGICAL*1 FIRST_TIME
	INTEGER*4 IOSB(2)
	CHARACTER*8 HEXVAL
C
C	FIRST_TIME is used to avoid reinitializing variables that can't
C	be set up via DATA statements
C
	DATA FIRST_TIME/.TRUE./
C
C	Start of code -- if first time in this routine, initialize the
C	itemlist constant fields
C
	IF(FIRST_TIME)THEN
	   FIRST_TIME=.FALSE.
	   ITEMLIST(1).ITEMCODE=SJC$_QUEUE
	   ITEMLIST(1).BUFADR=%LOC(SJC_OUR_QUEUE)
	   ITEMLIST(1).RETLENADR=0
	   ITEMLIST(2).ITEMCODE=SJC$_REQUEUE
	   ITEMLIST(2).BUFLEN=0
	   ITEMLIST(2).BUFADR=0
	   ITEMLIST(2).RETLENADR=0
	   ITEMLIST(3).ITEMCODE=SJC$_HOLD
	   ITEMLIST(3).BUFLEN=0
	   ITEMLIST(3).BUFADR=0
	   ITEMLIST(3).RETLENADR=0
	   ITEMLIST(4).TERMINATOR=0
	ENDIF
C
C	Receiving a task completion status from a process requires us to
C	wake up the mainline non-AST level code (since it may be hibernating
C	waiting for this to happen)
C
	CALL WAKEUP
C
C	If the IOSB shows other than success, skip processing and re-post
C	the I/O
C
	IF((STATIOSB(1).AND.1).EQ.0)GOTO90
C
C	If the status buffer contains twelve characters, the last two must be
C	a comma and a stream identifier ("0"=0, ..., "9"=9, "A"=10, ..., "V"=31)
C	and the indicated stream must have a stored PID of hex FFFFFFFF (which
C	indicates that the stream's processor is a permanent process.)
C
	IF(STATIOSBW(2).EQ.12)THEN
	   IF(STATBUF(11:11).NE.',')GOTO5
	   I=ICHAR(STATBUF(12:12))-'30'X
	   IF(I.GT.9)I=I-7
	   IF(I.LT.0.OR.I.GE.MAX_STREAMS)GOTO5
	   IF(STREAM(I).PID.NE.'FFFFFFFF'X)THEN
	      CALL SYS$FAO('!8XL',,HEXVAL,%VAL(STATIOSB(2)))
	      CALL LOGGIT('Status message '//STATBUF//
	1      ' for wrong stream from PID '//HEXVAL,)
	      CALL SYS$FAO('!8XL',,HEXVAL,%VAL(STREAM(I).PID))
	      CALL LOGGIT('Stream is owned by PID '//HEXVAL,)
	      GOTO90
	   ENDIF
	   GOTO1
5	   CALL SYS$FAO('!8XL',,HEXVAL,%VAL(STATIOSB(2)))
	   CALL LOGGIT('Invalid status message '//STATBUF//
	1   ' from PID '//HEXVAL,)
	   GOTO90
	ENDIF
C
C	Use the second longword of the IOSB on the mailbox read (which, for
C	the mailbox device, contains the PID of the process that wrote to 
C	the mailbox) to determine which stream's process returned the 
C	status value.  This also eliminates spurious mailbox messages.
C
	DO I=0,MAX_STREAMS-1
	   IF(STREAM(I).PID.EQ.STATIOSB(2))GOTO1
	ENDDO
C
C	If it wasn't from one of our processes, ignore it.
C	(Actually, first check for a debug command.  If it isn't a command, 
C	then ignore it.  If it is, process it and reset to "NOOP" in case
C	we drop through to here on a mailbox EOF.)
C
	IF(STATBUF(1:6).EQ.'$DEBUG')THEN
	   CALL DEBUG_MODE_COMMAND(STATBUF(7:10))
	   STATBUF(7:10)='NOOP'
	ELSE
	   CALL SYS$FAO('!8XL',,HEXVAL,%VAL(STATIOSB(2)))
	   CALL LOGGIT('Status received from unknown PID '//HEXVAL,)
	ENDIF
	GOTO90
C
C	Set the stream status by default to "abort", then try to read the
C	status message.  The status message is expected to be ten characters
C	long, in ASCII, with the last eight characters containing the status
C	value in hex.  This is done so that a process command procedure
C	can copy $STATUS directly to the mailbox -- the $STATUS DCL symbol
C	ordinarily has a string value in the format "%Xnnnnnnnn".
C
1	NSTREAM=I
	STREAM(NSTREAM).STATUS='2C'X
	CALL LIB$CVT_HTB(%VAL(8),%REF(STATBUF(3:10)),
	1 %REF(STREAM(NSTREAM).STATUS))
C
C	Determine the severity of the status.  If it isn't 2 (error) 
C	or 4 (fatal error), we accept it.  Also, if no requeue time was
C	specified for the queue, we accept the status regardless of severity.
C
9	SEVERITY=(STREAM(NSTREAM).STATUS).AND.7
	IF(SEVERITY.NE.2.AND.SEVERITY.NE.4)GOTO90
	IF(.NOT.BTEST(STREAM(NSTREAM).FLAGS,FLG_V_DO_REQUEUE))GOTO90
C
C	If the task had an error or fatal error completion status, and the 
C	stream has a requeue time specified, we will attempt to abort the
C	job and requeue it for retry.  (The job controller doesn't know, at
C	this point, that the task failed -- since we didn't tell it yet!)
C	Set up the dynamic portions of the itemlist with information from
C	the stream's SCB, and send the job controller an ABORT_JOB request,
C	asking for a HOLD status and a requeue on the job.  Note that the
C	ABORT_JOB request cannot possibly complete while we're here, since
C	this code runs at AST level.  The job controller will send a 
C	STOP_TASK request to us, which MSG_AST will pass on to the mainline
C	non-AST level code that is designed to handle aborting the task and
C	requeuing it with an AFTER-time.
C
	SJC_OUR_QUEUE=STREAM(NSTREAM).OURNAME(1:STREAM(NSTREAM).OURNAME_LEN)
	ITEMLIST(1).BUFLEN=STREAM(NSTREAM).OURNAME_LEN
	ISTAT=SYS$SNDJBCW(,%VAL(SJC$_ABORT_JOB),,%REF(ITEMLIST),IOSB,,)
	CALL LOGGIT('$SNDJBCW ABORT status = ',ISTAT)
	CALL LOGGIT('$SNDJBCW ABORT IOSB = ',IOSB(1))
C
C	Before we leave, we clear the status so that the mainline code won't
C	do anything to this task until the ABORT_TASK request comes in.
C	Also, we set the stream's ABORT flag, indicating that we are the ones
C	who requested the task abort.  (This lets us distinguish our aborts,
C	which should be requeued, from those done by someone else for reasons
C	that we know nothing about.  We let the person doing the abort worry
C	about the consequences in that case.)
C
	STREAM(NSTREAM).STATUS=0
	STREAM(NSTREAM).FLAGS=STREAM(NSTREAM).FLAGS.OR.FLG_M_ABORT
C
C	All paths merge here, to post another read to the status mailbox,
C	and return.
C
90	CALL READ_STAT_MBX
	RETURN
	END
C
C	WRITE_MBX -- routine to write a message to a process command mailbox
C
	SUBROUTINE WRITE_MBX(ISTREAM,MESS,NOWFLG,WAITING)
	IMPLICIT INTEGER*4 (A-Z)
	INCLUDE 'EXECSYMB.INC'
	INCLUDE '($IODEF)'
	EXTERNAL WAKEUP
	INTEGER*4 ISTREAM
	CHARACTER*(*) MESS
	LOGICAL*1 NOWFLG,WAITING
C
C	Check to make sure the stream still has a channel assigned to its
C	mailbox.  If it doesn't, don't do a whole lot.
C
	IF(STREAM(ISTREAM).MBX_CHAN.NE.-1)THEN
	   L=LEN(MESS)
C
C	If the write will demand an immediate return, don't bother with
C	the IOSB; otherwise, initialize the IOSB to zero.  Then post the
C	QIO; if immediate return, do a QIOW, else do a plain QIO and specify
C	WAKEUP as the AST routine, so that we can hibernate until the request
C	completes.
C
	   IF(NOWFLG)THEN
	      ISTAT=SYS$QIOW(,%VAL(STREAM(ISTREAM).MBX_CHAN),
	1      %VAL(IO$_WRITEVBLK+IO$M_NOW),,,,%REF(MESS),%VAL(L),,,,)
	      CALL LOGGIT('WRITE_MBX $QIOW status = ',ISTAT)
C
C	If "WAITING" flag is set, then this is a QIO that uses the IOSB and 
C	requires a WAKEUP when it completes.
C
	   ELSE IF(WAITING)THEN
	      STREAM(ISTREAM).IOSB(1)=0
	      ISTAT=SYS$QIO(%VAL(ISTREAM+1),%VAL(STREAM(ISTREAM).MBX_CHAN),
	1      %VAL(IO$_WRITEVBLK),STREAM(ISTREAM).IOSB,WAKEUP,,
	2      %REF(MESS),%VAL(L),,,,)
	      CALL LOGGIT('WRITE_MBX $QIO 1 status = ',ISTAT)
	   ELSE
	      ISTAT=SYS$QIO(,%VAL(STREAM(ISTREAM).MBX_CHAN),
	1      %VAL(IO$_WRITEVBLK),,,,%REF(MESS),%VAL(L),,,,)
	      CALL LOGGIT('WRITE_MBX $QIO 2 status = ',ISTAT)
	   ENDIF                                           
	ENDIF
C
C	Return after posting the QIO (or after doing nothing, if no mailbox)
C
	RETURN
	END
C
C	WAKEUP -- make sure mainline code wakes up (call from/as AST routine)
C
	SUBROUTINE WAKEUP
	IMPLICIT INTEGER*4 (A-Z)
	INCLUDE 'EXECSYMB.INC'
	INTEGER*4 SYS$WAKE
C
C	If the flag indicating a wakeup is necessary is set, call SYS$WAKE
C	to do it.  (Mainline code sets DO_WAKE when it's about to hibernate.
C	This lets us avoid excessive SYS$WAKE calls, possible if we just
C	base our decision on the value of DO_HIBER.)
C
	IF(DO_WAKE)THEN
	   ISTAT=SYS$WAKE(,)
	   CALL LOGGIT('$WAKE status = ',ISTAT)
C
C	If the wakeup flag is not set, simply clear the DO_HIBER flag, so
C	that the mainline code will know not to hibernate at the end of 
C	this pass through the streams.
C
	ELSE
	   DO_HIBER=.FALSE.
	ENDIF
C
C	That done, we return.
C
	RETURN
	END
C
C	WRITE_MBX_ASCII -- routine to write the command mailbox for a stream
C	that uses the ASCII format for item transmission
C
C	The ASCII format for item transmission is intended for use with 
C	streams serviced by command procedures.  Each item is sent as two
C	mailbox messages:  the first contains the name of the item, and the
C	second contains the value of the item, edited (in most cases) from
C	its binary format into a suitable ASCII format, if necessary.  This
C	enables DCL code like the following to read all items:
C		$ INPUT_LOOP:
C		$    read/end=eof cmd_mbx itemname	! read name of item
C		$    read/end=eof cmd_mbx 'itemname	! read value of item
C		$    if itemname .nes. "EXEC_STEP" then goto INPUT_LOOP
C	Some items are not edited into ASCII, but are passed as-is, even 
C	though they contain some binary information.  See the main program's
C	table of item conversion information for details.
C
	SUBROUTINE WRITE_MBX_ASCII(ISTREAM,ITEMNAME,MESS)
	IMPLICIT INTEGER*4 (A-Z)
	INCLUDE 'EXECSYMB.INC'
	CHARACTER*(*) ITEMNAME,MESS
	LOGICAL*1 X_ENTRY,ASTSET
	X_ENTRY=.FALSE.
	ASTSET=.FALSE.
	GOTO1
C
C	Alternate entry for non-itemlist writes (e.g. EXIT or RESET commands)
C
	ENTRY WRITE_MBX_ASC_X(ISTREAM,ITEMNAME,MESS)
	X_ENTRY=.TRUE.
	ASTSET=.FALSE.
	IF(STREAM(ISTREAM).MBX_CHAN.NE.-1)THEN
	   ISTAT=SYS$CANCEL(%VAL(STREAM(ISTREAM).MBX_CHAN))
	   CALL LOGGIT('$CANCEL status in ASC = ',ISTAT)
	   CALL SETAST(%REF(0))
	   ASTSET=.TRUE.
	ENDIF
C
C	First of all, if the processor is dynamic and it's not there,
C	start it.
C
1	IF(BTEST(STREAM(ISTREAM).FLAGS,FLG_V_DYNAMIC).AND.
	1 BTEST(STREAM(ISTREAM).FLAGS,FLG_V_NO_PROCESSOR))
	1 CALL START_PROCESSOR(ISTREAM,%REF(TEMP))
C
C	Make sure there's a mailbox channel for this stream.
C
	IF(STREAM(ISTREAM).MBX_CHAN.NE.-1)THEN
C
C	Write the item name using the common routine
C
	   CALL WRITE_MBX(ISTREAM,ITEMNAME,X_ENTRY,.FALSE.)
C
C	Write the item value, posting the QIO but not waiting for it
C
	   CALL WRITE_MBX(ISTREAM,MESS,X_ENTRY,.TRUE.)
	ENDIF
C
C	Re-enable AST's once we are done, and return
C
	IF(ASTSET)CALL SETAST(%REF(1))
	RETURN
	END
C
C	WRITE_MBX_BINARY -- write the item code and value to the process
C	command mailbox, without formatting or editing the item value.
C	This routine is for use with streams that specify the BINARY format
C	of item value transmission.  In the BINARY format, one message is
C	written for each item.  The first four bytes of the message contain
C	the longword item code in binary, while the remainder of the message
C	contains the item value, as received from the job controller.  This
C	is designed primarily for queues serviced by programs written in
C	"real" (i.e. non-DCL) programming languages.
C
	SUBROUTINE WRITE_MBX_BINARY(ISTREAM,ITEMCODE,MESS)
	IMPLICIT INTEGER*4 (A-Z)
	INCLUDE 'EXECSYMB.INC'
	CHARACTER*(*) MESS
	CHARACTER*4 TEMP
	INTEGER*4 ITEMCODE
	LOGICAL*1 X_ENTRY,ASTSET
	X_ENTRY=.FALSE.
	ASTSET=.FALSE.
	GOTO1
C
C	Alternate entry for non-itemlist writes (e.g. EXIT or RESET commands)
C
	ENTRY WRITE_MBX_BIN_X(ISTREAM,ITEMNAME,MESS)
	X_ENTRY=.TRUE.
	ASTSET=.FALSE.
	IF(STREAM(ISTREAM).MBX_CHAN.NE.-1)THEN
	   ISTAT=SYS$CANCEL(%VAL(STREAM(ISTREAM).MBX_CHAN))
	   CALL LOGGIT('$CANCEL status in BIN = ',ISTAT)
	   CALL SETAST(%REF(0))
	   ASTSET=.TRUE.
	ENDIF
C
C	First of all, if the processor is dynamic and it's not there,
C	start it.
C
1	IF(BTEST(STREAM(ISTREAM).FLAGS,FLG_V_DYNAMIC).AND.
	1 BTEST(STREAM(ISTREAM).FLAGS,FLG_V_NO_PROCESSOR))
	1 CALL START_PROCESSOR(ISTREAM,%REF(TEMP))
C
C	Make sure the mailbox channel exists
C
	IF(STREAM(ISTREAM).MBX_CHAN.NE.-1)THEN
C
C	Write the item code into a character string in binary
C
	   CALL LIB$SCOPY_R_DX(%REF(4),ITEMCODE,TEMP)
C
C	Post the new write QIO to the mailbox
C
	   CALL WRITE_MBX(ISTREAM,TEMP//MESS,X_ENTRY,.TRUE.)
	ENDIF
C
C	Finally, re-enable AST's
C
	IF(ASTSET)CALL SETAST(%REF(1))
C
C	Return when done
C
	RETURN
	END
C
C	LOGGIT -- Routine for logging messages to the log file (and to the
C	operator's console)
C
	SUBROUTINE LOGGIT(STRING,THING)
	IMPLICIT INTEGER*4 (A-Z)
	INCLUDE 'EXECSYMB.INC'
	INTEGER*4 ISTAT
	CHARACTER*(*) STRING
	INTEGER*4 THING
	CHARACTER*9 CDATE
	CHARACTER*8 CTIME
	CHARACTER*8 HEXVAL
	LOGICAL*1 DO_OPER
	CHARACTER*14 FILLER
	CHARACTER*1 SYMB_INDEX
	COMMON /PRCNAM/ FILLER,SYMB_INDEX
C
C	Disable AST's to prevent interference
C
	CALL SETAST(%REF(0))
C
C	Determine if this call specified the status argument
C
	LTHING=%LOC(THING)
C
C	Assume the operator will not be notified
C
	DO_OPER=.FALSE.
C
C	If the status field was specified, and the status wasn't a
C	success code (i.e. low bit was zero), notify the operator
C
	IF(LTHING.NE.0)THEN
C
C	If it was a success status, don't log anything anywhere.
C	(NOTE:  for debugging purposes, to get all messages into the log file 
C	and to the operator, comment out the next line.  To just get 
C	everything into the log file, comment out the next two lines.
C
	   IF((THING.AND.'FFFF'X).EQ.1)GOTO90
	   DO_OPER=.TRUE.
	   SEVERITY=(THING.AND.7)
	ELSE
	   SEVERITY=1
	ENDIF
C
C	Get the current date and time
C
	CALL DATE(CDATE)
	CALL TIME(CTIME)
C
C	Make sure the message will fit, by truncating it at 228 characters
C	(This allows 28 characters for the date and time, and the status
C	value in hexadecimal, for a total maximum of 256 characters per 
C	record)
C
	L=LEN(STRING)
	IF(L.GT.228)L=228
	IF(LTHING.NE.0)THEN
	   CALL SYS$FAO('!8XL',,HEXVAL,%VAL(THING))
	ELSE
	   HEXVAL='        '
	ENDIF
C
C	If notifying the operator, send the message with the status value.
C	(OPCOM does its own time stamping and formatting)
C
	IF(DO_OPER)CALL TELL_OPER(STRING(1:L)//HEXVAL,SEVERITY)
C
C	Open the log file for append.  If it fails to open, go create a
C	new version, and return to the next line to write the message.
C
	OPEN(UNIT=1,NAME=LOGDIR(1:L_LD)//':EXECSYMB'//SYMB_INDEX//
	1 '.LOG',TYPE='OLD',ACCESS='APPEND',ERR=99)
C
C	If the message is too long for one line, write it in two lines
C
1	IF(L.GT.104)THEN
	   WRITE(1,2)CDATE,CTIME,STRING(1:104),STRING(105:L),HEXVAL
2	   FORMAT(3(1X,A)/1X,A,A)
C
C	If it fits in one line, write it in one line.
C
	ELSE
	   WRITE(1,2)CDATE,CTIME,STRING//HEXVAL
	ENDIF
C
C	Close the logfile, allowing it to be examined.
C
	CLOSE(UNIT=1)
C
C	Re-enable AST's now, and return
C
90	CALL SETAST(%REF(1))
	RETURN
C
C	If we couldn't open the log file for append, create a new version,
C	and go back to the code that writes the message
C
99	OPEN(UNIT=1,NAME=LOGDIR(1:L_LD)//':EXECSYMB'//SYMB_INDEX//
	1 '.LOG',TYPE='NEW',CARRIAGECONTROL='LIST',ERR=90)
	GOTO1
	END
C
C	TELL_OPER -- routine to send message to CENTRAL operator
C
	SUBROUTINE TELL_OPER(MESS,SEVERITY)
	INCLUDE '($OPCDEF)'
	CHARACTER*(*) MESS
	INTEGER*4 SEVERITY
	CHARACTER*280 REALMESS
	CHARACTER*14 FILLER
	CHARACTER*1 SYMB_INDEX
	COMMON /PRCNAM/ FILLER,SYMB_INDEX
	BYTE MSGTYPE(0:7)
	DATA MSGTYPE/'W','S','E','I','F','?','?','?'/
	L=LEN(MESS)
C
C	Format the message with a sort-of-standard VMS-like header.
C	Include the process index, in case multiple copies of this symbiont
C	are running at the same time.
C
	REALMESS(1:1)=CHAR(OPC$_RQ_RQST)
	CALL LIB$SCOPY_R_DX(%REF(3),%REF(OPC$M_NM_OPER12),
	1 %DESCR(REALMESS(2:4)))
	CALL LIB$SCOPY_R_DX(%REF(4),%REF(0),%DESCR(REALMESS(5:8)))
	REALMESS(9:L+27)='%SYMBIONT_EXEC_'//SYMB_INDEX//'-'//
	1 CHAR(MSGTYPE(SEVERITY))//'-'//MESS(1:L)
	L=L+27
C
C	Send it to OPCOM, and return
C
	CALL SYS$SNDOPR(REALMESS(1:L),)
	RETURN
	END
C
C	ITEMEDIT -- routine to edit item value into ASCII
C
	SUBROUTINE ITEMEDIT(EDITBUF,ITEM,LE,L,TYPE)
	IMPLICIT INTEGER*4 (A-Z)
	CHARACTER*(*) ITEM
	CHARACTER*256 EDITBUF
	INTEGER*2 L
	BYTE TYPE
	BYTE BYTE
	INTEGER*2 WORD
	INTEGER*4 LONG
	INTEGER*4 TEMP(2)
C
C	(See main program for table of item types)
C
C	Note that type 1 is NOT handled here!!!
C
	GOTO(10,20,30,40,50),TYPE
10	RETURN
20	CALL SYS$FAO('%X!8XL',,EDITBUF(1:10),%VAL(LONG(ITEM(1:4))))
23	LE=10
	GOTO10
30	TEMP(1)=LONG(ITEM(1:4))
	TEMP(2)=LONG(ITEM(5:8))
	CALL SYS$ASCTIM(,%DESCR(EDITBUF(1:23)),%REF(TEMP),)
	LE=23
	GOTO10
40	CALL SYS$FAO('!15%U',,EDITBUF(1:15),%VAL(LONG(ITEM(1:4))))
43	LE=15
	GOTO10
50	LE=1
	DO I=1,L/4
	   TEMP(1)=LONG(ITEM(I*4-3:I*4))
	   DO J=0,31
	      IF(BTEST(TEMP(1),J))CALL OUTDEC(EDITBUF,LE,I*32+J-32)
	      IF(LE.GE.253)GOTO59
	   ENDDO
	ENDDO
59	LE=LE-1
	GOTO10
	END
C
C	OUTDEC -- output value in 0:127 range as one, two, or three digits,
C	followed by a comma.
C	The output is placed in a string at a given position, and the position
C	is advanced by the length of the string inserted.
C
	SUBROUTINE OUTDEC(STRING,IPOS,VALUE)
	INTEGER*4 IPOS,VALUE
	CHARACTER*(*) STRING
C
C	Determine how many digits it should have
C
	IF(VALUE.LT.10)THEN
	   IDIG=1
	ELSE IF(VALUE.LT.100)THEN
	   IDIG=2
	ELSE
	   IDIG=3
	ENDIF
C
C	Format the value with the required number of digits, and a comma.
C	(If this results in an error, don't change the position or the 
C	contents of the string.)
C
	CALL SYS$FAO('!#UL,',,STRING(IPOS:IPOS+IDIG),%VAL(IDIG),%VAL(VALUE))
C
C	Advance the position in the string
C
	IPOS=IPOS+IDIG+1
C
C	Return when done
C
9	RETURN
	END
C
C	PARSE_ITEMS -- routine to parse the "ITEMS=list" parameter for a 
C	stream and fill an ORDER array with the specified item numbers.
C	The list of item numbers can contain either single decimal values
C	or ranges.  Numbers are separated by commas.  A range consists of
C	two decimal numbers separated by a colon, and implies inclusion of
C	all of the numbers between the two numbers, including the endpoints,
C	and in the order given.  For example, the list "4,19:21,5,33:30,7"
C	is the same as the list "4,19,20,21,5,33,32,31,30,7".
C	This routine may also accept some slightly erroneous item lists,
C	and try to make sense of them.  In any case, since the results
C	will be fed back into the queue parameters, the user will see which
C	items were actually selected.
C
	SUBROUTINE PARSE_ITEMS(STRING,ORDER)
	INCLUDE 'SMBDEF.INC'
	CHARACTER*(*) STRING
	BYTE ORDER(SMBMSG$K_MAX_ITEM_CODE)
	INTEGER*2 FROM,THRU,CH,N,IX,K
	LOGICAL*1 EOS
C
C	Initialize:  not at the end of the string, the input pointer is
C	at the start of the string, and the item number counter is at the
C	first position in ORDER.  Also, get the string's length.
C
	EOS=.FALSE.
	IPTR=1
	K=1
	L=LEN(STRING)
C
C	State 1:  looking for the first number in a pair, or a single number
C	Initialize range values both to zero (which isn't a valid item number)
C
1	FROM=0
	THRU=0
C
C	If at the end of the string, set the flag, consider the last number
C	as a single number, and go add it to ORDER.
C
10	IF(IPTR.GT.L)THEN
	   EOS=.TRUE.
	   THRU=FROM
	   GOTO30
	ENDIF
C
C	Otherwise, get the next character and advance the pointer
C
	CH=ICHAR(STRING(IPTR:IPTR))
	IPTR=IPTR+1
C
C	If it's a digit, add it into the beginning-of-range value and
C	loop for another character in this state.
C
	IF(CH.GE.ICHAR('0').AND.CH.LE.ICHAR('9'))THEN
	   FROM=FROM*10+(CH-48)
	   GOTO10
C
C	If it's a comma, this was a single number, so go add it to ORDER.
C
	ELSE IF(CH.EQ.ICHAR(','))THEN
	   THRU=FROM
	   GOTO30
C
C	If it's a colon, this is a range, so drop through to state 2 to get 
C	the end-of-range number.  Otherwise, we've hit an invalid character,
C	which probably means that the last comma followed the last number
C	in the list.  If we picked up a number after the last comma, add it
C	to ORDER, but in any case assume we've reached the end of the list.
C
	ELSE IF(CH.NE.ICHAR(':'))THEN
	   IF(FROM.EQ.0)GOTO90
	   EOS=.TRUE.
	   THRU=FROM
	   GOTO30
	ENDIF
C
C	State 2:  looking for the end-of-range value.
C	First, check to make sure we haven't reached the end of the string.
C	If we have, go add the final range to the list.  If no end-of-range
C	value was given, recover from the error by adding the start-of-range
C	as a single number.
C
20	IF(IPTR.GT.L)THEN
	   EOS=.TRUE.
	   IF(THRU.EQ.0)THRU=FROM
	   GOTO30
	ENDIF
C
C	Otherwise, pick up the next character and advance the pointer.
C
	CH=ICHAR(STRING(IPTR:IPTR))
	IPTR=IPTR+1
C
C	If it's a digit, add it into the end-of-range value and repeat.
C
	IF(CH.GE.ICHAR('0').AND.CH.LE.ICHAR('9'))THEN
	   THRU=THRU*10+(CH-48)
	   GOTO20
C
C	If it's a comma, fall through to add the range to ORDER.
C	Otherwise, decide we're at the end of the string, and then fall
C	through anyway.
C
	ELSE IF(CH.NE.ICHAR(','))THEN
	   EOS=.TRUE.
	ENDIF
C
C	Come here to add a number or range of numbers to ORDER.
C	First, make the numbers valid by range-checking them against
C	the valid item codes.
C
30	IF(FROM.LT.1)FROM=1
	IF(FROM.GE.SMBMSG$K_MAX_ITEM_CODE)FROM=SMBMSG$K_MAX_ITEM_CODE-1
	IF(THRU.LT.1)THRU=1
	IF(THRU.GE.SMBMSG$K_MAX_ITEM_CODE)THRU=SMBMSG$K_MAX_ITEM_CODE-1
C
C	Next, determine which way the range goes, up or down.
C
	IF(FROM.GT.THRU)THEN
	   IX=-1
	   N=FROM-THRU+1
	ELSE
	   IX=1
	   N=THRU-FROM+1
	ENDIF
C
C	If too many items were specified, truncate the extra ones.
C
	IF(K+N.GT.SMBMSG$K_MAX_ITEM_CODE)THEN
	   THRU=THRU-(IX*(K+N-SMBMSG$K_MAX_ITEM_CODE))
	ENDIF
C
C	Fill in the range of values.
C
	DO I=FROM,THRU,IX
	   ORDER(K)=I
	   K=K+1
	ENDDO
C
C	If more room is left in the ORDER array, and we haven't reached
C	the end of the list, go back for more.
C
	IF((K.LT.SMBMSG$K_MAX_ITEM_CODE).AND.(.NOT.EOS))GOTO1
C
C	When we're finally done, fill the remaining slots in ORDER with
C	zeroes.  (The first zero serves as the list terminator, and we
C	never allow the last slot in ORDER to be filled with a real item
C	code.)
C
90	DO I=K,SMBMSG$K_MAX_ITEM_CODE
	   ORDER(I)=0
	ENDDO
C
C	Return when done
C
	RETURN
	END
C
C	EDIT_NJRM -- routine to format item list into new JOB_RESET_MODULES 
C	value
C
	SUBROUTINE EDIT_NJRM(ORDER,LINE,L)
	IMPLICIT INTEGER*4 (A-Z)
	INCLUDE 'SMBDEF.INC'
	BYTE ORDER(SMBMSG$K_MAX_ITEM_CODE)
	CHARACTER*(*) LINE
	INTEGER*2 L
	CHARACTER*256 TEMP
C
C	If there are no items in ORDER, don't put anything into the list
C
	IF(ORDER(1).EQ.0)RETURN
C
C	Set up a pointer into the output string, and insert a leading comma
C	if this isn't the first thing in the string (as it may not be, if
C	the "TIME=..." field was already inserted.)
C
	IP=L+1
	IF(IP.GT.1)THEN
	   LINE(IP:IP)=','
	   IP=IP+1
	ENDIF
C
C	Insert the leading "ITEMS=" string
C
	LINE(IP:IP+5)='ITEMS='
	IP=IP+6
	I=0
C
C	Loop:  keep going until we process all of ORDER or we find a zero code
C
1	I=I+1
2	IF(I.GT.SMBMSG$K_MAX_ITEM_CODE)GOTO10
	K=ORDER(I)
	IF(K.EQ.0)GOTO10
C
C	Put the decimal value of the item number in the string (followed by 
C	a comma)
C
	CALL OUTDEC(LINE,IP,K)
C
C	Inner loop:  check subsequent item codes to see if they form a range
C	with the present one.  A range consists of two or more item codes
C	where all codes are either +1 apart or -1 apart.  Do the same checks
C	here as well (looking for end-of-array or a zero terminator)
C
	J=I+1
	IX=0
	IF(J.GT.SMBMSG$K_MAX_ITEM_CODE)GOTO10
	KK=ORDER(J)
	IF(KK.EQ.0)GOTO10
	IX=KK-K
C
C	If the next value is not +1 or -1 from the present one, there's no
C	range; go back to the main loop
C
	IF(ABS(IX).NE.1)GOTO1
C
C	Got a range started, now find the end.  Same checks for end-of-array
C	and zero terminator are here as well.
C
3	J=J+1
	IF(J.GT.SMBMSG$K_MAX_ITEM_CODE)GOTO4
	KK=KK+IX
	IF(KK.NE.ORDER(J))GOTO4
	IF(KK.NE.0)GOTO3
C
C	If we found a complete range, insert a colon and the terminating value.
C
4	LINE(IP-1:IP-1)=':'
	CALL OUTDEC(LINE,IP,KK-IX)
	I=J
C
C	Loop back when done with the range
C
	GOTO2
C
C	When done with everything, back up the pointer to eliminate the last
C	comma that was inserted.
C
10	L=IP-2
	RETURN
	END
C
C	PARSEMSG -- routine to parse symbiont message into items
C
C	This routine takes the message received from the job controller,
C	and finds the position and length of each item within the message.
C	This information is stored in the POS and SIZE arrays, which are
C	subsequently used to retrieve item values.  This eliminates the
C	need to call SMB$READ_MESSAGE_ITEM, which copies each item's value
C	from the message buffer to another buffer.  It allows direct access
C	to item values at much lower overhead.
C
	SUBROUTINE PARSEMSG(MSG,POS,SIZE,MSGLEN)
	INCLUDE 'SMBDEF.INC'
	CHARACTER*(*) MSG
	INTEGER*2 POS(SMBMSG$K_MAX_ITEM_CODE-1)
	INTEGER*2 SIZE(SMBMSG$K_MAX_ITEM_CODE-1)
	INTEGER*2 WORD,IPTR
C
C	Initialize the position and size values for all items to zero
C
	DO I=1,SMBMSG$K_MAX_ITEM_CODE-1
	   POS(I)=0
	   SIZE(I)=0
	ENDDO
C
C	Point to the byte past the request header
C
	IPTR=SMBMSG$S_REQUEST_HEADER+1
C
C	Loop:  get the size and item code for the next item in the message
C
1	ISIZE=WORD(MSG(IPTR+SMBMSG$W_ITEM_SIZE:IPTR+SMBMSG$W_ITEM_SIZE+1))
	ICODE=WORD(MSG(IPTR+SMBMSG$W_ITEM_CODE:IPTR+SMBMSG$W_ITEM_CODE+1))
C
C	If the item code is zero, it's the message terminator, so calculate
C	the actual length of the message (for use in copying the message
C	to a stream's SCB, to avoid copying blank-padding at the end), and
C	return.
C
	IF(ICODE.EQ.0)THEN
	   MSGLEN=IPTR-1
	   RETURN
	ENDIF
C
C	Advance the pointer by the size of the item header
C
	IPTR=IPTR+SMBMSG$S_ITEM_HEADER
C
C	Store the pointer in the item's POS slot, and the size in the SIZE slot
C
	POS(ICODE)=IPTR
	SIZE(ICODE)=ISIZE
C
C	Advance the pointer by the item size, and loop.
C
	IPTR=IPTR+ISIZE
	GOTO1
	END
C
C	LONG -- function to return the longword equivalent of a four-byte string
C
	INTEGER*4 FUNCTION LONG(STRING)
	CHARACTER*(*) STRING
	CHARACTER*4 CTEMP
	BYTE BTEMP(4)
	INTEGER*4 LTEMP
	EQUIVALENCE (CTEMP,BTEMP),(BTEMP,LTEMP)
	CTEMP=STRING(1:4)
	LONG=LTEMP
	RETURN
	END
C
C	WORD -- function to return the word equivalent of a two-byte string
C
	INTEGER*2 FUNCTION WORD(STRING)
	CHARACTER*(*) STRING
	CHARACTER*2 CTEMP
	BYTE BTEMP(2)
	INTEGER*2 ITEMP
	EQUIVALENCE (CTEMP,BTEMP),(BTEMP,ITEMP)
	CTEMP=STRING(1:2)
	WORD=ITEMP
	RETURN
	END
C
C	BYTE -- function to return the byte equivalent of a one-byte string
C	(Yes, I know that ICHAR does something similar, but this one returns
C	a BYTE result, not a longword.)
C
	BYTE FUNCTION BYTE(STRING)
	CHARACTER*(*) STRING
	BYTE=ICHAR(STRING(1:1))
	RETURN
	END
C
C	DEBUG_MODE_COMMAND -- routine to process debugging commands passed
C	through the status mailbox (STAT_W_MBX)
C
	SUBROUTINE DEBUG_MODE_COMMAND(CMD)
	INCLUDE 'EXECSYMB.INC'
	CHARACTER*4 CMD
	CHARACTER*280 LINE
	CHARACTER*14 FILLER
	CHARACTER*1 SYMB_INDEX
	INTEGER*4 LONG
	INTEGER*2 WORD
	BYTE BYTE
	COMMON /PRCNAM/ FILLER,SYMB_INDEX
C
C	Symbiont item definitions
C
	CHARACTER*21 SMBITEM(SMBMSG$K_MAX_ITEM_CODE-1)
	BYTE SMBITEMLEN(SMBMSG$K_MAX_ITEM_CODE-1)
	BYTE SMBITEMTYPE(SMBMSG$K_MAX_ITEM_CODE-1)
	COMMON /SMBITEMINFO/ SMBITEM,SMBITEMLEN,SMBITEMTYPE
C
C	Start of code
C
C	At present, these commands are recognized:
C
C		DUMP -- dump all stream statuses to logfile
C		EXIT -- exit immediately (aborting all streams)
C		NOOP -- do absolutely nothing
C
	IF(CMD.EQ.'EXIT')THEN
	   CALL SYS$EXIT(%VAL('2C'X))
	ELSE IF(CMD.EQ.'NOOP')THEN
	   CONTINUE
	ELSE IF(CMD.EQ.'DUMP')THEN
	   CALL LOGGIT('Dump of stream status follows',)
	   OPEN(UNIT=1,NAME=LOGDIR(1:L_LD)//':EXECSYMB'//SYMB_INDEX//
	1   '.LOG',TYPE='OLD',ACCESS='APPEND',ERR=99)
11	   DO I=0,MAX_STREAMS-1
	      WRITE(1,12,ERR=1201)I
12	      FORMAT(/'Stream number ',I2/)
1201	      CALL REQ_ASC(STREAM(I).REQUEST,LINE,L)
	      WRITE(1,1301,ERR=1202)STREAM(I).PID,LINE(1:L),STREAM(I).INDEX,
	1      STREAM(I).ITEMNO
1301	      FORMAT('pid = ',Z8.8,'  request = ',A,'  index = ',I6,
	1      '  itemno = ',I6)
1202	      WRITE(1,1302,ERR=1203)STREAM(I).MBX_CHAN,STREAM(I).IOSB,
	1      STREAM(I).STATUS
1302	      FORMAT('mbx_chan = ',Z4.4,'  iosb = ',Z8.8,'/',Z8.8,
	1      '  status = ',Z8.8)
1203	      CALL SYS$ASCTIM(L,LINE,STREAM(I).REQUEUE_TIME,)
	      WRITE(1,1303,ERR=12032)STREAM(I).ENTRY,LINE(1:L)
1303	      FORMAT('entry = ',I12,'  requeue_time = ',A)
12032	      CALL SYS$ASCTIM(L,LINE,STREAM(I).DYNAMIC_TIME,)
	      WRITE(1,13032,ERR=1204)LINE(1:L),BTEST(STREAM(I).FLAGS,
	1      FLG_V_DYNAMIC),BTEST(STREAM(I).FLAGS,FLG_V_NO_PROCESSOR),
	2      BTEST(STREAM(I).FLAGS,FLG_V_EXITING)
13032	      FORMAT('dynamic_time = ',A,'  dynamic = ',L1,
	1      '  no_processor = ',L1,'  exiting = ',L1)
1204	      WRITE(1,1304,ERR=1205)STREAM(I).DEVICE_NAME_LEN,
	1      STREAM(I).COM_FILE_LEN,STREAM(I).OURNAME_LEN
1304	      FORMAT('string lengths:  device_name = ',I6,'  com_file = ',
	1      I6,'  ourname = ',I6)
1205	      WRITE(1,1305,ERR=1206)STREAM(I).SPOOLDIR_LEN,
	1      STREAM(I).FILESPEC_LEN,STREAM(I).QUEUENAME_LEN
1305	      FORMAT('spooldir = ',I6,'  filespec = ',I6,'  queuename = ',
	1      I6)
1206	      WRITE(1,1306,ERR=1207)STREAM(I).FID
1306	      FORMAT('fileid = (',I6,',',I6,',',I6,')'/'pos, size, order:')
1207	      DO J=0,42,14
	         WRITE(1,1307,ERR=1208)'n',(J+K,K=1,14),'pos',
	1         (STREAM(I).POS(J+K),K=1,14),'size',(STREAM(I).SIZE(J+K),
	2         K=1,14),'order',(STREAM(I).ORDER(J+K),K=1,14)
1307	         FORMAT(A5,14I5)
1208	         WRITE(1,1308,ERR=1209)
1308	         FORMAT(A)
1209	         CONTINUE
	      ENDDO
	      WRITE(1,1308,ERR=1210)'valid non-null items:'
1210	      DO J=1,SMBMSG$K_MAX_ITEM_CODE-1
	         IF(STREAM(I).POS(J).GT.0.AND.STREAM(I).SIZE(J).GT.0)THEN
	            IF(SMBITEMTYPE(J).EQ.1)THEN
	               L=STREAM(I).SIZE(J)
	               LINE(1:L)=STREAM(I).MESSAGE(STREAM(I).POS(J):
	1               STREAM(I).POS(J)+L-1)
	            ELSE
	               CALL ITEMEDIT(LINE,STREAM(I).MESSAGE(STREAM(I).POS(J):
	1               STREAM(I).POS(J)+STREAM(I).SIZE(J)-1),L,
	2               STREAM(I).SIZE(J),SMBITEMTYPE(J))
	            ENDIF
	            LINE(1:L+SMBITEMLEN(J)+3)=SMBITEM(J)(1:SMBITEMLEN(J))//
	1            ' = '//LINE(1:L)
	            L=L+SMBITEMLEN(J)+3
	            IF(L.GT.132)THEN
	               WRITE(1,1308,ERR=1211)LINE(1:132)
	               IF(L.GT.264)THEN
	                  WRITE(1,1308,ERR=1211)LINE(133:264)
	                  WRITE(1,1308,ERR=1211)LINE(265:L)
	               ELSE
	                  WRITE(1,1308,ERR=1211)LINE(133:L)
	               ENDIF
	            ELSE
	               WRITE(1,1308,ERR=1211)LINE(1:L)
	            ENDIF
1211	            CONTINUE
	         ENDIF
	      ENDDO
	      WRITE(1,1308,ERR=1212)'other stream flags:'
1212	      WRITE(1,1309,ERR=1213)BTEST(STREAM(I).FLAGS,FLG_V_BINARY),
	1      BTEST(STREAM(I).FLAGS,FLG_V_DO_REQUEUE),BTEST(STREAM(I).FLAGS,
	2      FLG_V_ENTERED),BTEST(STREAM(I).FLAGS,FLG_V_PRINTQUEUE),
	3      BTEST(STREAM(I).FLAGS,FLG_V_NULL)
1309	      FORMAT('binary = ',L1,'  do_requeue = ',L1,'  entered = ',L1,
	1      '  printqueue = ',L1,'  null = ',L1)
1213	      WRITE(1,1310,ERR=1214)BTEST(STREAM(I).FLAGS,FLG_V_ABORT),
	1      BTEST(STREAM(I).FLAGS,FLG_V_CHECKPOINT),BTEST(STREAM(I).FLAGS,
	2      FLG_V_COPYALL),BTEST(STREAM(I).FLAGS,FLG_V_COPYFIRST)
1310	      FORMAT('abort = ',L1,'  checkpoint = ',L1,'  copyall = ',L1,
	1      '  copyfirst = ',L1)
1214	      CALL WRITE_DEBUG('device_name = ',STREAM(I).DEVICE_NAME,
	1      STREAM(I).DEVICE_NAME_LEN)
	      CALL WRITE_DEBUG('filespec = ',STREAM(I).FILESPEC,
	1      STREAM(I).FILESPEC_LEN)
	      CALL WRITE_DEBUG('com_file = ',STREAM(I).COM_FILE,
	1      STREAM(I).COM_FILE_LEN)
	      CALL WRITE_DEBUG('ourname = ',STREAM(I).OURNAME,
	1      STREAM(I).OURNAME_LEN)
	      CALL WRITE_DEBUG('spooldir = ',STREAM(I).SPOOLDIR,
	1      STREAM(I).SPOOLDIR_LEN)
	      CALL WRITE_DEBUG('queuename = ',STREAM(I).QUEUENAME,
	1      STREAM(I).QUEUENAME_LEN)
	      WRITE(1,1311,ERR=1221)BYTE(STREAM(I).FILEINFO(1:1)),
	1      STREAM(I).FILEINFO(2:16),
	1      (WORD(STREAM(I).FILEINFO(J:J+1)),J=17,27,2)
1311	      FORMAT('fileinfo = ',I2,' "',A,'" ',6I7)
1221	      CONTINUE
	   ENDDO
	ENDIF
	CALL LOGGIT('Dump of stream status complete',)
90	RETURN
99	OPEN(UNIT=1,NAME=LOGDIR(1:L_LD)//':EXECSYMB'//SYMB_INDEX//
	1 '.LOG',TYPE='NEW',CARRIAGECONTROL='LIST',ERR=90)
	GOTO11
	END
	SUBROUTINE WRITE_DEBUG(STR1,STR2,L2)
	CHARACTER*(*) STR1,STR2
	INTEGER*2 L1,L2
	L1=LEN(STR1)
	L=L1+L2
	IF(L.GT.132)THEN
	   WRITE(1,1,ERR=9)STR1(1:L1)//STR2(1:132-L1)
1	   FORMAT(A)
	   IF(L.GT.264)THEN
	      WRITE(1,1,ERR=9)STR2(133-L1:264-L1)
	      WRITE(1,1,ERR=9)STR2(265-L1:L2)
	   ELSE
	      WRITE(1,1,ERR=9)STR2(133-L1:L2)
	   ENDIF
	ELSE
	   WRITE(1,1,ERR=9)STR1(1:L1)//STR2(1:L2)
	ENDIF
9	RETURN
	END
C
	SUBROUTINE REQ_ASC(REQ,BUF,L)
	INTEGER*4 REQ,L
	CHARACTER*(*) BUF
	CHARACTER*12 REQS(1:7)
	INTEGER*2 LENS(1:7)
	DATA REQS/'PAUSE_TASK','RESET_STREAM','RESUME_TASK','START_STREAM',
	1 'START_TASK','STOP_STREAM','STOP_TASK'/
	DATA LENS/10,12,11,12,10,11,9/
	IF(REQ.LT.1.OR.REQ.GT.7)THEN
	   BUF(1:10)='??????????'
	   CALL SYS$FAO('%X!8XL',,BUF(1:10),%VAL(REQ))
2	   L=10
	ELSE
	   BUF(1:9)='SMBMSG$K_'
	   L=LENS(REQ)+9
	   BUF(10:L)=REQS(REQ)(1:LENS(REQ))
	ENDIF
	RETURN
	END
C
	SUBROUTINE FIXTIME(TIMSTR,L)
	CHARACTER*(*) TIMSTR
	INTEGER*4 L,IP,LL,II
	LL=LEN(TIMSTR)
	L=0
	DO IP=1,LL
	   IF(TIMSTR(IP:IP).NE.' ')GOTO1
	ENDDO
	GOTO9
1	L=L+1
	TIMSTR(L:L)=TIMSTR(IP:IP)
2	IP=IP+1
	IF(IP.GT.LL)GOTO3
	IF(IP.LE.LL-2)THEN
	   IF(TIMSTR(IP:IP+2).EQ.':00')THEN
	      L=L+1
	      TIMSTR(L:L)=':'
	      IP=IP+2
	      GOTO2
	   ENDIF
	ENDIF
	IF(IP.LE.LL-1)THEN
	   IF(TIMSTR(IP:IP+1).EQ.'00')THEN
	      IP=IP+1
	      GOTO2
	   ELSE IF(TIMSTR(IP:IP+1).EQ.':0')THEN
	      L=L+1
	      TIMSTR(L:L)=':'
	      IP=IP+1
	      GOTO2
	   ELSE IF(TIMSTR(IP:IP+1).EQ.'  ')THEN
	      GOTO2
	   ENDIF
	ENDIF
	L=L+1
	TIMSTR(L:L)=TIMSTR(IP:IP)
	GOTO2
3	IF(TIMSTR(L:L).EQ.'0')L=L-1
	IF(TIMSTR(L:L).EQ.'.')L=L-1
4	IF(TIMSTR(L:L).EQ.':')THEN
	   L=L-1
	   GOTO4
	ENDIF
9	RETURN
	END
C
	SUBROUTINE DEFINE_GROUP_LNM(LOGNAME,EQUIVNAME)
	CHARACTER*(*) LOGNAME,EQUIVNAME
	INCLUDE '($LNMDEF)'
	INCLUDE 'ITEMDEF.INC'
	INTEGER*4 SYS$CRELNM,ISTAT
	RECORD /ITEM/ LNM_LIST(2)
	LNM_LIST(1).BUFLEN=LEN(EQUIVNAME)
	LNM_LIST(1).ITEMCODE=LNM$_STRING
	LNM_LIST(1).BUFADR=%LOC(EQUIVNAME)
	LNM_LIST(1).RETLENADR=0
	LNM_LIST(2).TERMINATOR=0
	ISTAT=SYS$CRELNM(,'LNM$GROUP',LOGNAME,,LNM_LIST)
	CALL LOGGIT('DEFINE_GROUP_LNM SYS$CRELNM status = ',ISTAT)
	RETURN
	END
C
C	Subroutine to start processor
C
C	(code taken from MSG_AST code)
C
	SUBROUTINE START_PROCESSOR(NSTREAM,ISTAT_CREPRC)
C
C	First, make implicit declaration and specify INCLUDE files
C
	IMPLICIT INTEGER*4 (A-Z)
	INCLUDE 'EXECSYMB.INC'
	INCLUDE '($PRCDEF)'
	INCLUDE '($SSDEF)'
C
C	Declare arguments
C
	INTEGER*4 NSTREAM,ISTAT_CREPRC
C
C	System services called as functions
C
	INTEGER*4 SYS$CREPRC,SYS$CREMBX,SYS$WAKE
C
C	Process creation flags for processors
C
	INTEGER*4 PRC_FLAGS
	PARAMETER (PRC_FLAGS=PRC$M_NOUAF+PRC$M_DETACH+PRC$M_HIBER)
C
C	Process name (need last character only)
C
	CHARACTER*14 FILLER
	CHARACTER*1 SYMB_INDEX
	COMMON /PRCNAM/ FILLER,SYMB_INDEX
	CHARACTER*15 PROCESSOR_NAME
	INTEGER*2 LPN
	CHARACTER*20 LNM_SUFFIX
	INTEGER*2 L_LNM_SUF
C
C	Local variables
C          
	INTEGER*4 ISTAT
C
	CALL SETAST(%REF(0))
	STREAM(NSTREAM).IOSB(1)=1
	LPN=MIN(12,STREAM(NSTREAM).OURNAME_LEN)+3
	PROCESSOR_NAME=SYMB_INDEX//'__'//STREAM(NSTREAM).OURNAME
	IF(NSTREAM.LE.9)THEN
	   PROCESSOR_NAME(2:2)=CHAR(NSTREAM+'30'X)
	ELSE
	   PROCESSOR_NAME(2:2)=CHAR(NSTREAM-10+'41'X)
	ENDIF
C
C	If it's a permanent processor, skip the process creation stuff
C
  	IF(BTEST(STREAM(NSTREAM).FLAGS,FLG_V_PERMANENT))THEN
	   L_LNM_SUF=STREAM(NSTREAM).PERMNAME_LEN
	   LNM_SUFFIX(1:L_LNM_SUF)=STREAM(NSTREAM).PERMNAME(1:L_LNM_SUF)
	   ISTAT_CREPRC=1
	   GOTO2
	ENDIF
	IF(STREAM(NSTREAM).USERNAME.NE.'SYSTEM')THEN
	   ISTAT=LIB_SET_USERNAME(STREAM(NSTREAM).USERNAME)
	   CALL LOGGIT('LIB_SET_USERNAME status = ',ISTAT)
	ENDIF
1	ISTAT_CREPRC=SYS$CREPRC(STREAM(NSTREAM).PID,
	1 'SYS$SYSTEM:LOGINOUT.EXE',
	1 STREAM(NSTREAM).COM_FILE(1:STREAM(NSTREAM).COM_FILE_LEN),
	1 LOGDIR(1:L_LD)//':'//
	1 STREAM(NSTREAM).OURNAME(1:STREAM(NSTREAM).OURNAME_LEN)//'.OUT',
	1 LOGDIR(1:L_LD)//':'//
	1 STREAM(NSTREAM).OURNAME(1:STREAM(NSTREAM).OURNAME_LEN)//'.ERR',,,
	1 PROCESSOR_NAME(1:LPN),%VAL(BASE_PRIO),,%VAL(EXIT_MBX_UNIT),
	1 %VAL(PRC_FLAGS))
	CALL LOGGIT('$CREPRC status = ',ISTAT_CREPRC)
	CALL SYS$FAO('!8XL',L_LNM_SUF,LNM_SUFFIX,%VAL(STREAM(NSTREAM).PID))
2	IF((ISTAT_CREPRC.AND.7).EQ.1)THEN
	   CALL DEFINE_GROUP_LNM('STAT_W_MBX_'//LNM_SUFFIX(1:L_LNM_SUF),
	1         'STAT_W_MBX_'//SYMB_INDEX)
	   IF(BTEST(STREAM(NSTREAM).FLAGS,FLG_V_PERMANENT))THEN
	      ISTAT=SYS$ASSIGN('CMD_MBX_'//STREAM(NSTREAM).PERMNAME(1:
	1      STREAM(NSTREAM).PERMNAME_LEN),STREAM(NSTREAM).MBX_CHAN,,)
	      CALL LOGGIT('Permanent CMD_MBX assign status = ',ISTAT)
	      IF((ISTAT.AND.1).EQ.0)THEN
	         ISTAT_CREPRC=ISTAT
	      ELSE
	         STREAM(NSTREAM).PID='FFFFFFFF'X
	         IF(BTEST(STREAM(NSTREAM).FLAGS,FLG_V_BINARY))THEN
	            CALL WRITE_MBX_BINARY(NSTREAM,%REF(0),
	1            'ID!'//PROCESSOR_NAME(2:2))
	         ELSE
	            CALL WRITE_MBX_ASCII(NSTREAM,'EXEC_STEP','ID!'//
	1            PROCESSOR_NAME(2:2))
	         ENDIF
	      ENDIF
	   ELSE
	      CALL DEFINE_GROUP_LNM('QUEUE_NAME_'//LNM_SUFFIX(1:L_LNM_SUF),
	1      STREAM(NSTREAM).OURNAME(1:STREAM(NSTREAM).OURNAME_LEN))
	      ISTAT=SYS$CREMBX(,%REF(STREAM(NSTREAM).MBX_CHAN),%VAL(260),
	1      %VAL(520),%VAL('F000'X),,
	2      %DESCR('CMD_MBX_'//LNM_SUFFIX(1:L_LNM_SUF)))
	      CALL LOGGIT('CMD_MBX $CREMBX status = ',ISTAT)
	      ISTAT=SYS$WAKE(%REF(STREAM(NSTREAM).PID),)
	      CALL LOGGIT('SYS$WAKE processor status = ',ISTAT)
	      CALL TELL_OPER('Starting process '//PROCESSOR_NAME(1:LPN)//
	1      ' for queue '//
	2      STREAM(NSTREAM).OURNAME(1:STREAM(NSTREAM).OURNAME_LEN),3)
	   ENDIF
	ELSE IF(ISTAT_CREPRC.EQ.SS$_DUPLNAM)THEN
	   IF(PROCESSOR_NAME(3:3).EQ.'_')THEN
	      PROCESSOR_NAME(3:3)='A'
	      GOTO1
	   ELSE IF(PROCESSOR_NAME(3:3).EQ.'Z')THEN
	      CALL LOGGIT('Internal error -- duplicate process name '//
	1      PROCESSOR_NAME(1:LPN),)
	   ELSE
	      PROCESSOR_NAME(3:3)=CHAR(ICHAR(PROCESSOR_NAME(3:3))+1)
	      GOTO1
	   ENDIF
	ENDIF
	IF(STREAM(NSTREAM).USERNAME.NE.'SYSTEM')THEN
	   ISTAT=LIB_SET_USERNAME('SYSTEM      ')
	   CALL LOGGIT('LIB_SET_USERNAME SYSTEM status = ',ISTAT)
	ENDIF
	IF((ISTAT_CREPRC.AND.1).NE.0)STREAM(NSTREAM).FLAGS=
	1 STREAM(NSTREAM).FLAGS.AND..NOT.FLG_M_NO_PROCESSOR
	CALL SETAST(%REF(1))
	RETURN
	END
C
C	Subroutine to set timer request for stream, if processor is dynamic
C
	SUBROUTINE SET_TIMER(NSTREAM)
C
C	First, make implicit declaration and specify INCLUDE files
C
	IMPLICIT INTEGER*4 (A-Z)
	INCLUDE 'EXECSYMB.INC'
C
C	Declare arguments
C
	INTEGER*4 NSTREAM
C
C	Declare routines called as functions
C
	INTEGER*4 SYS$SETIMR
C
C	Timer AST routine
C
	EXTERNAL DYN_TIMER_AST
C
C	Local variables
C
	INTEGER*4 ISTAT
C
	CALL SETAST(%REF(0))
	IF(BTEST(STREAM(NSTREAM).FLAGS,FLG_V_DYNAMIC))THEN
	   ISTAT=SYS$SETIMR(,STREAM(NSTREAM).DYNAMIC_TIME,DYN_TIMER_AST,
	1   %VAL(NSTREAM+1))
	   CALL LOGGIT('SYS$SETIMR status = ',ISTAT)
	ENDIF
	CALL SETAST(%REF(1))
	RETURN
	END
C
C	Dynamic processor timer AST routine
C
	SUBROUTINE DYN_TIMER_AST(ARG)
C
C	First, make implicit declaration and specify INCLUDE files
C
	IMPLICIT INTEGER*4 (A-Z)
	INCLUDE 'EXECSYMB.INC'
C
C	Declare arguments
C
	BYTE ARG
	INTEGER*4 NSTREAM
C
C	Local variables
C
	INTEGER*4 ISTAT
C
	NSTREAM=%LOC(ARG)-1
	IF(NSTREAM.LT.0.OR.NSTREAM.GE.MAX_STREAMS)THEN
	   CALL LOGGIT('Invalid stream number in DYN_TIMER_AST',)
	ELSE IF(.NOT.BTEST(STREAM(NSTREAM).FLAGS,FLG_V_DYNAMIC))THEN
	   CALL LOGGIT('DYN_TIMER_AST on static stream ',NSTREAM*256+4)
	ELSE IF(STREAM(NSTREAM).REQUEST.EQ.SMBMSG$K_START_STREAM)THEN
	   CALL SEND_EXIT_MSG(NSTREAM)
	   STREAM(NSTREAM).FLAGS=STREAM(NSTREAM).FLAGS.OR.FLG_M_NO_PROCESSOR
	ELSE
	   CALL LOGGIT('Timeout received for non-idle stream ',NSTREAM*256+3)
	ENDIF
	RETURN
	END
C
C	Subroutine to track AST enable/disable
C
	SUBROUTINE SETAST(ASTVAL)
	LOGICAL*1 ASTVAL,LIB$AST_IN_PROG
	COMMON /ASTSTUFF/ AST_DISABLE_LEVEL
	DATA AST_DISABLE_LEVEL/0/
	IF(LIB$AST_IN_PROG())RETURN
	IF(ASTVAL)THEN
	   AST_DISABLE_LEVEL=AST_DISABLE_LEVEL-1
	   IF(AST_DISABLE_LEVEL.EQ.0)CALL SYS$SETAST(%VAL(1))
	ELSE
	   IF(AST_DISABLE_LEVEL.EQ.0)CALL SYS$SETAST(%VAL(0))
	   AST_DISABLE_LEVEL=AST_DISABLE_LEVEL+1
	ENDIF
	RETURN
	END
C
C	Subroutine to tell processor to exit
C
	SUBROUTINE SEND_EXIT_MSG(NSTREAM)
	INCLUDE 'EXECSYMB.INC'
	INTEGER*2 NSTREAM
	INTEGER*4 SYS$SETIMR,SYS$DASSGN
	EXTERNAL EXIT_TIMER_AST
	INTEGER*4 ISTAT
	INTEGER*4 EXIT_TIMEOUT(2)
	LOGICAL*1 INITED
	CALL SETAST(%REF(0))
	IF(.NOT.INITED)THEN
	   INITED=.TRUE.
C
C	Here's definition of exit timeout value
C
	   CALL SYS$BINTIM('0 00:01:00.00',EXIT_TIMEOUT)
	ENDIF
	IF(BTEST(STREAM(NSTREAM).FLAGS,FLG_V_BINARY))THEN
	   CALL WRITE_MBX_BIN_X(NSTREAM,0,'EXIT')
	ELSE
	   CALL WRITE_MBX_ASC_X(NSTREAM,'EXEC_STEP','EXIT')
	ENDIF
	CALL LOGGIT('Exit requested for processor ',NSTREAM*256+3)
C
C	Since "EXIT" message has been sent, no further messages can be
C	sent on this mailbox channel, so deassign the channel.
C
	IF(STREAM(NSTREAM).MBX_CHAN.NE.-1)THEN
	   ISTAT=SYS$DASSGN(%VAL(STREAM(NSTREAM).MBX_CHAN))
	   CALL LOGGIT('$DASSGN exiting processor MBX status = ',ISTAT)
	   STREAM(NSTREAM).MBX_CHAN=-1
	ENDIF
C
C	If it's a permanent processor, set the PID to zero (indicating that
C	it has "exited") and set NO_PROCESSOR flag
C
	IF(BTEST(STREAM(NSTREAM).FLAGS,FLG_V_PERMANENT))THEN
	   STREAM(NSTREAM).PID=0
	   STREAM(NSTREAM).FLAGS=STREAM(NSTREAM).FLAGS.OR.
	1   FLG_M_NO_PROCESSOR
	ENDIF
C
C	If it's a dynamic stream and not a permanent processor, set the
C	EXITING flag and exit timer
C
	IF(BTEST(STREAM(NSTREAM).FLAGS,FLG_V_DYNAMIC).AND.
	1 .NOT.BTEST(STREAM(NSTREAM).FLAGS,FLG_V_PERMANENT))THEN
	   STREAM(NSTREAM).FLAGS=STREAM(NSTREAM).FLAGS.OR.FLG_M_EXITING
	   ISTAT=SYS$SETIMR(,EXIT_TIMEOUT,EXIT_TIMER_AST,%VAL(NSTREAM+65))
	   CALL LOGGIT('SYS$SETIMR EXIT TIMER status = ',ISTAT)
	ENDIF
	CALL SETAST(%REF(1))
	RETURN
	END
C
C	Exit timeout AST routine
C
	SUBROUTINE EXIT_TIMER_AST(ARG)
C
C	First, make implicit declaration and specify INCLUDE files
C
	IMPLICIT INTEGER*4 (A-Z)
	INCLUDE 'EXECSYMB.INC'
C
C	Declare arguments
C
	BYTE ARG
	INTEGER*4 NSTREAM
C
C	Local variables
C
	INTEGER*4 ISTAT
	CHARACTER*8 PROCPID
C
	NSTREAM=%LOC(ARG)-65
	IF(NSTREAM.LT.0.OR.NSTREAM.GE.MAX_STREAMS)THEN
	   CALL LOGGIT('Invalid stream number in EXIT_TIMER_AST',)
	ELSE IF(BTEST(STREAM(NSTREAM).FLAGS,FLG_V_EXITING))THEN
	   IF(STREAM(NSTREAM).PID.NE.0)THEN
	      CALL SYS$FAO('!8XL',,PROCPID,%VAL(STREAM(NSTREAM).PID))
	      CALL LOGGIT('Dynamic processor never exited; PID '//PROCPID//
	1      ' being stopped for stream: ',NSTREAM*256+3)
	      ISTAT=SYS$DELPRC(STREAM(NSTREAM).PID,)
	      CALL LOGGIT('$DELPRC status = ',ISTAT)
	      ISTAT=SYS$DELLNM('LNM$GROUP','QUEUE_NAME_'//PROCPID,)
	      CALL LOGGIT('$DELLNM QUEUE_NAME status =',ISTAT)
	      ISTAT=SYS$DELLNM('LNM$GROUP','STAT_W_MBX_'//PROCPID,)
	      CALL LOGGIT('$DELLNM STAT_W_MBX status =',ISTAT)
	   ENDIF
C
C	Clear EXITING flag here, in case $DELPRC fails to really kill the
C	process (e.g. if process is in MWAIT); then, $CREPRC will notice
C	duplicate process name, and will increment the name for the new one.
C
	   STREAM(NSTREAM).FLAGS=STREAM(NSTREAM).FLAGS.AND..NOT.FLG_M_EXITING
	   CALL WAKEUP
	ENDIF
	RETURN
	END
