/*	trans - translate ebcdic to ascii or vise versa
**
**	Author:  Ron Reidy
**		 Access Research Corp
**		 527 Encinitas Blvd
**		 Encinitas, CA 92024
**
**	Environment:  VAX 11/750  VMS 4.4  VAX C V2.2
**
**	Program History:
**		original 7-10-86
**		updated  10-14-86	Delete output file
**					when invalid characters
**					are encountered.
**
**	Synopsis:
**		This program will translate ASCII characters
**		to EBCDIC or vise versa.  All characters are
**		translated.  Invalid characters and
**		source/destination length mismatch will cause
**		the program to abort.
*/

#include stdio
#include ssdef
#include descrip
#include libdef
#include fab
#include rmsdef

struct FAB fblock;
int rms_status;

int LIB$TRA_EBC_ASC();		/* declare System & RMS routines */
int LIB$TRA_ASC_EBC();		/* as functions returning an integer */
int SYS$ERASE();

#define MAXCHAR 256

struct transdesc {
    unsigned short dsc$w_length;               
    unsigned char dsc$b_dtype; 
    unsigned char dsc$b_class; 
    char *dsc$a_pointer; 
} ;

main(argc,argv)
int argc;
char **argv;
{
    char way[MAXCHAR];
    
    if (argc != 3) {
    	usage();
    	exit();	
    }

    strcpy(way,argv[2]);

    if (tolower(way[0]) == 'e' || tolower(way[0]) == 'a')
	trans(argv[1],way[0]);		
    else{
    	printf("incorrect option: %s\n",argv[2]);
    	usage();
    	exit();
    }
}

usage()		/* display usage message */
{        
    puts("trans - EBCDIC to ASCII or ASCII to EBCDIC conversion");
    puts("usage: trans filename A,E");
    puts("\tA  convert from EBCDIC to ASCII");
    puts("\tE  convert from ASCII to EBCDIC");
}

trans(fname,swtch)	/* translate characters */
char *fname,*swtch;
{
    struct transdesc instruct;
    struct transdesc outstruct;

    FILE *infile, *outfile, *fopen();
    int status, to_ascii, c, i, bad_char = FALSE;
    unsigned char instr[MAXCHAR], outstr[MAXCHAR];
    void init();

    if(swtch == 'e')
    	to_ascii = FALSE;
    else
    	to_ascii = TRUE;        

    instruct.dsc$b_dtype = DSC$K_DTYPE_T;
    instruct.dsc$b_class = DSC$K_CLASS_S;

    outstruct.dsc$b_dtype = DSC$K_DTYPE_T;
    outstruct.dsc$b_class = DSC$K_CLASS_S;

    if ((infile = fopen(fname,"r")) == NULL) {
	printf("\nUnable to open input file %s.",*fname);
	exit();
    }

	/* open the output file to contain variable length 
	records see chapter 18, UNIX I/O Functions and Macros, 
	of the VAX C manual for details on the use of this
	fopen() */

    if ((outfile = fopen(fname,"w","rat = cr","rfm = var")) == NULL) {
	printf("\nUnable to open output file %s.",*fname);
        fclose(infile);
	exit();
    }

    init(fname);
    
    while ((feof(infile)) == 0){
        i = 0;
        instr[i] = '\0';
        
        while((c = fgetc(infile)) != EOF && i < MAXCHAR)
            instr[i++] = c;

        instr[i++] = '\0';
        
    	instruct.dsc$w_length = outstruct.dsc$w_length = strlen(instr);
	instruct.dsc$a_pointer = instr;
	outstruct.dsc$a_pointer = outstr;

    	if(to_ascii){
            if((status = LIB$TRA_EBC_ASC(&instruct,&outstruct)) != SS$_NORMAL){
                printf("\n%s\n",status == LIB$_INVCHA ? 
                    "Invalid character read from input file." : 
                    "Source and destination string length mismatch.");
                puts("No output file created.");
                bad_char = TRUE;
                break;
            }
	    else
    		fwrite(outstruct.dsc$a_pointer,outstruct.dsc$w_length,1,outfile);
    	}
    	else{
	    if((status = LIB$TRA_ASC_EBC(&instruct,&outstruct)) != SS$_NORMAL){
		printf("\n%s\n",status == LIB$_INVCHA ? 
                    "Invalid character read from input file." : 
                    "Source and destination string length mismatch.");
                puts("No output file created.");
                bad_char = TRUE;
                break;
            }
            else{
    		fwrite(outstruct.dsc$a_pointer,outstruct.dsc$w_length,1,outfile);
    		fputc('\n',outfile);
    	    }
    	}
    }

    fclose(infile);
    fclose(outfile);
    
    if(bad_char){
        if((rms_status = SYS$ERASE(&fblock)) != RMS$_SUC)
            printf("\0x07RMS ERROR - %x\nNotify Software Department!\n",rms_status);
    }
    else
        printf("\nTranslation Completed");
}
                                                
void init(fn)	/* initialize FAB */
char *fn;
{
    fblock = cc$rms_fab;
    fblock.fab$b_fac = FAB$M_DEL | FAB$M_GET | FAB$M_PUT | FAB$M_UPD;
    fblock.fab$l_fna = fn;
    fblock.fab$b_fns = strlen(fn);
    fblock.fab$l_fop = FAB$M_CIF;
    fblock.fab$b_org = FAB$C_SEQ;
    fblock.fab$b_rat = FAB$M_CR;
    fblock.fab$b_rfm = FAB$C_RFM_DFLT;
    fblock.fab$b_shr = FAB$M_NIL;
}
