[INHERIT('SYS$LIBRARY:STARLET'),
 ENVIRONMENT('BATENV'),
 IDENT('1.02')]
 MODULE BATENV;

(*++
    Title:
	BATENV - Battelle Pascal utility routines

    Facility:
	Pascal programming support

    Abstract:
    	This module consists of various Pascal utility definitions which
    	have proved widely needed in systems programming.
    	They include Pascal definitions for the routines in BATRTL, and
    	interface routines, such as the CLI routines using
        VARYING strings.  This library should be expanded as new
    	definitions are required.
    	
    	Some effort should be made to make these definitions as complete
    	as possible.  Where string parameters are required, they have
    	in general been defined as Pascal VARYING strings. Where functions
    	return a status value, they functions have generally been declared
    	as type UNSIGNED;

    Environment:
	Native Mode. No other considerations.

    Author: 
	Gary L. Grebus, Creation date: 13-Jul-1984 22:13:11
	Battelle Columbus Labs

    Modified by:

	1.01 - Gary L. Grebus, 16-Jul-1984 15:51:00
		Added definitions of QUAD, WORD, BYTE, Natural, and Positive.

        1.02 - Don Hamparian,  6-Mar-1986 
                Make the CLI$ functions [ASYNCHRONOUS]
--*)

TYPE                                                  

    Byte = [BYTE] -128..127;
    Word = [WORD] -32768..32767;
    Quad = [QUAD,UNSAFE] RECORD
    	       L0 : UNSIGNED;
	       L1 : INTEGER;
    	       END;
    UByte = [BYTE] 0..255;
    UWord = [WORD] 0..65535;
    UQuad = [QUAD,UNSAFE] RECORD
    		L0,L1 : UNSIGNED;
    		END;

    Natural = 0..MAXINT;
    Positive = 1..MAXINT;

(* String to hold a VMS file specification *)
Filespec = VARYING[NAM$C_MAXRSS] OF CHAR;

(* Record to hold a VMS Binary time *)
Binary_Time = [QUAD,UNSAFE] RECORD
    		L0 : UNSIGNED;
    		L1 : INTEGER;
    		END;

(* Exit handler block for $DCLEXH *)
    Exit_Block = RECORD
    	Link : UNSIGNED;
	Handler_Address : UNSIGNED;
	Argument_Count : INTEGER;
	Status_Address : UNSIGNED;
	END;

(* A VMS User Identification Code *)
    UIC = RECORD
    	    Member : [POS(0)] UWORD;
    	    Group : [POS(16)] UWORD;
    	    END;

(* A BCL account number string *)
    Account_String = VARYING[10] OF CHAR;

(* Useful types for Condition Handlers *)
    Sig_Args = RECORD
	CHF$L_SIG_ARGS : INTEGER;
	CHF$L_SIG_NAME : UNSIGNED;
	CHF$L_SIG_ARG1 : ARRAY[1..100] OF UNSIGNED;
	    END;

    Mech_Args = CHF2$TYPE; (* From STARLET *)

    Cond_Value = STS$TYPE; (*  From STARLET *)

(*  Note this gets you a variable whose ADDRESS is the constant *)
(*  It is a kludge needed to access the output of the VMS message compiler *)
    External_Constant = [EXTERNAL] UNSIGNED;

(* A generic IOSB *)
    IOSB = PACKED ARRAY[0..3] OF Uword;

(* An IOSB for QIO's to a terminal class device *)
    Terminal_IOSB = RECORD
    		Status : [POS(0)] UWord;
    		Terminator_Offset : [POS(16)] UWord;
    		Terminator : [POS(32)] UWord;
    		Terminator_Size : [POS(48)] Uword;
    		END;

(* BATRTL Routines *)

(* BATRTL - Account Number Validation Routines *)
FUNCTION BAT$Get_Cur_Acc 
    (%REF Acc : PACKED ARRAY[l1..l2:INTEGER] OF UByte) : UNSIGNED;
    EXTERNAL;

FUNCTION BAT$Decode_Acc
    (%REF Acc : PACKED ARRAY[l1..u1:INTEGER] OF UByte;
     %STDESCR Acc_String : PACKED ARRAY[l2..u2:INTEGER] OF CHAR) : UNSIGNED;
     EXTERNAL;

(* Definitions of TAB to blank routines which should go into BATRTL someday *)
[EXTERNAL] PROCEDURE Wiptab
    (%REF Tab_Stops : ARRAY[l1..l2:INTEGER] OF INTEGER;
     %REF Array_Size : INTEGER);
    EXTERNAL;

[EXTERNAL] PROCEDURE Regtab
    (%REF Interval : INTEGER;
     %REF Tab_Stops : ARRAY[l1..l2:INTEGER] OF INTEGER;
     %REF Array_Size : INTEGER);
    EXTERNAL;

[EXTERNAL] PROCEDURE Ldetab 
    (%REF Rec_In : PACKED ARRAY[l1..l2:INTEGER] OF CHAR;
     %REF Len_In : INTEGER;
     %REF  Rec_Out : PACKED ARRAY[l3..l4:INTEGER] OF CHAR;
     %REF Len_Out : INTEGER;
     %REF Tab_Stops : ARRAY[l5..l6:INTEGER] OF INTEGER;
     %REF Array_Size : INTEGER);
    EXTERNAL;

(* CLI Parsing Utility Routines *)

    [ASYNCHRONOUS] FUNCTION  CLI$Present
	(%STDESCR Qualifier : [READONLY] PACKED ARRAY[l1..u1:INTEGER] OF
	 CHAR) : BOOLEAN;
	EXTERNAL;

    [ASYNCHRONOUS]FUNCTION CLI$Get_Value
	(%STDESCR Qualifier : [READONLY] PACKED ARRAY[l1..u1:INTEGER] OF CHAR;
	 %DESCR Value : VARYING[l2] OF CHAR) : INTEGER;
	EXTERNAL;

(* The following routines are included for documentation purposes.  We 
   aren't keeping the code for these anywhere, but they can be easily 
   duplicated wherever needed. *)
FUNCTION Present 
    (VAR Qualifier : [READONLY] VARYING[l1] OF CHAR) : BOOLEAN;

BEGIN
Present := CLI$Present( SUBSTR(Qualifier, 1, Qualifier.LENGTH) );
END;  (* Present*)

FUNCTION Get_Value
    	(VAR Qualifier : [READONLY] VARYING[l1] OF CHAR;
	 VAR Value : VARYING[l2] OF CHAR) : INTEGER;
BEGIN
Get_Value := CLI$Get_Value (SUBSTR(Qualifier, 1, Qualifier.LENGTH),
    				Value);
END;	(* Get_Value *)

END.
                  
