[INHERIT('SYS$LIBRARY:STARLET'),
 ENVIRONMENT('VMSRTL'),
 IDENT('1.06')]
 MODULE VMSRTL;

(*++
    Title:
	VMSRTL - Pascal definitions for VMS RTL routines

    Facility:
	Pascal programming support

    Abstract:
    	This module consists of Pascal definitions of the routines 
    	in the VAX/VMS Run-Time Library.  This is not a complete set of
    	definitions, but should be added to as new routines are required.
    	
    	Some effort should be made to make these definitions as complete
    	as possible.  Where string parameters are required by the RTL 
    	routine, they have been defined as Pascal VARYING strings (if
    	the RTL routine supports them).  Functions which return a status
    	value have been generally declared as TYPE UNSIGNED.

    Environment:
	Native Mode. No other considerations.

    Author: 
	Gary L. Grebus, Creation date: 13-Jul-1984 22:13:11
	Battelle Columbus Labs

    Modified by:

	1.01 - Gary L. Grebus, 16-Jul-1984 15:52:28
		Added QUAD, BYTE, WORD, LIB$ADDX, LIB$SUBX, LIB$EMUL

    	1.02 - Gary L. Grebus, 20-Jul-1984 14:46:52
		Make Byte, Word, etc hidden so they do not cause conflicts.

    	1.03 - Gary L. Grebus,  6-Aug-1984 17:17:45
		Fixed definition of LIB$FILE_SCAN to pass routines as %IMMED.

    	1.04 - Gary L. Grebus, 20-Jun-1985 09:28:35
		Added LIB$GETDVI.

    	1.05 - Gary L. Grebus, 12-Sep-1985 11:51:03
		Added VMS V4 flags to LIB$SPAWN_Flags.

	1.06 - Gary L. Grebus, 13-Sep-1985 10:32:58
		Added LIB$STOP.

--*)

[HIDDEN] TYPE

    Byte = [BYTE] -128..127;
    Word = [WORD] -32768..32767;
    Quad = [QUAD,UNSAFE] RECORD
    	       L0 : UNSIGNED;
	       L1 : INTEGER;
    	       END;
    UByte = [BYTE] 0..255;
    UWord = [WORD] 0..65535;
    UQuad = [QUAD,UNSAFE] RECORD
    		L0,L1 : UNSIGNED;
    		END;

    Filespec = VARYING[NAM$C_MAXRSS] OF CHAR;

(* Record to hold a VMS Binary time *)
    Binary_Time = [QUAD,UNSAFE] RECORD
    		L0 : UNSIGNED;
    		L1 : INTEGER;
    		END;

(* Flags parameter to LIB$SPAWN *)
TYPE    LIB$SPAWN_Flags = RECORD	
    	CLI$V_NOWAIT : [POS(0),BIT(1),UNSAFE]BOOLEAN;
    	CLI$V_NOCLISYM : [POS(1),BIT(1),UNSAFE]BOOLEAN;
    	CLI$V_NOLOGNAM : [POS(2),BIT(1),UNSAFE]BOOLEAN;
    	CLI$V_NOKEYPAD : [POS(3),BIT(1),UNSAFE]BOOLEAN;
        CLI$V_NOTIFY : [POS(4),BIT(1),UNSAFE]BOOLEAN;
    	CLI$V_NOCONTROL : [POS(5),BIT(27),UNSAFE]BOOLEAN;
    	END;

(* VMS Run-Time Library Routines *)

[ASYNCHRONOUS] FUNCTION LIB$GET_FOREIGN
    (%STDESCR Get_Str : PACKED ARRAY[l1..u1:INTEGER] OF CHAR;
     %STDESCR Prompt_Str : [READONLY] PACKED ARRAY[l2..u2:INTEGER] OF CHAR;
     %REF Out_Len : UWord ) : UNSIGNED;
    EXTERNAL;

[ASYNCHRONOUS] FUNCTION OTS$CVT_TZ_L
    (%STDESCR Inp_Str : PACKED ARRAY[l1..l2:INTEGER] OF CHAR;
     VAR Value : INTEGER;
     %IMMED Value_Size : INTEGER := %IMMED 0;
     %IMMED Flags : INTEGER := %IMMED 1) : UNSIGNED;
    EXTERNAL;

[EXTERNAL(LIB$SIGNAL),ASYNCHRONOUS] PROCEDURE Signal
(* Treat arguments as integers.  Caller must insure he causes correct
   parameter passing mechanism to be used *)
    (Arg : [READONLY,LIST,UNSAFE] INTEGER);
    EXTERNAL;

[ASYNCHRONOUS] PROCEDURE LIB$SIGNAL
(* Treat arguments as integers.  Caller must insure he causes correct
   parameter passing mechanism to be used *)
    (Arg : [READONLY,LIST,UNSAFE] INTEGER);
    EXTERNAL;

[ASYNCHRONOUS] PROCEDURE LIB$STOP
(* Treat arguments as integers.  Caller must insure he causes correct
   parameter passing mechanism to be used *)
    (Arg : [READONLY,LIST,UNSAFE] INTEGER);
    EXTERNAL;

[EXTERNAL,ASYNCHRONOUS] FUNCTION LIB$MATCH_COND
	(Value : [UNSAFE] INTEGER;
	 Value_List : [UNSAFE,LIST] INTEGER) : UNSIGNED;
	EXTERNAL;

[EXTERNAL] FUNCTION LIB$SPAWN
    (Command_String : VARYING[l1] OF CHAR := %IMMED 0;
     Input_File : VARYING[l2] OF CHAR := %IMMED 0;
     Output_File : VARYING[l3] OF CHAR := %IMMED 0;
     Flags : [UNSAFE] UNSIGNED := %IMMED 0;
     Process_Name : VARYING[l4] OF CHAR := %IMMED 0;
     %REF Process_ID : UNSIGNED := %IMMED 0;
     %REF Completion_Status : [VOLATILE]UNSIGNED := %IMMED 0;
     %REF Completion_EFN : [READONLY] UBYTE := %IMMED 0;
     %IMMED [UNBOUND,ASYNCHRONOUS ] PROCEDURE Completion_ASTadr := %IMMED 0;
     %IMMED Completion_ASTprm : INTEGER := %IMMED 0) : UNSIGNED;
    EXTERNAL;

    [EXTERNAL] FUNCTION LIB$DAY
    	(%REF Day_Number : INTEGER;
    	 %REF User_Time : Binary_Time := %IMMED 0;
    	 %REF Day_Time : INTEGER := %IMMED 0) : UNSIGNED;
    EXTERNAL;

[EXTERNAL] FUNCTION LIB$FIND_FILE
    (%DESCR File_Name : Filespec;
     %DESCR Result_Name : Filespec;
     %REF Context : INTEGER;
     %DESCR Default_Name : Filespec := %IMMED 0;
     %DESCR Related_Name : Filespec := %IMMED 0) : UNSIGNED;
    EXTERNAL;

(* Note that parameters to Success_Rtn and Error_Rtn are really the
   same as to  LIB$FILE_SCAN.  However, you can't express this declaration 
   in Pascal!  So, we assume only the FAB parameter to the action routines *)
[EXTERNAL,ASYNCHRONOUS] FUNCTION LIB$FILE_SCAN
    (%REF FAB : FAB$TYPE;
     %IMMED [UNBOUND]FUNCTION Success_Rtn (VAR FAB : FAB$TYPE) : UNSIGNED := %IMMED 0;
     %IMMED [UNBOUND]FUNCTION Error_Rtn (VAR FAB : FAB$TYPE) : UNSIGNED := %IMMED 0)
    	 : UNSIGNED;
    EXTERNAL;

[EXTERNAL,ASYNCHRONOUS] FUNCTION STR$TRIM
    (%DESCR Dst_Str : VARYING[l1] OF CHAR;
     %DESCR Src_Str : VARYING[l2] OF CHAR;
     %REF Out_Len : UWord := %IMMED 0) : UNSIGNED;
    EXTERNAL;

[EXTERNAL] FUNCTION LIB$SET_SYMBOL
    (%STDESCR Symbol : [READONLY] PACKED ARRAY[l1..u2:INTEGER] OF CHAR;
     %DESCR Value : [READONLY] VARYING[l2] OF CHAR;
     %REF Tbl : [READONLY] INTEGER := %IMMED 0) : UNSIGNED;
    EXTERNAL;

[EXTERNAL] FUNCTION LIB$DO_COMMAND
    (%DESCR Cmd_Text : [READONLY] VARYING[l1] OF CHAR) : UNSIGNED;
    EXTERNAL;

[EXTERNAL,ASYNCHRONOUS] FUNCTION STR$COMPARE
(* STR$COMPARE interface for VARYING CHAR data *)
    (%DESCR Src1_Str : VARYING[l1] OF CHAR;
     %DESCR Src2_Str : VARYING[l2] OF CHAR) : UNSIGNED;
    EXTERNAL;

[EXTERNAL] FUNCTION STR$REPLACE
(* STR$REPLACE interface for VARYING CHAR data *)
    (%DESCR Dst_Str : VARYING[l1] OF CHAR;
     %DESCR Src_Str : [READONLY] VARYING[l2] OF CHAR;
     %REF Start_Pos : [READONLY] INTEGER;
     %REF End_Pos : [READONLY] INTEGER;
     %DESCR Rpl_Str : [READONLY] VARYING[l3] OF CHAR) : UNSIGNED;
    EXTERNAL;

[EXTERNAL] FUNCTION LIB$EMUL
    (%REF Multiplier : [READONLY] INTEGER;
     %REF Multiplicand : [READONLY] INTEGER;
     %REF Addend : [READONLY] INTEGER;
     %REF Product :  Quad) : UNSIGNED;
    EXTERNAL;

[EXTERNAL] FUNCTION LIB$ADDX
    (%REF A : [READONLY,UNSAFE] INTEGER;
     %REF B : [READONLY,UNSAFE] INTEGER;
     %REF Result : [UNSAFE] INTEGER;
     %REF Len : INTEGER := %REF 2) : UNSIGNED;
    EXTERNAL;

[EXTERNAL] FUNCTION LIB$SUBX
    (%REF A : [READONLY,UNSAFE] INTEGER;
     %REF B : [READONLY,UNSAFE] INTEGER;
     %REF Result : [UNSAFE] INTEGER;
     %REF Len : INTEGER := %REF 2) : UNSIGNED;
    EXTERNAL;

[EXTERNAL,ASYNCHRONOUS] FUNCTION LIB$GETDVI
    (%REF Item_Code : [READONLY,UNSAFE] INTEGER;
     %REF Channel : [READONLY,UNSAFE] Word := %IMMED 0;
     %DESCR Device_Name : [READONLY] VARYING[l1] OF CHAR := %IMMED 0;
     %REF Out_Value : [WRITEONLY,UNSAFE] UNSIGNED := %IMMED 0;
     %DESCR Out_String : [WRITEONLY] VARYING[l2] OF CHAR := %IMMED 0;
     %REF Out_Len : [WRITEONLY] Word := %IMMED 0) : UNSIGNED;
    EXTERNAL;

END.
