	INTEGER*4 FUNCTION TPU$CALLUSER 
	2        (DISPATCH_INTEGER, INPUT_STRING, OUTPUT_STRING)
C++
C
C Title:
C	TPUCALLUSER.FOR - TPU callable user routines.
C
C Version:
C	1.0
C
C Facility:
C	TPU.
C
C Abstract:
C	This module contains routines that can be accessed via the
C	CALL_USER command in TPU. Note that this first routine just
C	dispatches to a subroutine, based on the integer value that
C	is passed.
C
C Environment:
C	Shareable image.
C
C Author:
C	Mark Oakley	Battelle Memorial Institute	16-Dec-1986
C
C Modified:
C
C 04-Feb-1987	Mark Oakley	Added routines to do SET TERMINAL/PASTHRU
C				and SET TERMINAL/NOPASTHRU.
C	
C
C--

	STRUCTURE /DESC/
	  INTEGER*2	LEN
	  BYTE		TYPE
	  BYTE		CLASS
	  INTEGER*4	ADDR
	  ENDSTRUCTURE
C
C  Arguments passed to TPU
	INTEGER*4	DISPATCH_INTEGER	! Arg 1 (we dispatch on this)
	CHARACTER	INPUT_STRING*(*)	! Arg 2 (fixed-len string)
	RECORD /DESC/	OUTPUT_STRING		! Arg 3 (dynamic-len string)

	INTEGER*4	STR$COPY_DX		! RTL routine. Note that we
						! MUST use this for the 
						! OUTPUT_STRING, because it
						! is passed by via DYNAMIC
						! descriptor!
	INTEGER*4	STATUS
	CHARACTER	MSG*80			! Message buffer.
	INTEGER*4	SET_DEVICE_DEFAULT	! Change device default rtn.
 	INTEGER*4	SET_DIRECTORY_DEFAULT	! Change directory default rtn.
	INTEGER*4	SET_TERMINAL_PASTHRU	! $ SET TERMINAL/PASTHRU
	INTEGER*4	SET_TERMINAL_NO_PASTHRU	! $ SET TERMINAL/NOPASTHRU
	INTEGER*4	XOR_RECORD		! Encode a text record.

	GO TO (100, 200, 300, 400, 500, 600), DISPATCH_INTEGER

	MSG = 'Dispatch value out of range'
 	STATUS = STR$COPY_DX (OUTPUT_STRING, 'Dispatch value out of range')
	TPU$CALLUSER = 1
	RETURN

100	TPU$CALLUSER = SET_DEVICE_DEFAULT (INPUT_STRING, OUTPUT_STRING)
	RETURN

200	TPU$CALLUSER = SET_DIRECTORY_DEFAULT (INPUT_STRING, OUTPUT_STRING)
	RETURN

300	TPU$CALLUSER = SET_TERMINAL_PASTHRU (OUTPUT_STRING)
	RETURN

400	TPU$CALLUSER = SET_TERMINAL_NO_PASTHRU (OUTPUT_STRING)
	RETURN

500	TPU$CALLUSER = XOR_RECORD (INPUT_STRING, OUTPUT_STRING)
	RETURN

600	CONTINUE
	RETURN
	END

	INTEGER*4 FUNCTION SET_DEVICE_DEFAULT (DEV_SPEC, RETURN_STRING)
C++
C
C  Functional Description:
C	This routine changes the default device of the TPU process. The
C	is changed by re-defining the SYS$DISK logical. 
C
C  Calling Sequence:
C	Status = SET_DEFAULT (DEV_SPEC)
C
C  Input Parameters:
C	DEV_SPEC  - Character string of the device set default to.
C	
C  Output Parameters:
C	RETURN_STRING - Status message if operations fails.
C
C  Implicit Inputs:
C	SYS$DISK logical.
C
C  Implicit Outputs:
C	SYS$DISK logical.
C
C  Procedures called:
C 	Various VMS and RMS system services.
C
C  Completion Status:
C 	Returned in R0 (SET_DEVICE_DEFAULT).
C
C  Side Effects:
C	SYS$DISK is changed.
C
C--

	INCLUDE		'($SYSSRVNAM)/NOLIST'	! System service defs.
	INTEGER*4	SYS$SETDDIR		! Set default function.
	INCLUDE		'($LNMDEF)/NOLIST'	! LNM defs.
	INCLUDE		'($PSLDEF)/NOLIST'	! PSL defs.
	INCLUDE		'($SSDEF)/NOLIST'	! Termination defs.

	STRUCTURE /DESC/
	  INTEGER*2	LEN
	  BYTE		TYPE
	  BYTE		CLASS
	  INTEGER*4	ADDR
	  ENDSTRUCTURE
	RECORD	/DESC/	RETURN_STRING

	INTEGER*4	STATUS			! Return status.
	INTEGER*4	IOSB(2)			! More return status.
	INTEGER*4	DEV_DESC(2)		! Device name descriptor.
	INTEGER*4	DIR_DESC(2)		! Directory name descriptor.

	CHARACTER	DEV_SPEC*(*)		! Dev/dir spec.

	STRUCTURE /ITEM_LIST/			! Item list structure for
		INTEGER*2	BUFF		! creating/assigning a
		INTEGER*2	CODE		! logical name.
		INTEGER*4	ADDR
		INTEGER*4	RET
		INTEGER*4	END
		ENDSTRUCTURE

	RECORD	/ITEM_LIST/ DEV_ITMLST

	DEV_ITMLST.BUFF = LEN (DEV_SPEC)	! Construct item list for
	DEV_ITMLST.CODE = LNM$_STRING		! device name.
	DEV_ITMLST.ADDR = %LOC (DEV_SPEC)                       

C
C Change the device.
	STATUS = SYS$CRELNM (, 'LNM$PROCESS', 'SYS$DISK', PSL$C_SUPER,
	2                    DEV_ITMLST)
	IF (.NOT. STATUS) THEN
	  DUM = STR$COPY_DX (RETURN_STRING, 'Unable to change device spec')
	  SET_DEFAULT = STATUS
	  RETURN
	  ENDIF

	SET_DEVICE_DEFAULT = SS$_NORMAL

	RETURN
	END

	INTEGER*4 FUNCTION SET_DIRECTORY_DEFAULT (DIR_SPEC, RETURN_STRING)
C++
C
C  Functional Description:
C	This routine changes the default device and/or directory of the
C	TPU process. The default device is changed by re-defining the
C	SYS$DISK logical. The default directory is changed via a call to
C	the VMS system service SYS$SETDDIR.
C
C  Calling Sequence:
C	Status = SET_DEFAULT (DIR_SPEC)
C
C  Input Parameters:
C	DIR_SPEC  - Character string of the directory to set default to.
C	
C  Output Parameters:
C	RETURN_STRING - Status message if operations fails.
C
C  Implicit Inputs:
C	None.
C
C  Implicit Outputs:
C	None.
C
C  Procedures called:
C 	Various VMS and RMS system services.
C
C  Completion Status:
C 	Returned in R0 (SET_DIRECTORY_DEFAULT).
C
C  Side Effects:
C	Default directory is changed.
C
C--

	INCLUDE		'($SYSSRVNAM)/NOLIST'	! System service defs.
	INTEGER*4	SYS$SETDDIR		! Set default function.
	INCLUDE		'($SSDEF)/NOLIST'	! Termination defs.

	INTEGER*4	STATUS			! Return status.

	CHARACTER	DIR_SPEC*(*)		! Dev/dir spec.

	STRUCTURE /DESC/
	  INTEGER*2	LEN
	  BYTE		TYPE
	  BYTE		CLASS
	  INTEGER*4	ADDR
	  ENDSTRUCTURE
	RECORD	/DESC/	RETURN_STRING


C
C Change the directory.
	STATUS = SYS$SETDDIR (DIR_SPEC, ,)
	IF (.NOT. STATUS) THEN
	  DUM = STR$COPY_DX (RETURN_STRING, 'Unable to change directory spec')
	  SET_DEFAULT = STATUS
	  RETURN
	  ENDIF

	SET_DIRECTORY_DEFAULT = SS$_NORMAL

	RETURN
	END

	INTEGER*4 FUNCTION SET_TERMINAL_PASTHRU (RETURN_STRING)
C++
C
C  Functional Description:
C	This procedure sets the terminal "pass thru" mode. All terminal
C	input (including control-y, escape, etc.) will be passed to the
C	application.
C
C  Calling Sequence:
C	STATUS = SET_TERMINAL_PASTHRU ()
C
C  Input Parameters:
C	None.
C	
C  Output Parameters:
C	RETURN_STRING - Status message if operations fails.
C
C  Implicit Inputs:
C	Terminal characteristics buffer.
C
C  Implicit Outputs:
C	Terminal characteristics buffer.
C
C  Procedures called:
C 	SYS$ASSIGN, SYS$QIOW
C
C  Completion Status:
C 	Returned in R0.
C
C  Side Effects:
C	Terminal is in pass thru mode.
C
C--

	INCLUDE		'($SYSSRVNAM)/NOLIST'
	INCLUDE		'($IODEF)/NOLIST'
	INCLUDE		'($SSDEF)/NOLIST'
	INCLUDE		'($TT2DEF)/NOLIST'

	INTEGER*4	STATUS
	INTEGER*4	IOSB(2)

	STRUCTURE /DESC/
	  INTEGER*2	LEN
	  BYTE		TYPE
	  BYTE		CLASS
	  INTEGER*4	ADDR
	  ENDSTRUCTURE
	RECORD	/DESC/	RETURN_STRING

	INTEGER*4	CHANNEL
	INTEGER*4	CHARACTERISTICS(3)

C
C  Get a channel to the terminal to sense the characteristics.
C
	STATUS = SYS$ASSIGN ('SYS$COMMAND', CHANNEL, ,)
	IF (.NOT. STATUS) THEN
	  DUM = STR$COPY_DX (RETURN_STRING, 'Unable to assign channel')
	  SET_TERMINAL_PASTHRU = STATUS
	  RETURN
	  ENDIF
	
	STATUS = SYS$QIOW ( ,
	2                  %VAL(CHANNEL),
	3                  %VAL(IO$_SENSEMODE),
	4                  IOSB,
	5                   ,
 	6                   ,
	1                  CHARACTERISTICS,
	2                  %VAL (12),
 	3                   ,
	4                   ,
	5                   ,
	6                   )

	IF (STATUS) STATUS = IOSB(1)
	IF (.NOT. STATUS) THEN
	  DUM = STR$COPY_DX (RETURN_STRING, '$QIOW sense-mode failed.')
	  SET_TERMINAL = STATUS
	  RETURN
	  ENDIF

C
C  Enable pass thru in the characteristics buffer.
C
	CHARACTERISTICS(3) = CHARACTERISTICS(3) .OR. TT2$M_PASTHRU

C
C  Set the terminal to pass thru, and get rid of the channel.
C
	STATUS = SYS$QIOW ( ,
	2                  %VAL(CHANNEL),
	3                  %VAL(IO$_SETMODE),
	4                  IOSB,
	5                   ,
	6                   ,
	1                  CHARACTERISTICS,
	2                  %VAL (12),
	3                   ,
	4                   ,
	5                   ,
	6                   )

	IF (STATUS) STATUS = IOSB(1)
	IF (.NOT. STATUS) THEN
	  DUM = STR$COPY_DX (RETURN_STRING, '$QIOW set-mode failed.')
	  SET_TERMINAL_PASTHRU = STATUS
	  RETURN
	  ENDIF

	STATUS = SYS$DASSGN (%VAL (CHANNEL))
	IF (.NOT. STATUS) THEN
	  DUM = STR$COPY_DX (RETURN_STRING, '$DASSGN failed.')
	  SET_TERMINAL_PASTHRU = STATUS
	  RETURN
	  ENDIF

	SET_TERMINAL_PASTHRU = STATUS

	RETURN
	END

	INTEGER*4 FUNCTION SET_TERMINAL_NO_PASTHRU (RETURN_STRING)
C++
C
C  Functional Description:
C	This procedure sets the terminal "no pass thru" mode. The terminal
C	driver will process control-y, escape sequences, etc., after this
C	routine executes.
C
C  Calling Sequence:
C	STATUS = SET_TERMINAL_NO_PASTHRU ()
C
C  Input Parameters:
C	None.
C	
C  Output Parameters:
C	RETURN_STRING - Status message if operations fails.
C
C  Implicit Inputs:
C	Terminal characteristics buffer.
C
C  Implicit Outputs:
C	Terminal characteristics buffer.
C
C  Procedures called:
C 	SYS$ASSIGN, SYS$QIOW
C
C  Completion Status:
C 	Returned in R0.
C
C  Side Effects:
C	Terminal is in no pass thru mode.
C
C--

	INCLUDE		'($SYSSRVNAM)/NOLIST'
	INCLUDE		'($IODEF)/NOLIST'
	INCLUDE		'($SSDEF)/NOLIST'
	INCLUDE		'($TT2DEF)/NOLIST'

	INTEGER*4	STATUS
	INTEGER*4	IOSB(2)

	STRUCTURE /DESC/
	  INTEGER*2	LEN
	  BYTE		TYPE
	  BYTE		CLASS
	  INTEGER*4	ADDR
	  ENDSTRUCTURE
	RECORD	/DESC/	RETURN_STRING

	INTEGER*4	CHANNEL
	INTEGER*4	CHARACTERISTICS(3)

C
C  Get a channel to the terminal to sense the characteristics.
C
	STATUS = SYS$ASSIGN ('SYS$COMMAND', CHANNEL, ,)
	IF (.NOT. STATUS) THEN
	  DUM = STR$COPY_DX (RETURN_STRING, 'Unable to assign channel')
	  SET_TERMINAL_PASTHRU = STATUS
	  RETURN
	  ENDIF
	
	STATUS = SYS$QIOW ( ,
	2                  %VAL(CHANNEL),
	3                  %VAL(IO$_SENSEMODE),
	4                  IOSB,
	5                   ,
 	6                   ,
	1                  CHARACTERISTICS,
	2                  %VAL (12),
 	3                   ,
	4                   ,
	5                   ,
	6                   )

	IF (STATUS) STATUS = IOSB(1)
	IF (.NOT. STATUS) THEN
	  DUM = STR$COPY_DX (RETURN_STRING, '$QIOW sense-mode failed.')
	  SET_TERMINAL = STATUS
	  RETURN
	  ENDIF

C
C  Disable pass thru in the characteristics buffer.
C
	CHARACTERISTICS(3) = CHARACTERISTICS(3) .AND. (.NOT. TT2$M_PASTHRU)

C
C  Set the terminal to no pass thru, and get rid of the channel.
C
	STATUS = SYS$QIOW ( ,
	2                  %VAL(CHANNEL),
	3                  %VAL(IO$_SETMODE),
	4                  IOSB,
	5                   ,
	6                   ,
	1                  CHARACTERISTICS,
	2                  %VAL (12),
	3                   ,
	4                   ,
	5                   ,
	6                   )

	IF (STATUS) STATUS = IOSB(1)
	IF (.NOT. STATUS) THEN
	  DUM = STR$COPY_DX (RETURN_STRING, '$QIOW set-mode failed.')
	  SET_TERMINAL_PASTHRU = STATUS
	  RETURN
	  ENDIF

	STATUS = SYS$DASSGN (%VAL (CHANNEL))
	IF (.NOT. STATUS) THEN
	  DUM = STR$COPY_DX (RETURN_STRING, '$DASSGN failed.')
	  SET_TERMINAL_PASTHRU = STATUS
	  RETURN
	  ENDIF

	SET_TERMINAL_PASTHRU = STATUS

	RETURN
	END

	INTEGER*4 FUNCTION XOR_RECORD (RECORD, ENCODED_RECORD)
C++
C
C  Functional Description:
C	This routine encodes a record by performing an exclusive
C	OR against a key, and returns the encoded record. 
C
C  Calling Sequence:
C	status = XOR_RECORD (record, encoded_record)
C
C  Input Parameters:
C	RECORD - Input record to be encoded. This variable is actually
C	         made up of 4 fields:
C
C			Bytes 01-02  Length of encoding key
C			Bytes 03-04  Pointer to characters in key
C			Bytes 03-nn  Encoding key
C			Bytes nn-mm  Data record to encode
C	
C  Output Parameters:
C	ENCODED_RECORD - Result of applying encoding key to data record.
C
C  Implicit Inputs:
C	None.
C
C  Implicit Outputs:
C	None.
C
C  Procedures called:
C 	None.
C
C  Completion Status:
C 	Returned in R0.
C
C  Side Effects:
C	Dynamic string allocated.
C
C--
	CHARACTER	RECORD*(*)

	CHARACTER	DATA_RECORD*512
	BYTE		DATA_BYTE(512)
	EQUIVALENCE	(DATA_RECORD, DATA_BYTE)

	CHARACTER	ENCODING_KEY*255
	BYTE		KEY_BYTE(255)
	EQUIVALENCE	(ENCODING_KEY, KEY_BYTE)

	CHARACTER	KEY_PTR_STR*2
	INTEGER*4	KEY_LEN, KEY_PTR, KEY_VAL
	INTEGER*4	DATA_LEN, DATA_PTR, DATA_VAL
	INTEGER*4	CODED_VAL
	INTEGER*4	STATUS

	INTEGER*4	STR$COPY_DX		! RTL routine. Note that we
						! MUST use this for the 
						! OUTPUT_STRING, because it
						! is passed by via DYNAMIC
						! descriptor!
	STRUCTURE /DESC/
	  INTEGER*2	LEN
	  BYTE		TYPE
	  BYTE		CLASS
	  INTEGER*4	ADDR
	  ENDSTRUCTURE
	RECORD	/DESC/  ENCODED_RECORD

C
C  Get the length of the encoding key and the encoding key.
C
	READ (RECORD, FMT='(2I2)') KEY_LEN, KEY_PTR
	ENCODING_KEY(1:KEY_LEN) = RECORD(5:KEY_LEN+4)
C
C  Get the length of the data part of the record, and the
C  actual data.
C
	DATA_LEN = LEN(RECORD) - 4 - KEY_LEN
	DATA_RECORD(1:DATA_LEN) = RECORD(5+KEY_LEN:)

C
C  Loop through the encoding key and the data record and encode
C  the data record.
C
	DATA_PTR = 1
	DO WHILE (DATA_PTR .LE. DATA_LEN)
C
C  Use the numeric values of the characters in the data record
C  and encoding key.
C
		DATA_BYTE(DATA_PTR) = DATA_BYTE(DATA_PTR) .XOR. 
	2	                      KEY_BYTE(KEY_PTR)
		DATA_PTR = DATA_PTR + 1
		KEY_PTR = KEY_PTR + 1
		IF (KEY_PTR .GT. KEY_LEN) KEY_PTR = 1
		ENDDO
C
C  Remember our place in the encoding key, and return the
C  encoded record.
C
	WRITE (KEY_PTR_STR, '(I2)') KEY_PTR
	STATUS = STR$COPY_DX (ENCODED_RECORD,
	2                     KEY_PTR_STR //DATA_RECORD(1:DATA_LEN))

	XOR_RECORD = 1
	RETURN
	END
