	INTEGER*4 FUNCTION Lib_Copy_File(Infile, Outfile)

C++
c
C Author: Jonathan Welch
c
C Functional Description:
c
c	Copies one file to another.  Currently works for sequential files only.
c
C Formal Arguments:
c
c	Infile
c	  VMS Usage  : char_string
c	  type       : character string
c	  access     : read only
c	  mechanism  : by descriptor
c
c	The name of the input file to be copied.
c
c	Outfile
c	  VMS Usage  : char_string
c	  type       : character string
c	  access     : read only
c	  mechanism  : by descriptor
c
c	The name of the output file to create.
c
C Implicit inputs:
c
c	none
c
C Implicit outputs:
c
c	none
c
C Completion Status:
c
c	SS$_NORMAL	Function completed successfully.
c
c	Any RMS value returned by the following RMS functions:
c
c	$CONNECT
c	$CREATE
c	$GET
c	$OPEN
c	$PUT
c
C--

	INCLUDE '($FABDEF)'
	INCLUDE '($RABDEF)'
	INCLUDE '($RMSDEF)'
	INCLUDE '($SSDEF)'

	BYTE Buffer(65535)	! Buffer to hold each record as it is read in
	BYTE Header(2)		! Holds 2 byte VFC header
	BYTE In(255)		! Name of input file
	BYTE Out(255)		! Name of output file

	CHARACTER*(*) Infile	! Passed parameters
	CHARACTER*(*) Outfile

	INTEGER*2 Isi

	INTEGER*4 Loop		! Obvious
	INTEGER*4 Sts		! Holds function return values.

	INTEGER*4 SYS$CREATE	! RMS functions.
	INTEGER*4 SYS$CONNECT
	INTEGER*4 SYS$GET
	INTEGER*4 SYS$OPEN
	INTEGER*4 SYS$PUT

	RECORD /Fabdef/ Fab_in	! Input File Access Block
	RECORD /Fabdef/ Fab_out	! Output FAB
	RECORD /Rabdef/ Rab_in	! Input Record Access Block
	RECORD /Rabdef/ Rab_out	! Output RAB

	Fab_in.FAB$B_BLN = FAB$C_BLN	! Identify Fab as a valid FAB
	Fab_in.FAB$B_BID = FAB$C_BID	!
	Fab_in.FAB$L_FNA = %LOC(In)	! File Name Address
	Fab_in.FAB$B_FNS = LEN(Infile)	! File Name Size

	Rab_in.RAB$B_BLN = RAB$C_BLN	! Identify Rab as a valid RAB
	Rab_in.RAB$B_BID = RAB$C_BID	!
	Rab_in.RAB$L_FAB = %LOC(Fab_in)	! Location of input FAB
	Rab_in.RAB$L_RHB = %LOC(Header)	! Location of VFC header
	Rab_in.RAB$L_UBF = %LOC(Buffer(1))	! Where to put an inputted record
	Rab_in.RAB$W_USZ = 65535	! max. size of a record

	DO Loop = 1, LEN(Infile)	! Make character string descriptors
	  In(Loop) = ICHAR(Infile(Loop:Loop))
	END DO				! into byte arrays.

	DO Loop = 1, LEN(Outfile)
	  Out(Loop) = ICHAR(Outfile(Loop:Loop))
	END DO

	Sts = SYS$OPEN(Fab_in)		! Open the input file
	IF(.NOT. Sts)THEN
	  Lib_Copy_File = Sts
	  RETURN			! Return on an error
	END IF

	Fab_out = Fab_in		! Make the output fab equal to the input
	Fab_out.FAB$B_FAC = FAB$M_PUT	! fab with a few exceptions.
	Fab_out.FAB$L_FNA = %LOC(Out)	! Output file name address
	Fab_out.FAB$B_FNS = LEN(Outfile)! and its size.
	Fab_out.FAB$W_IFI = 0		! A different Internal File Identifier will be created when the output file is opened.

	Sts = SYS$CREATE(Fab_out)	! Create the output file
	IF(.NOT. Sts)THEN
	  Lib_Copy_File = Sts
	  RETURN			! Return if there is an error.
	END IF

	Sts = SYS$CONNECT(Rab_in)	! Connect the input Record Access Block.
	IF(.NOT. Sts)THEN
	  Lib_Copy_File = Sts
	  RETURN			! Return if there is an error.
	END IF

	Rab_out = Rab_in		! Make the output RAB = input RAB
	Rab_out.RAB$L_FAB = %LOC(Fab_out) ! but don't mix up the FABs
	Rab_out.RAB$W_ISI = 0		! The Internal Stream Identifier will be created when the connect is performed.

	Sts = SYS$CONNECT(Rab_out)	! Connect the output RAB to its FAB.
	IF(.NOT. Sts)THEN
	  Lib_Copy_File = Sts
	  RETURN			! Return on an error.
	END IF

	Isi = Rab_out.RAB$W_Isi		! Make a local copy of the output ISI.

	Sts = SYS$GET(Rab_in)		! Get a record
	DO WHILE(Sts)			! Loop while $GET returns success.
	  Rab_out = Rab_in		! Make the output RAB = the input RAB.
	  Rab_out.RAB$L_FAB = %LOC(Fab_out) ! but keep the FAB and the
	  Rab_out.RAB$W_ISI = Isi	! Isi from being mangled.

	  Sts = SYS$PUT(Rab_out)	! Write the record to the output file.
	  IF(.NOT. Sts)THEN
	    Lib_Copy_File = Sts
	    RETURN			! Return if there is an error.
	  END IF

	  Sts = SYS$GET(Rab_in)		! Get another record and loop.
	END DO

C End Of File is expected, so convert that into a success, otherwise
c return the error code from $GET.

	IF(.NOT. Sts .AND. Sts .NE. 0 .AND. Sts .NE. RMS$_EOF)THEN
	  Lib_Copy_File = Sts
	ELSE
	  Lib_Copy_File = SS$_NORMAL
	END IF

	RETURN
	END
