	INTEGER*4 FUNCTION Lib_Fid_To_Filename(Device, Fid, Filename, Iosb)

C++
c
C Author: Jonathan Welch	Creation Date: 27-Jul-1987 12:20
c
C Functional Description:
c	The filename associated with a file ID (FID) is determined and
c	returned to the caller.
c
C Calling sequence:
c	Sts = Lib_Fid_To_Filename(Device, Fid, Filename, Iosb)
c
C Formal Arguments:
c	Device
c	  VMS Usage  : char_string
c	  type       : character string
c	  access     : read only
c	  mechanism  : by descriptor
c
c	Name of the device the file resides on.
c
c	Fid
c	  VMS Usage  : word
c	  type       : word(unsigned)
c	  access     : read only
c	  mechanism  : by reference, array reference
c
c	The File ID (FID) to generate a filename for.
c
c	Filename
c	  VMS USage  : char_string
c	  type       : character_string
c	  access     : write only
c	  mechanism  : by descriptor
c
c	The filename associated with the supplied FID.
c
c	Iosb
c	  VMS Usage  : io_status_block
c	  type       : quadword(unsigned)
c	  access     : write only
c	  mechanism  : by reference
c
c	I/O status block to receive the final completion status of the I/O
c	operation.  The iosb is the address of the quadword I/O status block.
c
C Implicit Inputs:
c
c	none
c
C Implicit Outputs:
c
c	none
c
C Completion status:
c	Any possible values from the following:
c	SYS$ASSIGN
c	SYS$DASSGN
c	SYS$QIOW
c
C Side effects:
c
c	none
C--

	INCLUDE 'CSD$LIBRARY:Attr.inc'

	INCLUDE '($ATRDEF)'
	INCLUDE '($FIBDEF)'
	INCLUDE '($IODEF)'

	BYTE Filespec(512)

	CHARACTER*(*) Device
	CHARACTER*(*) Filename

	INTEGER*2 Chan
	INTEGER*2 Fid(3)

	INTEGER*4 Iosb(2)
	INTEGER*4 Loop
	INTEGER*4 Sts

	INTEGER*4 SYS$ASSIGN
	INTEGER*4 SYS$DASSGN
	INTEGER*4 SYS$QIOW

	STRUCTURE /Dsc/
	  INTEGER*2 Length
	  BYTE Null1
	  BYTE Null2
	  INTEGER*4 Addr
	END STRUCTURE

	RECORD /Atrdef/ Atr(2)
	RECORD /Dsc/ Descr
	RECORD /Fibdef/ Fib

	Fib.FIB$W_FID_NUM = Fid(1)
	Fib.FIB$W_FID_SEQ = Fid(2)
	Fib.FIB$W_FID_RVN = Fid(3)

	Sts = SYS$ASSIGN(Device, Chan,,)
	IF(.NOT. Sts .AND. Sts .NE. 0)THEN
	  Lib_Fid_To_Filename = Sts
	  RETURN
	END IF

	Atr(1).ATR$W_SIZE = ATR$S_FILE_SPEC
	Atr(1).ATR$W_TYPE = ATR$C_FILE_SPEC
	Atr(1).ATR$L_ADDR = %LOC(Filespec)

	Atr(2).ATR$W_SIZE = 0
	Atr(2).ATR$W_TYPE = 0

	Descr.Length = FIB$C_ACCDATA
	Descr.Addr = %LOC(Fib)

	CALL LIB$MOVC5(0, 0, ICHAR(' '), 512, Filespec)

	Sts = SYS$QIOW(,%VAL(Chan),
     >                  %VAL(IO$_ACCESS),
     >                  Iosb,,,
     >                  Descr,,,,
     >                  Atr,)
	IF(.NOT. Sts .AND. Sts .NE. 0)THEN
	  Lib_Fid_To_Filename = Sts
	  RETURN
	END IF

	DO Loop = 1, MIN(510,
     >                   Filespec(1) + 256 * Filespec(2),
     >                   LEN(Filename))
	  Filename(Loop:Loop) = CHAR(Filespec(Loop + 2))
	END DO

	Lib_Fid_To_Filename = SYS$DASSGN(%VAL(Chan))

	RETURN
	END
