	INTEGER*4 Function Lib_File_LNMTable(File, Table, Stv)

C++
c
C Author: Jonathan Welch		Creation Date: 30-Jun-1987 11:45
c
c Functional Description:
c
c	The logical names and their equivalent values in the specified
c	file are read and inserted in the specified logical name table.
c	Their access mode and attributes are restored, too.
c	Note:
c	  When re-creating the logical names the saved access mode 
c	  of the logical name will be maximized with the access mode
c	  of the user.
c
c	The structure of the file is:
c	  Lognam
c	  [Equivalence_strings]
c
c	  where
c	  Equivalence_strings = 
c	    Access mode (byte), Attributes (longword), Equivalence (char_string)
c
c	Note:
c	  Logical names which are table entries will be in the
c	  file without any equivalence strings.  Currently these
c	  entries should be ignored when re-loading the file
c	  into a table.
c
C Calling Sequence:
c
c	Sts = Lib_File_LNMTable(Filename, Tablename, Stv)
c
C Formal arguments:
c
c	Filename
c	  VMS Usage  : char_string
c	  type       : character string
c	  access     : read only
c	  mechanism  : by descriptor
c
c	Specifies the name of the file the logical names in the specified
c	logical name table are to be read from.
c
c	Tablename
c	  VMS Usage  : char_string
c	  type       : character string
c	  access     : read only
c	  mechanism  : by descriptor
c
c	The name of the logical name table to insert the logical names into.
c	Logical names which are logical name table headers will be read
c	from the file and ignored.
c
c	Stv
c	  VMS Usage  : longword(unsigned)
c	  type       : unsigned longword
c	  access     : write only
c	  mechanism  : by reference
c
c	If a call to a RMS service returns an error Stv will contain the
c	secondary status value RMS$L_STV.
c
C Implicit Inputs:
c
c	none
c
C Implicit Outputs:
c
c	none
c
C Completion status:
c
c	SS$_NORMAL - Routine completed successfully.
c
c	Any values returned by the following routines:
c
c	LIB$EXTZB
c	SYS$OPEN
c	SYS$CONNECT
c	SYS$GET
c	SYS$CRELNM
c
c Condition value signaled:
c
c	SS$_ROPRAND	A reserved operand fault is signaled if a size greated
c			than 32 is specified in the LIB$INSV call.
c
C--

	INCLUDE '($FABDEF)'
	INCLUDE '($LNMDEF)'
	INCLUDE '($PSLDEF)'
	INCLUDE '($RABDEF)'
	INCLUDE '($RMSDEF)'
	INCLUDE '($SSDEF)'

	! Access mode + Attributes + Equivalence Name.
	PARAMETER Record_Size = 1 + 4 + 255

	! Buffer to hold a logical name and its equivalences.
	BYTE Buffer(128 * Record_Size)
	BYTE FName(255)		! Name of input file.

	CHARACTER*(*) File	! Passed parameter.
	CHARACTER*255 Lognam	! Logical name to create.
	CHARACTER*(*) Table	! Padded parameter.

	INTEGER*2 Lnm_Len	! Length of a string.

	INTEGER*4 Attr
	INTEGER*4 Idx
	INTEGER*4 Loop
	INTEGER*4 Loop2
	INTEGER*4 Sts
	INTEGER*4 Sts2
	INTEGER*4 Stv		! Passed parameter

	INTEGER*4 LIB$EXTZV
	INTEGER*4 SYS$CONNECT
	INTEGER*4 SYS$CRELNM
	INTEGER*4 SYS$GET
	INTEGER*4 SYS$OPEN

	LOGICAL * 1 Done

	STRUCTURE /Itmlst/
	  UNION
	    MAP
	      INTEGER*2 BufLen
	      INTEGER*2 Code
	      INTEGER*4 Addr
	      INTEGER*4 RetLen
	    END MAP
	    MAP
	      INTEGER*4 End_List
	    END MAP
	  END UNION
	END STRUCTURE

	RECORD /Fabdef/ Fab		! File Access Block
	RECORD /Itmlst/ Itemlist(128 * 2 + 1) ! Itemlist for $TRNLNM calls.
	RECORD /Rabdef/ Rab		! Record Attributes Block

	Lib_File_LNMTable = SS$_NORMAL	! Assume successful completion

	DO Loop = 1, LEN(File)
	  FName(Loop) = ICHAR(File(Loop:Loop))
	END DO

	Fab.FAB$B_BLN = FAB$C_BLN	! Identify Fab as a valid FAB
	Fab.FAB$B_BID = FAB$C_BID	!
	Fab.FAB$L_FNA = %LOC(FName)	! File Name Address
	Fab.FAB$B_FNS = LEN(File)	! File Name Size
	Fab.FAB$B_RAT = FAB$M_CR	! Carriage-return Record ATtributes
	Fab.FAB$B_RFM = FAB$C_VAR	! VARiable length records

	Rab.RAB$B_BLN = RAB$C_BLN	! Identify Rab as a valid RAB
	Rab.RAB$B_BID = RAB$C_BID	!
	Rab.RAB$L_FAB = %LOC(Fab)	! Location of input FAB
	Rab.RAB$L_UBF = %LOC(Buffer(1))	! User BuFfer
	Rab.RAB$W_USZ = Record_Size	! User SiZe of buffer.

	Sts = SYS$OPEN(Fab)		! Open the file
	IF(.NOT. Sts .AND. Sts .NE. 0)THEN
	  Lib_File_LNMTable = Sts	! Return if there is an error.
	  Stv = Fab.FAB$L_STV		! Pass back the secondary status value.
	  RETURN
	END IF

	Sts = SYS$CONNECT(Rab)		! Associate the Rab with the Fab
	IF(.NOT. Sts .AND. Sts .NE. 0)THEN
	  Lib_File_LNMTable = Sts	! Return if there is an error.
	  Stv = Fab.FAB$L_STV		! Pass back the secondary status value.
	  RETURN
	END IF

	Sts = SYS$GET(Rab)
	IF(.NOT. Sts .AND. Sts .NE. 0)THEN
	  Lib_File_LNMTable = Sts	! Return if there is an error.
	  Stv = Fab.FAB$L_STV		! Pass back the secondary status value.
	  RETURN
	END IF

	Idx = 0

	DO WHILE(Sts .NE. RMS$_EOF)

	  DO Loop = 1, Rab.RAB$W_RSZ
	    Lognam(Loop:Loop) = CHAR(Buffer(Idx * Record_Size + Loop))
	  END DO

	  Lnm_Len = Rab.RAB$W_RSZ

	  Done = .FALSE.
	  Idx = 0
	  Sts = SS$_NORMAL

	  DO WHILE(Sts .AND. Idx .LE. 127 .AND. .NOT. Done)

	    Rab.RAB$L_UBF = %LOC(Buffer(Idx * Record_Size + 1))
	    Sts = SYS$GET(Rab)
	    IF(.NOT. Sts .AND. Sts .NE. 0 .AND. Sts .NE. RMS$_EOF)THEN
	      Lib_File_LNMTable = Sts	! Return if there is an error.
	      Stv = Fab.FAB$L_STV		! Pass back the secondary status value.
	      RETURN
	    END IF

	    IF(Buffer(Idx * Record_Size + 1) .GE. PSL$C_KERNEL .AND.
     >         Buffer(Idx * Record_Size + 1) .LE. PSL$C_USER .AND.
     >         Sts .NE. RMS$_EOF)THEN

	      Attr = LIB$EXTZV(0,
     >                         32,
     >                         %VAL(%LOC(Buffer(Idx * Record_Size+2))))


	      CALL LIB$INSV(Attr .AND. (LNM$M_CONCEALED + LNM$M_TERMINAL),
     >                      0,
     >                      32,
     >                      Buffer(Idx * Record_Size + 2))

	      Itemlist(Idx * 2 + 1).Buflen = 4
	      Itemlist(Idx * 2 + 1).Code = LNM$_ATTRIBUTES
	      Itemlist(Idx * 2 + 1).Addr = %LOC(Buffer(Idx * Record_Size + 2))
	      Itemlist(Idx * 2 + 1).Retlen = 0

	      Itemlist(Idx * 2 + 2).Buflen = Rab.RAB$W_RSZ - 4 - 1
	      Itemlist(Idx * 2 + 2).Code = LNM$_STRING
	      Itemlist(Idx * 2 + 2).Addr = %LOC(Buffer(Idx * Record_Size + 6))
	      Itemlist(Idx * 2 + 2).Retlen = 0

	      Idx = Idx + 1
	    ELSE
	      Done = .TRUE.

	      Itemlist(Idx * 2 + 1).End_List = 0

	      Sts2 = SYS$CRELNM(Attr .AND. (LNM$M_CONFINE + LNM$M_NO_ALIAS),
     >                         Table,
     >                         Lognam(:Lnm_Len),
     >                         Buffer((Idx - 1) * Record_Size + 1),
     >                         Itemlist)
	      IF(.NOT. Sts2 .AND. Sts2 .NE. 0)THEN
	        Lib_File_LNMTable = Sts2
	        RETURN
	      END IF
	    END IF
	  END DO
	END DO

	END
