	INTEGER*4 FUNCTION Lib_Get_File_Info(Filespec,
     >                                       Default_Name,
     >                                       User_Action_Routine,
     >                                       Err)

C++
C Author:	Jonathan Welch		Creation Date: September, 1986
c
C Functional description:
C	For each file that matches Filespec the following call is made:
c		CALL User_Action_Routine(Sts, Iosb [,Fab, Fib, Nam, Attributes])
c
c		Sts
c		  VMS Usage  : longword_unsigned
c		  type       : longword (unsigned)
c		  access     : read only
c		  mechanism  : by reference
c
c		IF POSITIVE:
c		  Return value from either the QIOW or ASSIGN system
c                 service request generated in the SUCCESS routine.
c		    IF Iosb(1) = 0
c		      It is from the ASSIGN call (and you will probably be
c		      getting an error from QIOW if the ASSIGN failed)
c		    ELSE it is from the QIOW call.
c
c		IF NEGATIVE:
c		  The negative of the return value in the FAB$L_STS field
c		  Iosb(1) = FAB$L_STV
c
c		Note: If the value is negative the rest of the parameters
c		      in the call to User_Action_Routine will not be present
c		      so DO NOT try to access them.  If you do you will
c		      get an access violation.
c
c		Iosb
c		  VMS Usage  : io_status_block
c		  type       : quadword (unsigned)
c		  access     : read only
c		  mechanism  : by reference
c
c		A two longword array
c		  The first longword is used for secondary status return values.
c
c		Fab
c		  VMS Usage  : fab
c		  type       : unspecified
c		  access     : read only
c		  mechanism  : by reference
c
c		The File Access Block for the file currently being processed.
c
c		Fib
c		  VMS Usage  : fib
c		  type       : unspecified
c		  access     : read only
c		  mechanism  : by reference
c
c		The File Information Block for the file currently
c		being processed.
c
c		Nam
c		  VMS Usage  : nam
c		  type       : unspecified
c		  access     : read only
c		  mechanism  : by reference
c
c		The NAMe Block for the file currently being processed.
c
c		Attributes
c		  VMS Usage  : user_arg
c		  type       : unspecified
c		  access     : read only
c		  mechanism  : by reference
c
c		ATR$C_xxxxxx requests.  Only the fields up through
c		ATR$C_JOURNAL (but not ATR$C_ENDLBLAST) are filled in.
c		See the section ACP-QIO Interface in the I/O User's
c		Reference Manual pp.1-17:1-24
c
c		Note:
c		    (1) The subfields for the Status Block (SBK) and record
c		        attributes (FAT) are included as part of the ATR block.
c                   (2) SBK$L_FILESIZE is not present.  Instead use
c		        SBK$W_FILESIZH and SBK$W_FILESIZL.
c	The user is free to process the above parameters in whatever
c	way he/she finds most useful.
c
c
C Calling sequence:
c	Sts = Lib_Get_File_Info(Filespec, Defaultname, User_Action_Routine, Err)
c
C Formal Arguments:
c
c	Filespec
c	  VMS Usage  : char_string
c	  type       : character string
c	  access     : read only
c	  mechanism  : by descriptor
c
c	A [wildcarded] filespec.
c
c	Default_Name
c	  VMS Usage  : char_string
c	  type       : character string
c	  access     : read only
c	  mechanism  : by descriptor
c
c	Default filespec pieces to scan for.
c
c	User_Action_Routine
c	  VMS Usage  : procedure
c	  type       : procedure entry mask
c	  access     : function call (before return)
c	  mechanism  : by reference, procedure reference
c
c	User supplied action routine to be called for each file processed
c	found by LIB$FILE_SCAN
c
c
c	Err
c	  VMS Usage  : vector_longword_unsigned
c	  type       : longword (unsigned)
c	  access     : read only
c	  mecnanism  : by reference
c
c	Err(1) - The return status from the SYS$DASSGN call
c	Err(2) - The return status from the LIB$FILE_SCAN_END call
c
C Implicit Inputs:
c	none
c
C Implicit Outputs:
c	none
c
C Completion status:
c	A function completion code from the LIB$FILE_SCAN routine.
c
C--

	INCLUDE '($RMSDEF)'
	INCLUDE '($FABDEF)'
	INCLUDE '($NAMDEF)'

	STRUCTURE /Descr/
	  INTEGER*4 Length
	  INTEGER*4 Addr
	END STRUCTURE

	RECORD /Fabdef/ Fab
	RECORD /Namdef/ Nam
	RECORD /Descr/ Dsc   
C This strdescr is used to extract the device name from the Nam

	COMMON /Action_Rtn/Action_Routine
	COMMON Chan, Nam

	BYTE Default_Name_Bytes(255)
	BYTE Filespec_Bytes(255)

	CHARACTER*(*) Default_Name
	CHARACTER*255 Es, Rs !Expanded string, Resultant string
	CHARACTER*(*) Filespec

	EXTERNAL Lib_Success, Lib_Error  ! Action routines

	INTEGER*2 Chan

	INTEGER*4 Context
	INTEGER*4 Err(2)
	INTEGER*4 Loop
	INTEGER*4 Sts

	INTEGER*4 Action_Routine
	INTEGER*4 User_Action_Routine

	INTEGER*4 LIB$FILE_SCAN
	INTEGER*4 LIB$FILE_SCAN_END
	INTEGER*4 SYS$DASSGN

	Action_Routine = %LOC(User_Action_Routine)

	DO Loop = 1, LEN(Default_Name)
	  Default_Name_Bytes(Loop) = ICHAR(Default_Name(Loop:Loop))
	END DO

	DO Loop = 1, LEN(Filespec)
	  Filespec_Bytes(Loop) = ICHAR(Filespec(Loop:Loop))
	END DO

	Fab.FAB$B_BLN = FAB$C_BLN ! Identify Fab as a valid FAB
	Fab.FAB$B_BID = FAB$C_BID !
	Fab.FAB$L_DNA = %LOC(Default_Name_Bytes)
	Fab.FAB$B_DNS = LEN(Default_Name)
	Fab.FAB$L_FNA = %LOC(Filespec_Bytes)
	Fab.FAB$B_FNS = LEN(Filespec)
	Fab.FAB$L_NAM = %LOC(Nam) ! Tell the Fab where to find the NAM block

	CALL LIB$MOVC5 (0,, 0, NAM$C_BLN, NAM)

	Nam.NAM$B_BLN = NAM$C_BLN ! Identify Nam as a valid NAM
	Nam.NAM$B_BID = NAM$C_BID !
	Nam.NAM$L_ESA = %LOC(Es)  ! Tell it where to find the Expanded and
	Nam.NAM$L_RSA = %LOC(Rs)  ! Resultant strings
	Nam.NAM$B_ESS = 255       ! Their lengths
	Nam.NAM$B_RSS = 255

	Context = 0

	Sts = LIB$FILE_SCAN(Fab, Lib_Success, Lib_Error, Context)

	Lib_Get_File_Info = Sts

	IF(Chan .NE. 0)Err(1) = SYS$DASSGN(%VAL(Chan))

	Chan = 0

	Err(2) = LIB$FILE_SCAN_END(Fab, Context)

	RETURN
	END

	SUBROUTINE Lib_Success(Fab)

	INCLUDE '($ATRDEF)'
	INCLUDE '($FABDEF)'
	INCLUDE '($FIBDEF)'
	INCLUDE '($IODEF)'
	INCLUDE '($NAMDEF)'
	INCLUDE '($RMSDEF)'
	INCLUDE '($SBKDEF)'
	INCLUDE '($DSCDEF)'

	STRUCTURE /Dsc/
	  INTEGER*4 Length
	  INTEGER*4 Addr
	END STRUCTURE

	INCLUDE 'CSD$LIBRARY:attr.inc'

	RECORD /Dsc/ Descr
	RECORD /Atrdef/ Atr(26)
	RECORD /Attr/ Attributes
	RECORD /Fabdef/ Fab
	RECORD /Fibdef/ Fib
	RECORD /Namdef/ Nam
	RECORD /Sbkdef/ Sbk

	COMMON /Action_Rtn/Action_Routine
	COMMON Chan, Nam

	INTEGER*2 Chan

	INTEGER*4 Qiow_Sts, Sts
	INTEGER*2 Iosb(4)

	INTEGER*4 Action_Routine

	INTEGER*4 SYS$ASSIGN
	INTEGER*4 SYS$QIOW

	IF(Chan. EQ. 0)THEN
	  Descr.Length = Nam.NAM$B_DEV ! Build a Strdescr for the device name
	  Descr.Addr = Nam.NAM$L_DEV

	  Iosb(1) = 0
	  Sts = SYS$ASSIGN(Descr ,Chan,,) ! Assign a channel to the device
	  IF(.NOT. Sts .AND. Sts .NE. 0)
     >      CALL Lib_Invoke_Action_Routine(%VAL(Action_Routine),
     >                                     Sts,
     >                                     Iosb,
     >                                     Fab,
     >                                     Fib,
     >                                     Nam,
     >                                     Attributes)


	  Atr(1).ATR$W_SIZE = ATR$S_UCHAR
	  Atr(1).ATR$W_TYPE = ATR$C_UCHAR
	  Atr(1).ATR$L_ADDR = %LOC(Attributes.Uchar)

	  Atr(2).ATR$W_SIZE = ATR$S_RECATTR
	  Atr(2).ATR$W_TYPE = ATR$C_RECATTR
	  Atr(2).ATR$L_ADDR = %LOC(Attributes.FAT$B_RTYPE)

	  Atr(3).ATR$W_SIZE = ATR$S_FILNAM
	  Atr(3).ATR$W_TYPE = ATR$C_FILNAM
	  Atr(3).ATR$L_ADDR = %LOC(Attributes.Filnam)

	  Atr(4).ATR$W_SIZE = ATR$S_FILTYP
	  Atr(4).ATR$W_TYPE = ATR$C_FILTYP
	  Atr(4).ATR$L_ADDR = %LOC(Attributes.Filtyp)

	  Atr(5).ATR$W_SIZE = ATR$S_FILVER
	  Atr(5).ATR$W_TYPE = ATR$C_FILVER
	  Atr(5).ATR$L_ADDR = %LOC(Attributes.Filver)

	  Atr(6).ATR$W_SIZE = ATR$S_EXPDAT
	  Atr(6).ATR$W_TYPE = ATR$C_EXPDAT
	  Atr(6).ATR$L_ADDR = %LOC(Attributes.Expdat)

	  Atr(7).ATR$W_SIZE = ATR$S_STATBLK
	  Atr(7).ATR$W_TYPE = ATR$C_STATBLK
	  Atr(7).ATR$L_ADDR = %LOC(Attributes.SBK$L_STLBN)

	  Atr(8).ATR$W_SIZE = ATR$S_HEADER
	  Atr(8).ATR$W_TYPE = ATR$C_HEADER
	  Atr(8).ATR$L_ADDR = %LOC(Attributes.Header)

	  Atr(9).ATR$W_SIZE = ATR$S_BLOCKSIZE
	  Atr(9).ATR$W_TYPE = ATR$C_BLOCKSIZE
	  Atr(9).ATR$L_ADDR = %LOC(Attributes.Blocksize)

	  Atr(10).ATR$W_SIZE = ATR$S_USERLABEL
	  Atr(10).ATR$W_TYPE = ATR$C_USERLABEL
	  Atr(10).ATR$L_ADDR = %LOC(Attributes.Userlabel)

	  Atr(11).ATR$W_SIZE = ATR$S_ASCDATES
	  Atr(11).ATR$W_TYPE = ATR$C_ASCDATES
	  Atr(11).ATR$L_ADDR = %LOC(Attributes.Ascdates)

	  Atr(12).ATR$W_SIZE = ATR$S_ALCONTROL
	  Atr(12).ATR$W_TYPE = ATR$C_ALCONTROL
	  Atr(12).ATR$L_ADDR = %LOC(Attributes.Alcontrol)

	  Atr(13).ATR$W_SIZE = ATR$S_ASCNAME
	  Atr(13).ATR$W_TYPE = ATR$C_ASCNAME
	  Atr(13).ATR$L_ADDR = %LOC(Attributes.Ascname)

	  Atr(14).ATR$W_SIZE = ATR$S_CREDATE
	  Atr(14).ATR$W_TYPE = ATR$C_CREDATE
	  Atr(14).ATR$L_ADDR = %LOC(Attributes.Credate)

	  Atr(15).ATR$W_SIZE = ATR$S_REVDATE
	  Atr(15).ATR$W_TYPE = ATR$C_REVDATE
	  Atr(15).ATR$L_ADDR = %LOC(Attributes.Revdate)

	  Atr(16).ATR$W_SIZE = ATR$S_EXPDATE
	  Atr(16).ATR$W_TYPE = ATR$C_EXPDATE
	  Atr(16).ATR$L_ADDR = %LOC(Attributes.Expdate)

	  Atr(17).ATR$W_SIZE = ATR$S_BAKDATE
	  Atr(17).ATR$W_TYPE = ATR$C_BAKDATE
	  Atr(17).ATR$L_ADDR = %LOC(Attributes.Bakdate)

	  Atr(18).ATR$W_SIZE = ATR$S_UIC
	  Atr(18).ATR$W_TYPE = ATR$C_UIC
	  Atr(18).ATR$L_ADDR = %LOC(Attributes.Uic)

	  Atr(19).ATR$W_SIZE = ATR$S_FPRO
	  Atr(19).ATR$W_TYPE = ATR$C_FPRO
	  Atr(19).ATR$L_ADDR = %LOC(Attributes.Fpro)

	  Atr(20).ATR$W_SIZE = ATR$S_RPRO
	  Atr(20).ATR$W_TYPE = ATR$C_RPRO
	  Atr(20).ATR$L_ADDR = %LOC(Attributes.Rpro)

	  Atr(21).ATR$W_SIZE = ATR$S_ACLEVEL
	  Atr(21).ATR$W_TYPE = ATR$C_ACLEVEL
	  Atr(21).ATR$L_ADDR = %LOC(Attributes.Aclevel)

	  Atr(22).ATR$W_SIZE = ATR$S_SEMASK
	  Atr(22).ATR$W_TYPE = ATR$C_SEMASK
	  Atr(22).ATR$L_ADDR = %LOC(Attributes.Semask)

	  Atr(23).ATR$W_SIZE = ATR$S_UIC_RO
	  Atr(23).ATR$W_TYPE = ATR$C_UIC_RO
	  Atr(23).ATR$L_ADDR = %LOC(Attributes.Uic_Ro)

	  Atr(24).ATR$W_SIZE = ATR$S_DIRSEQ
	  Atr(24).ATR$W_TYPE = ATR$C_DIRSEQ
	  Atr(24).ATR$L_ADDR = %LOC(Attributes.Dirseq)

	  Atr(25).ATR$W_SIZE = ATR$S_BACKLINK
	  Atr(25).ATR$W_TYPE = ATR$C_BACKLINK
	  Atr(25).ATR$L_ADDR = %LOC(Attributes.Backlink)

	  Atr(26).ATR$W_SIZE = 0
	  Atr(26).ATR$W_TYPE = 0

	END IF

	Fib.FIB$W_FID_NUM = Nam.NAM$W_FID_NUM ! Setup File ID values
	Fib.FIB$W_FID_SEQ = Nam.NAM$W_FID_SEQ
	Fib.FIB$W_FID_RVN = Nam.NAM$W_FID_RVN

	Fib.FIB$W_DID_NUM = Nam.NAM$W_DID_NUM ! Same for Directory ID values
	Fib.FIB$W_DID_SEQ = Nam.NAM$W_DID_SEQ
	Fib.FIB$W_DID_RVN = Nam.NAM$W_DID_RVN

	Descr.Length=FIB$C_ACCDATA
	Descr.Addr = %LOC(Fib)

C Get the requests
	Qiow_Sts = SYS$QIOW(,%VAL(Chan), %VAL(IO$_ACCESS),Iosb,,,
     >                      Descr,,,,Atr,)

	CALL Lib_Invoke_Action_Routine(%VAL(Action_Routine), Qiow_Sts, Iosb,
     >                                 Fab, Fib, Nam, Attributes)

	RETURN
	END

	SUBROUTINE Lib_Error(Fab)

	INCLUDE '($FABDEF)'
	RECORD /Fabdef/ Fab

	COMMON /Action_Rtn/Action_Routine

	INTEGER*4 Iosb(2)

	INTEGER*4 Action_Routine

	IF(.NOT. Fab.FAB$L_STS .AND. Fab.FAB$L_STS .NE. 0)THEN
	  Iosb(1) = Fab.FAB$L_STV
	  CALL Lib_Invoke_Action_Routine(%VAL(Action_Routine),
     >                                   -Fab.FAB$L_STS, Iosb, 
     >                                   0, 0, 0, 0)
	END IF

	RETURN
	END
