	INTEGER*4 Function Lib_LNMTable_File(File, Table, Stv)

C++
c
C Author: Jonathan Welch		Creation Date: 29-Jun-1987 13:35
c
c Functional Description:
c
c	The logical names in the specified table are saved to the specified
c	file.  Their access mode and attributes are saved to the file, too.
c
c	The structure of the file is:
c	  Lognam
c	  [Equivalence_strings]
c
c	  where
c	  Equivalence_strings = 
c	    Access mode (byte), Attributes (longword), Equivalence (char_string)
c
c	Note:
c	  Logical names which are table entries will be in the
c	  file without any equivalence strings.  Currently these
c	  entries should be ignored when re-loading the file
c	  into a table.
c
C Calling Sequence:
c
c	Sts = Lib_LNMTable_File(Filename, Tablename, Stv)
c
C Formal arguments:
c
c	Filename
c	  VMS Usage  : char_string
c	  type       : character string
c	  access     : read only
c	  mechanism  : by descriptor
c
c	Specifies the name of the file the logical names in the specified
c	logical name table are to be written into.
c
c	Tablename
c	  VMS Usage  : char_string
c	  type       : character string
c	  access     : read only
c	  mechanism  : by descriptor
c
c	The name of the logical name table to save to the specified file.
c	Logical names which are table headers will be saved to the file
c	but will have to equivalence strings following them.
c
c	Stv
c	  VMS Usage  : longword(unsigned)
c	  type       : unsigned longword
c	  access     : write only
c	  mechanism  : by reference
c
c	If a call to a RMS service returns an error Stv will contain the
c	secondary status value RMS$L_STV.
c
C Implicit Inputs:
c
c	none
c
C Implicit Outputs:
c
c	none
c
C Completion status:
c
c	SS$_NORMAL - Routine completed successfully.
c
c	Any values returned by the following routines:
c
c	Lib_Assemble_LNMs
c	STR$TRIM
c	SYS$CREATE
c	SYS$CONNECT
c	SYS$PUT
c	SYS$TRNLNM
c
C Side effects:
c
c	STR$TRIM may also signal errors.
c
C--

	INCLUDE '($FABDEF)'
	INCLUDE '($LNMDEF)'
	INCLUDE '($RABDEF)'
	INCLUDE '($SSDEF)'

	BYTE Buffer(1 + 4 + 255)! Buffer to hold each record as it is outputted.
	BYTE FName(255)		! Byte array to hold name of output file.

	CHARACTER*(*) File
	CHARACTER*255 Logs(1024) ! No more than 1024 logicals in a table allowed
	CHARACTER*(*) Table

	INTEGER*4 Eqv_Len	! Length of an equivalence string.
	INTEGER*2 Str_Len	! Length of a string.

	INTEGER*4 Cnt
	INTEGER*4 Idx
	INTEGER*4 Loop
	INTEGER*4 Loop2
	INTEGER*4 Sts
	INTEGER*4 Stv

	INTEGER*4 Lib_Assemble_LNMs
	INTEGER*4 Sys_Btest

	INTEGER*4 STR$TRIM
	INTEGER*4 SYS$CONNECT
	INTEGER*4 SYS$CREATE
	INTEGER*4 SYS$PUT
	INTEGER*4 SYS$TRNLNM

	STRUCTURE /Itmlst/
	  UNION
	    MAP
	      INTEGER*2 BufLen
	      INTEGER*2 Code
	      INTEGER*4 Addr
	      INTEGER*4 RetLen
	    END MAP
	    MAP
	      INTEGER*4 End_List
	    END MAP
	  END UNION
	END STRUCTURE

	RECORD /Fabdef/ Fab		! File Access Block
	RECORD /Itmlst/ Itemlist(5)	! Itemlist for $TRNLNM calls.
	RECORD /Rabdef/ Rab		! Record Attributes Block

	Lib_LNMTable_File = SS$_NORMAL	! Assume successful completion

	Sts = Lib_Assemble_LNMs(Table, Logs, Cnt)
	IF(.NOT. Sts .AND. Sts .NE. 0)THEN
	  Lib_LNMTable_File = Sts
	  RETURN
	END IF

	DO Loop = 1, LEN(File)
	  FName(Loop) = ICHAR(File(Loop:Loop))
	END DO

	Fab.FAB$B_BLN = FAB$C_BLN	! Identify Fab as a valid FAB
	Fab.FAB$B_BID = FAB$C_BID	!
	Fab.FAB$L_FNA = %LOC(FName)	! File Name Address
	Fab.FAB$B_FNS = LEN(File)	! File Name Size
	Fab.FAB$B_RAT = FAB$M_CR	! Carriage-return Record ATtributes
	Fab.FAB$B_RFM = FAB$C_VAR	! VARiable length records

	Rab.RAB$B_BLN = RAB$C_BLN	! Identify Rab as a valid RAB
	Rab.RAB$B_BID = RAB$C_BID	!
	Rab.RAB$L_FAB = %LOC(Fab)	! Location of input FAB
	Rab.RAB$L_RBF = %LOC(Buffer(1))	! Record BuFfer

	Sts = SYS$CREATE(Fab)		! Open the file
	IF(.NOT. Sts .AND. Sts .NE. 0)THEN
	  Lib_LNMTable_File = Sts	! Return if there is an error.
	  Stv = Fab.FAB$L_STV		! Pass back the secondary status value.
	  RETURN
	END IF

	Sts = SYS$CONNECT(Rab)		! Associate the Rab with the Fab
	IF(.NOT. Sts .AND. Sts .NE. 0)THEN
	  Lib_LNMTable_File = Sts	! Return if there is an error.
	  Stv = Fab.FAB$L_STV		! Pass back the secondary status value.
	  RETURN
	END IF

	Itemlist(1).Buflen = 4
	Itemlist(1).Code = LNM$_INDEX
	Itemlist(1).Addr = %LOC(Idx)
	Itemlist(1).Retlen = 0

	Itemlist(2).Buflen = 1
	Itemlist(2).Code = LNM$_ACMODE
	Itemlist(2).Addr = %LOC(Buffer(1))
	Itemlist(2).Retlen = 0

	Itemlist(3).Buflen = 4
	Itemlist(3).Code = LNM$_ATTRIBUTES
	Itemlist(3).Addr = %LOC(Buffer(2))
	Itemlist(3).Retlen = 0

	Itemlist(4).Buflen = 255
	Itemlist(4).Code = LNM$_STRING
	Itemlist(4).Addr = %LOC(Buffer(6))
	Itemlist(4).Retlen = %LOC(Eqv_Len)

	Itemlist(5).End_List = 0

	DO Loop = 1, Cnt

	  Sts = STR$TRIM(Logs(Loop), Logs(Loop), Str_Len)
	  IF(.NOT. Sts .AND. Sts .NE. 0)THEN
	    Lib_LNMTable_File = Sts
	    RETURN
	  END IF

	  DO Loop2 = 1, Str_Len
	    Buffer(Loop2) = ICHAR(Logs(Loop)(Loop2:Loop2))
	  END DO

	  Rab.RAB$W_RSZ = Str_Len

	  Sts = SYS$PUT(Rab)
	  IF(.NOT. Sts .AND. Sts .NE. 0)THEN
	    Lib_LNMTable_File = Sts	! Return if there is an error.
	    Stv = Fab.FAB$L_STV		! Pass back the secondary status value.
	    RETURN
	  END IF

	  Idx = 0
	  Sts = SS$_NORMAL
	  DO WHILE(Sts .AND. Idx .LE. 127)

	    Sts = SYS$TRNLNM(,Table, Logs(Loop)(:Str_Len),, Itemlist)
	    IF(.NOT. Sts .AND. Sts .NE. 0)THEN
	      Lib_LNMTable_File = Sts
	      RETURN
	    END IF

	    IF(Eqv_Len .NE. 0)THEN
	      Rab.RAB$W_RSZ = 1 + 4 + Eqv_Len
	      Sts = SYS$PUT(Rab)
	      IF(.NOT. Sts .AND. Sts .NE. 0)THEN
	        Lib_LNMTable_File = Sts	! Return if there is an error.
	        Stv = Fab.FAB$L_STV		! Pass back the secondary status value.
	        RETURN
	      END IF
	    END IF

	    Idx = Idx + 1

	  END DO
	END DO

	END
