/* 
Author: Joe Bryan               Creation Date: 30-Sep-1987


Functional description:
        The RMS $PARSE service is called with the file-name and
        default-name fields as specified.  The field argument is used to
        specify which components of the parsed file specification are to be
        returned in the file-spec argument.  The type argument is used
        to set the NAM$B_NOP field in the NAM block. 

Calling sequence:
        Sts = RMS_PARSE(Filespec ,Filename ,[Default-name] ,[Field] ,[Type])

Formal arguments:
        While all five arguments must be supplied, arguments 3, 4 and 5 may 
        be zero or blank.

	Filespec
          VMS Usage  : char_string
          type       : character string
          access     : write only
          mechanism  : by descriptor, fixed length

        String that will receive the segment(s) of the parsed file
        specification requested by the field argument.  This argument
        must be specified. 

	Filename

          VMS usage  : char_string
          type       : character string, fixed length
          access     : read only
          mechanism  : by descriptor

        Primary components of file specification.  Any components not
        specified will be filed in buy the default-name string and
        process environment default.  This argument must be specified. 


	Default-name

          VMS usage  : VMS file specification
          type       : character string, fixed length
          access     : read only
          mechanism  : by descriptor

        Default components of file specification.  Any components not
        specified by either the filename or default-name strings will
        be filled in by the default file specification defined for the
        user's process (using the DCL command SET DEFAULT <filespec>).
        If the default-name string is blank or this argument is zero,
        only the process default will be used. 


	Field

          VMS Usage  : unsigned_byte 
          type       : byte (unsigned)
          access     : read only
          mechanism  : by reference

        Bit mask that specifies the components of the parsed file
        specification that are to be returned in the filespec string. 
        These codes may be or'ed together to return more than one
        component of the specification.  If no bits are set or this
        argument is zero, the complete file specification will be
        returned.

        The field codes are identical to the FSCN$M_<field> codes
        defined in the $FSCNDEF modules.  The numeric values are
        listed below:

        File spec       Bit field       Decimal    Symbolic
        Field           Offset          Value      Mask name
        --------------------------------------------------------
        Node            0                1         FSCN$M_NODE
        Device          1                2         FSCN$M_DEVICE
        Root            2                4         FSCN$M_ROOT
        Directory       3                8         FSCN$M_DIRECTORY
        Name            4               16         FSCN$M_NAME
        Type            5               32         FSCN$M_TYPE
        Version         6               64         FSCN$M_VERSION


        Type

          VMS Usage  : unsigned_byte 
          type       : byte (unsigned)
          access     : read only
          mechanism  : by reference

        Bit mask that specifies the type of parse to perform.  This
        argument is used to set the NAM$B_NOP field in the NAM block. 
        If this argument is zero, or no bits are set, the standard
        defaults will be used. 

        These bit masks are defined in the $NAMDEF modules.  The
        numeric values are listed below: 

        Parse                 Bit field     Decimal    Symbolic
        Type                  Offset        Value      Mask name
        --------------------------------------------------------------
        Perform syntax
        check only            3             8          NAM$M_SYNCHK

        Don't conceal
        device, directory
        logical               4            16          NAM$M_NOCONCEAL

Implicit arguments:

        none

Side effects:

        none

Completion status:

        RMS $PARSE service completion status.  If RMS$_ESS is
        returned, the filespec output string was truncated. 

Function characteristics:

        AST re-entrant, shareable.

 */

#define NOMAIN true	/* remove to include main driver test routine */
#ifdef	NOMAIN
/* 
#endif
#include "c_dev:typ_def.h"
#include "c_dev:dscr_def.h"

main () {
	long	stat, *fld_ptr, field, *typ_ptr, type;
	DSCR	dsc[4];
	char	filename[80], *def_ptr, defname[80], file_spec[41];
	char	tmpstr[80];

	file_spec[40] = '\0';
	while (1) {
	    *filename = '\0';
	    printf ("\n_File: "); gets (filename);
	    if (! strlen (filename)) break;

	    *defname = '\0';
	    printf ("\n_Default: "); gets (defname);
	    if (strlen (defname)) 
		def_ptr = defname;
	    else
		def_ptr = 0;

	    *tmpstr = '\0';
	    printf ("\n_Fields: "); gets (tmpstr);
	    if (strlen (tmpstr)) {
		field = atoi (tmpstr);
		fld_ptr = &field;
	    } else
		fld_ptr = 0;

	    *tmpstr = '\0';
	    printf ("\n_Type: "); gets (tmpstr);
	    if (strlen (tmpstr)) {
		type = atoi (tmpstr);
		typ_ptr = &type;
	    } else
		typ_ptr = 0;

	    printf ("\nFile: %s", filename);
	    if (def_ptr) printf ("  Default: %s", def_ptr);
	    else	 printf ("  Default: -none-");
	    if (fld_ptr) printf ("  Field: %2.2x", *fld_ptr);
	    else	 printf ("  Field: -none-");
	    if (typ_ptr) printf ("  Type: %2.2x", *typ_ptr);
	    else	 printf ("  Type: -none-");

	    stat = RMS_PARSE (
		 mkdscrl(&dsc[0],file_spec,40)
		,mkdscr(&dsc[1],filename)
		,mkdscr(&dsc[2],defname)
		,fld_ptr
		,typ_ptr
		);

	    if (stat & 1) {
		*(char*)strchr(file_spec,' ') = '\0';
		printf ("\nStat: %8.8x  Spec: %s", stat, file_spec);
	    } else 
		printf ("\nError stat: %8.8x", stat);
	};
}
#ifdef	NOMAIN
 */
#endif

/* RMS_PARSE.C */

#include rms
#include descrip
#include "csd$library:$fscndef.h"

#define DSCR		struct dsc$descriptor_s
#define word		unsigned short
#define byte		unsigned char
#define min(a,b)	(a > b ? b : a)

long 
RMS_PARSE (fil_spc, fil_nam, def_nam, field, type)
DSCR	*fil_spc;	/* OUT file_spec or file_spec field */
DSCR	*fil_nam;	/* IN  file_spec */
DSCR	*def_nam;	/* IN  default file_spec */
byte	*field;		/* IN  field(s) to return */
byte	*type;		/* IN  parse type flags */
{
	DSCR		dsc[1];
	struct FAB	fab;
	struct NAM	nam;
	char		expanded_name [NAM$C_MAXRSS];
	char		*nam$l_root;
	register byte	nam$b_root = 0;
	register long	stat;
	register long	file_spec = (long) (fil_spc->dsc$a_pointer);
	register byte	fld = 0;
	register word	len = (word) (fil_spc->dsc$w_length);
	register char	chr;
	long		tmp = 0;

	LIB$MOVC5 (&tmp, &tmp, " ", 
	    &fil_spc->dsc$w_length, fil_spc->dsc$a_pointer);
	fab = cc$rms_fab;
	nam = cc$rms_nam;
	fab.fab$l_fna = fil_nam->dsc$a_pointer;
	fab.fab$b_fns = fil_nam->dsc$w_length;
	if (def_nam && def_nam->dsc$w_length) {
	    fab.fab$l_dna = def_nam->dsc$a_pointer;
	    fab.fab$b_dns = def_nam->dsc$w_length;
	};
	fab.fab$l_nam = &nam;
	nam.nam$l_esa = expanded_name;
	nam.nam$b_ess = NAM$C_MAXRSS;
	if (type) nam.nam$b_nop = *type;

	stat = SYS$PARSE (&fab);

	if (stat & 1) {

	    /* extract all the fields if requested */
	    if (field) fld = *field;
	    if (! fld && len) {
		if (len < nam.nam$b_esl)
		    stat = RMS$_ESS;
		else
		    strncpy (file_spec, nam.nam$l_esa, nam.nam$b_esl);
	    } else {
		/* extract individual fields as required */

		/* extract root field from directory if requested */
		nam$l_root = nam.nam$l_dir;
		if (fld & FSCN$M_ROOT && nam.nam$l_fnb & NAM$M_ROOT_DIR) {
		    nam$b_root = nam.nam$b_dir;
		    nam.nam$l_dir = nam.nam$l_name;
		    nam.nam$b_dir = 0;
		    while (nam.nam$b_dir != nam$b_root 
		    && (chr = *nam.nam$l_dir) != '[' && chr != '<') {
			nam.nam$b_dir++;
			nam.nam$l_dir--;
		    };
		    nam$b_root -= nam.nam$b_dir;
		};

		/* add field elements */

#define add(B_FIELD,L_FIELD,MASK) { \
		if (fld & MASK) { \
		    if (! len) { \
			stat = RMS$_ESS; \
			fld = 0; \
		    } else { \
			B_FIELD = min (len, B_FIELD); \
			strncpy (file_spec, L_FIELD, B_FIELD); \
			len -= B_FIELD; \
			file_spec += (long) B_FIELD; \
		    }; \
		}}

		add (nam.nam$b_node, nam.nam$l_node, FSCN$M_NODE);
		add (nam.nam$b_dev,  nam.nam$l_dev,  FSCN$M_DEVICE);
		add (nam$b_root,     nam$l_root,     FSCN$M_ROOT);
		add (nam.nam$b_dir,  nam.nam$l_dir,  FSCN$M_DIRECTORY);
		add (nam.nam$b_name, nam.nam$l_name, FSCN$M_NAME);
		add (nam.nam$b_type, nam.nam$l_type, FSCN$M_TYPE);
		add (nam.nam$b_ver,  nam.nam$l_ver,  FSCN$M_VERSION);
	    };
	};

	/* release allocated space if any */
	if ((tmp = nam.nam$l_fnb) & NAM$M_NODE 
	|| tmp & NAM$M_SEARCH_LIST || tmp & NAM$M_WILDCARD) {
	    fab.fab$b_dns = 0;
	    nam.nam$b_nop = NAM$M_SYNCHK;
	    nam.nam$l_rlf = 0;
	    SYS$PARSE (&fab);
	};

	return stat;
}

