	INTEGER*4 FUNCTION Sys_Crmpsc(Filename, Flags, Section_Name,
     >                                Size, Out)

C++
c
C Author:	Jonathan Welch
c
C Functional description:
c
c	This routine provides a simplified interface to SYS$CRMPSC.
c
C Formal Arguments:
c
c	Filename
c	  VMS Usage  : char_string
c	  type       : character string
c	  access     : read only
c	  mechanism  : by descriptor
c
c	Filename the global section is to be mapped to.
c
c	Flags
c	  VMS Usage  : longword_unsigned
c	  type       : longword (unsigned)
c	  access     : read only
c	  mechanism  : by reference
c
c	Flag mask sepcifying the type of section to be created or mapped
c	to, as well as its characteristics.  The flags argument is a
c	longword bit vector wherein each bit corresponds to a flag.  The
c	$SECDEF macro defines a symbolic name for each flag.  The flags
c	argument is constructed by performing a logical OR operation
c	on the symbol names for all desired flags.
c
c	Section_Name
c	  VMS Usage  : char_string
c	  type       : character string
c	  access     : read only
c	  mechanism  : by descriptor
c
c	Name of the global section.  The Section_Name is the address of
c	a character string descriptor pointing to this name string.  Section
c	11.6.6.1 in the System Services manual describes the format of
c	this name string.
c
c	For group global section, VAX/VMS interprets the UIC group as
c	part of the global section name; thus, the names of global
c	sections are unique to UIC groups.
c
c	Size
c	  VMS Usage  : longword_unsigned
c	  type       : longword (unsigned)
c	  access     : read only
c	  mechanism  : by reference
c
c	Size of the file to create if the file specified by the
c	Filename argument does not exist.
c
c	Out
c	  VMS Usage  : address_range
c	  type       : longword (unsigned)
c	  access     : write only
c	  mechanism  : by reference, array reference
c
c	Starting and ending process virtual addresses into which the
c	section was actually mapped by $CRMPSC.  The Out argument is
c	the address of a two-longword array containing, in order, the
c	starting and ending process virtual addresses.
c
C Implicit inputs:
c
c	none
c
C Implicit outputs:
c
c	none
c
c Completion status:
c
c	Any possible return values from:
c
c	  SYS$CREATE
c	  SYS$CRMPSC
c
C--

	INCLUDE '($FABDEF)'
	INCLUDE '($SECDEF)'

	BYTE Default_Name(4)/'.','d','a','t'/	! open .dat file by default
	BYTE FN_Char(255)			! byte array to hold filename

	CHARACTER*(*) Filename			! passed arguments
	CHARACTER*(*) Section_Name

	INTEGER*2 Chan				! channel to section file

	INTEGER*4 Flags				! passed arguments
	INTEGER*4 In(2)				! presence causes section to be mapped
	INTEGER*4 Loop
	INTEGER*4 Out(2)			! passed arguments
	INTEGER*4 Size
	INTEGER*4 Sts				! local return values from function calls

	INTEGER*4 SYS$CREATE			! RMS file create service
	INTEGER*4 SYS$CRMPSC			! CReate and Map SeCtion

	RECORD /Fabdef/ Fab			! A File Access Block

	DO Loop = 1, LEN(Filename)
	  FN_Char(Loop) = ICHAR(Filename(Loop:Loop)) ! move filename into a byte array
	END DO

	Fab.FAB$L_ALQ = Size		! set up Fab - see RMS Reference Manual
	Fab.FAB$B_BID = FAB$C_BID	! for field descriptions.
	Fab.FAB$B_BLN = FAB$C_BLN
	Fab.FAB$L_DNA = %LOC(Default_Name(1))
	Fab.FAB$B_DNS = 4
	Fab.FAB$B_FAC = FAB$M_BIO
	Fab.FAB$L_FNA = %LOC(FN_Char(1))
	Fab.FAB$B_FNS = LEN(Filename)
	Fab.FAB$L_FOP = FAB$M_UFO + FAB$M_CIF
	Fab.FAB$B_SHR = FAB$M_SHRUPD + FAB$M_UPI

	Sts = SYS$CREATE(Fab,,)		! Create the section file if necessary.
	IF(.NOT. Sts .AND. Sts .NE. 0)THEN
	  Sys_Crmpsc = Sts
	  RETURN
	END IF

C	Map the section in.
	Sys_Crmpsc = SYS$CRMPSC(In,
     >                          Out,,
     >                          %VAL(Flags),
     >                          Section_Name,,,
     >                          %VAL(Fab.FAB$L_STV),,,,)

	RETURN
	END
