	FUNCTION LONG Sys_Submit(STRING Filename, STRING Queue, &
	                         LONG Modifiers() BY REF)

 !++
 !
 ! Author:	Jonathan Welch
 !
 ! Functional Description:
 !
 !	Submits a file to a queue.  Provides a simplified interface
 !	to $SNDJBC.
 !
 ! Calling Sequence:
 !
 !	Sts = Sys_Submit(Filename, Queue, Modifiers)
 !
 ! Formal arguments:
 !
 !	Filename
 !	  VMS Usage  : char_string
 !	  type       : character string
 !	  access     : read only
 !	  mechanism  : by descriptor
 !
 !	Name of a file to be submitted to a queue.
 !
 !	Queue
 !	  VMS Usage  : char_string
 !	  type       : character string
 !	  access     : read only
 !	  mechanism  : by descriptor
 !
 !	Name of a queue to submit a file to.
 !
 !	Modifiers
 !	  VMS Usage  : longword_unsigned
 !	  type       : longword (unsigned)
 !	  access     : read only
 !	  mechanism  : by reference, array reference
 !
 !	Longword array where the first longword is a count of how
 !	many boolean longword modifiers follow.  
 !
 !	For example:
 !
 !	+--------------------+
 !	|          2         |
 !      +--------------------+
 !      | SJC$_NO_LOG_DELETE |
 !      +--------------------+
 !      |     SJC$_NOTIFY    |
 !      +--------------------+
 !
 ! Implicit Inputs:
 !
 !	none
 !
 ! Implicit Outputs:
 !
 !	none
 !
 ! Completion status:
 !
 !	Any possible return value from SYS$SNDJBCW
 !
 !--

	DECLARE Item_List_3 Itemlist

	DECLARE BYTE Fn(254)
	DECLARE BYTE Qu(254)

	DECLARE LONG Dummy
	DECLARE LONG Fn_Len
	DECLARE LONG Loop
	DECLARE LONG Qu_Len
	DECLARE LONG Sts
	DECLARE LONG Sts2
	DECLARE LONG Sts2_Len

	EXTERNAL LONG CONSTANT SJC$_FILE_SPECIFICATION
	EXTERNAL LONG CONSTANT SJC$_JOB_STATUS_OUTPUT
	EXTERNAL LONG CONSTANT SJC$_QUEUE

	EXTERNAL LONG FUNCTION Sys_Report_Message(LONG BY VALUE)

	EXTERNAL LONG FUNCTION SYS$SNDJBCW(LONG BY VALUE, WORD BY VALUE, &
	                                   LONG BY VALUE, Item_List_3 BY REF, &
	                                   Iosb_Block BY REF, LONG BY REF, &
	                                   LONG BY VALUE)

	EXTERNAL WORD CONSTANT SJC$_ENTER_FILE

	RECORD Iosb_Block
	  WORD Datam(3)
	END RECORD

	RECORD Item_List_3
	  GROUP Item(254)
	    VARIANT
	    CASE
	      WORD Buf_len
	      WORD Code
	      LONG Buffer_Address
	      LONG Length_Address
	    CASE
	      LONG Terminator
	    END VARIANT
	  END GROUP
	END RECORD

	FOR Loop = 1 TO LEN(Queue)
	  Qu(Loop - 1) = ASC(MID$(Queue, Loop, 1))
	NEXT Loop

	FOR Loop = 1 TO LEN(Filename)
	  Fn(Loop - 1) = ASC(MID$(Filename, Loop, 1))
	NEXT Loop

	Itemlist::Item(0)::Buf_Len = LEN(Queue)
	Itemlist::Item(0)::Code = SJC$_QUEUE
	Itemlist::Item(0)::Buffer_Address = LOC(Qu(0))
	Itemlist::Item(0)::Length_Address = LOC(Qu_Len)

	Itemlist::Item(1)::Buf_Len = LEN(Filename)
	Itemlist::Item(1)::Code = SJC$_FILE_SPECIFICATION
	Itemlist::Item(1)::Buffer_Address = LOC(Fn(0))
	Itemlist::Item(1)::Length_Address = LOC(Fn_Len)

	FOR Loop = 1 TO Modifiers(0)
	  Itemlist::Item(Loop + 1)::Buf_Len = 4
          Itemlist::Item(Loop + 1)::Code = Modifiers(Loop)
          Itemlist::Item(Loop + 1)::Buffer_Address = LOC(Dummy)
          Itemlist::Item(Loop + 1)::Length_Address = LOC(Dummy)
	NEXT Loop

	Itemlist::Item(1 + Modifiers(0) + 1)::Terminator = 0

	Sys_Submit = SYS$SNDJBCW(, SJC$_ENTER_FILE,, Itemlist,,,)

	FUNCTIONEND
