	FUNCTION LONG SYS_TRNLOG(STRING Table, STRING Logical, STRING Trans)

 !++
 !
 ! Author:	Jonathan Welch
 !
 ! Functional Description:
 !
 ! Recursively translates the specified logical name in the specified
 ! logical name table and returns the final translation.
 !
 ! Formal Arguments:
 !
 !	Table
 !	  VMS Usage  : char_string
 !	  type       : character string
 !	  access     : read only
 !	  mechanism  : by descriptor
 !
 !	Name of a logical name table to search.
 !
 !	Logical
 !	  VMS Usage  : char_string
 !	  type       : character string
 !	  access     : read only
 !	  mechanism  : by descriptor
 !
 !	A logical name to recursively translate.
 !
 !	Trans
 !	  VMS Usage  : char_string
 !	  type       : character string
 !	  access     : write only
 !	  mechanism  : by descriptor
 !
 !	Variable to receive the final translation of the specified logical name.
 !
 ! Implicit Inputs:
 !
 !	none
 !
 ! Implicit Outputs:
 !
 !	none
 !
 ! Completion status:
 !
 !	SS$_NORMAL	Routine completed successfully.
 !
 !	Any possible return value from SYS$TRNLNM.
 !
 !--

	DECLARE Item_List_3 Itemlist

	DECLARE BYTE Chars(254)

	DECLARE LONG Loop
	DECLARE LONG Retlen
	DECLARE LONG Sts

	DECLARE STRING Logical2

	EXTERNAL LONG CONSTANT LNM$_STRING
	EXTERNAL LONG CONSTANT SS$_NORMAL

	EXTERNAL LONG FUNCTION SYS$TRNLNM(LONG BY REF, STRING BY DESC, &
	                                  STRING BY DESC, BYTE BY REF, &
	                                  Item_List_3 BY REF)

	RECORD Item_List_3
	  GROUP Item(1)
	    VARIANT
	    CASE
	      WORD Buf_len
	      WORD Code
	      LONG Buffer_Address
	      LONG Length_Address
	    CASE
	      LONG Terminator
	    END VARIANT
	  END GROUP
	END RECORD

	Itemlist::Item(0)::Buf_Len = 255
	Itemlist::Item(0)::Code = LNM$_STRING
	Itemlist::Item(0)::Buffer_Address = LOC(Chars(0))
	Itemlist::Item(0)::Length_Address = LOC(Retlen)

	Itemlist::Item(1)::Terminator = 0

	Sts = SYS$TRNLNM(,Table, Logical,, Itemlist)
	IF (Sts AND 1%) <> 1
	  THEN
	    Sys_Trnlog = Sts
	    FUNCTIONEXIT
	END IF

	WHILE (Sts AND 1%) <> 0

	  Logical2 = ""
	  FOR Loop = 0 TO Retlen - 1
	    Logical2 = Logical2 + CHR$(Chars(Loop))
	  NEXT Loop

	  Sts = SYS$TRNLNM(,Table, Logical2,, Itemlist)

	NEXT

	IF LEN(Trans) = 0
	  THEN
	    FOR Loop = 0 TO Retlen - 1
	      Trans = Trans + CHR$(Chars(Loop))
	    NEXT Loop
	  ELSE
	    FOR Loop = 0 TO Retlen - 1
	      MID$(Trans, Loop + 1, 1) = CHR$(Chars(Loop))
	    NEXT Loop
	END IF

	Sys_Trnlog = SS$_NORMAL

	FUNCTIONEND
