$ SET NOVER
$! COPY.COM	COPY WITH DUMMY FILE CONTAINMENT
$
$! The purpose of COPY.COM is to copy files to the output disk in a specific
$! order with contiguous space allocation. The actual copy is done with BACKUP
$! to preserve file dates, ACL's, etc. COPY/CONTIG cannot be used because VMS
$! may not start allocation at the beginning of the free space. Both BACKUP and
$! COPY cannot compensate for the directory expansion. 
$! 
$! Compensation for directory expansion represents the most difficult problem 
$! for this phase of the disk reorganization. To understand the problem one must
$! know how COPY.COM copies a file and how directories are modified by VMS. 
$! 
$! COPY creates a space large enough for the file by allocating a dummy file 
$! starting n blocks from the current beginning of the free space, where n is 
$! the number of blocks in the file. The length of the dummy is equal to the
$! total free disk blocks minus n. The starting position for the dummy is 
$! determined by keeping track of the lowest available free block. The object 
$! is to provide just enough space for the file. However, keeping all the space
$! allocated as described is not easy when the directory expands.
$! 
$! Directory expansion occurs when the directory does not have space to include
$! a new file. VMS accomplishes the expansion by copying the directory to a new
$! coniguous location with an allocation size equal to the current blocks plus 
$! one cluster. DISK_REO creates all of the directories in the center of the 
$! disk. Thus, each directory is one cluster in size after its initial 
$! creation. If the directory expands during COPY.COM it manifests itself in 
$! two ways:
$! 
$!   1.	If the free space created for the file is not large enough for the
$! 	directory expansion, VMS qives an ACP failure message and file full
$!	status. Your are left without the file copied and the directory in its
$! 	orginal place.
$! 
$!   2. If the free space created for the file is large enough for the
$! 	directory expansion, the directory expands into the space created
$! 	for the file and the file copy fails with a file full status.
$! 
$! Note that both failures present the same DCL status: file full.
$! 
$! Other surprises:
$! 
$!   1.	The output file size may be different from the input file size.
$! 
$!   2.	I have not found a way to pre-extend the length of the user directories 
$!	because free clusters are deallocated when a number of files are 
$!	deleted. So, the directories you want to keep centered must stay under
$!	one cluster in size.
$!	
$!	
$! The following diagram shows the output disk on entry to COPY.COM:
$!	
$! LBN: 0--768--------------------------445536--------------------------891071
$!	i-i111111111111		       d0biii2222222222222222222222222222
$!	-----------------------------------------------------------------
$!
$!	i-i	INDEXF.SYS + filler (DRO_DUMMY_16.DAT) + INDEXF.SYS.
$!	
$!	1...1	DUMMY_768_DATA.DAT to fill 768 to the start of the data files.
$!	
$!	spaces	Free space for 1/2 of the data files to be placed in the center.
$!	
$!	d0biii	Directories, subdirectories, [000000], BITMAP.SYS, INDEXF.SYS.
$!	
$!	2...2	DUMMY_DIR_END.DAT to fill from the end of INDEXF.SYS to 891072.
$! 
$!	
$! The following diagram shows the output disk on exit from COPY.COM:
$!	
$! LBN: 0--768--------------------------445536--------------------------891071
$!	i-i111111111111ffffffffffffffffd0biiiffffffffffffffff
$!	-----------------------------------------------------------------
$!
$!	i-i	INDEXF.SYS + filler (DRO_DUMMY_16.DAT) + INDEXF.SYS.
$!	
$!	1...1	DUMMY_768_DATA.DAT to fill 768 to the start of the data files.
$!	
$!	spaces	Free space.
$!	
$!	d0biii	Directories, subdirectories, [000000], BITMAP.SYS, INDEXF.SYS.
$!	
$! 	f...f	Centered data files. The first and last data file name is
$! 		passed to DISK_REO in global variables. This is used to create
$! 		the dummy file to reserve free space next to the data files.
$! 
$! Input parameters:
$
$!	P1 = File to calculate starting LBN
$!	P2 = File of file names to copy
$!	P3 = Dummy file specification
$
$	ON ERROR THEN GOTO BOMBER
$
$	OUT_DISK = F$EXTRACT(0, F$LOCATE("[", P3), P3)
$	
$	SHO SYM OUT_DISK
$	
$!	CALC STARTING LBN:
$
$	@GET_LBN 'P1'
$	
$	SHO SYM S_FILE_LBN
$	SHO SYM S_FILE_SIZE
$	
$	LBN = S_FILE_LBN + S_FILE_SIZE
$
$	LAST_LBN = LBN
$
$	WTO ""
$	WTO "LBN = ''LBN' ''S_FILE_LBN' ''S_FILE_SIZE'"
$	WTO ""
$
$!	CALC LOWER LIMIT OF ORGINAL DIRECTORY AREA 
$
$	LOWER_DIR_LIMIT = LBN + F$GETDVI("''OUT_DISK'", "FREEBLOCKS")
$	
$!	CALC UPPER LIMIT OF ORGINAL DIRECTORY AREA 
$
$	@GET_LBN 'OUT_DISK'[000000]DUMMY_DIR_END.DAT
$	
$	SHO SYM S_FILE_LBN
$	SHO SYM S_FILE_SIZE
$	
$	UPPER_DIR_LIMIT = S_FILE_LBN - 1
$	
$	LAST_DIR_SPEC = ""
$
$	LAST_DIR_LBN  = 0
$
$	COPY_FILE_CNT = 0
$
$	COPY_FILE_SIZE_CNT = 0
$
$	OPEN/READ ZONE_DATA 'P2'	! ACCESS LIST OF FILES TO COPY
$
$	ON ERROR THEN GOTO DIR_FAIL
$
$ COP_LOOP:
$	
$	READ/END=CLOSE_LOG_FILE ZONE_DATA ZD_REC
$
$	FILE = F$EDIT(F$EXTRACT(9, 80, ZD_REC), "TRIM")
$
$	FILE = F$EXTRACT(0, F$LOCATE(" ", FILE), FILE)
$	SHO SYM FILE
$ 	F_SIZE = F$INTEGER( F$EXTRACT( 117, 7, ZD_REC))
$	
$	! SAVE FIRST AND LAST DATA FILE FOR REAL FREE AREA DUMMY CREATION:
$	
$	IF S_FIRST_DATA_FILE .EQS. "" THEN S_FIRST_DATA_FILE == FILE
$	
$	S_LAST_DATA_FILE == FILE
$	
$ SPACE_OUT:
$	
$	OUT_DISK_FREE = F$GETDVI("''OUT_DISK'", "FREEBLOCKS")
$
$	IF F_SIZE .LE. OUT_DISK_FREE THEN GOTO SPACE_OK
$	
$	SET VER
$	
$	! TIE UP CURRENT HOLE:
$	
$	SH DEV 'OUT_DISK'
$	
$	IF OUT_DISK_FREE .GT. 0 THEN -
	  @GEN_DUMMY2 DUMMY.FDL 'OUT_DISK_FREE' 'OUT_DISK'[000000]DUMMY_DIR.FILL
$
$	! OPEN UP NEW HOLE:
$	
$	SH DEV 'OUT_DISK'
$	
$	@GET_LBN 'OUT_DISK'[000000]DUMMY_DIR_END.DAT
$	
$	SHO SYM S_FILE_LBN
$	SHO SYM S_FILE_SIZE
$	
$	LBN = S_FILE_LBN
$	
$	SHO SYM LBN
$	
$	DEL 'OUT_DISK'[000000]DUMMY_DIR_END.DAT;*/LOG
$
$	SH DEV 'OUT_DISK'
$	
$	OUT_DISK_FREE = F$GETDVI("''OUT_DISK'", "FREEBLOCKS")
$
$	SET NOVER
$	
$ SPACE_OK:
$	
$	LAST_LBN = LBN
$
$	LBN = LBN + F_SIZE
$
$	DSZ = OUT_DISK_FREE - F_SIZE
$
$	WTO "''FILE'	''OUT_DISK_FREE'	''F_SIZE'	''DSZ	''LBN'"
$	
$	IF DSZ .GT. 0 THEN @GEN_DUMMY DUMMY.FDL 'DSZ' 'LBN' 'P3'
$
$	BACKUP 'FILE' 'OUT_DISK'[*...]/OWN=ORIG
$
$	GOSUB ADD_CNT
$
$	GOSUB CHECH_OUT_SIZE
$
$	IF DSZ .GT. 0 THEN DEL 'P3';*
$
$	GOTO COP_LOOP
$	
$ DIR_FAIL:
$	
$	T_STATUS = $STATUS
$	
$	SHO SYM T_STATUS
$	
$	SET VER
$	
$	SH DEV 'OUT_DISK'
$	
$	U_DIR = F$EXTRACT( F$LOCATE("[",FILE) + 1, 99, FILE)
$
$	U_DIR = F$EXTRACT( 0, F$LOCATE("]",U_DIR), U_DIR)
$	
$	IF F$LOCATE(".", U_DIR) .EQ. F$LENGTH(U_DIR) THEN GOTO NO_SUB
$	
$	T_POS = F$LENGTH(U_DIR) - 1
$	
$ SUB_HUNT:
$	
$	IF F$EXTRACT( T_POS,1, U_DIR) .EQS. "." THEN GOTO SUB_HIT
$	
$	T_POS = T_POS - 1
$
$	GOTO SUB_HUNT
$	
$ SUB_HIT:
$	
$	U_ROOT = F$EXTRACT( 0, T_POS, U_DIR)
$
$	U_DIR  = F$EXTRACT( T_POS + 1, 99, U_DIR)
$
$	DIR_SPEC = OUT_DISK + "[" + U_ROOT + "]" + U_DIR + ".DIR"
$	
$	GOTO GET_DIR_SIZE
$	
$ NO_SUB:
$
$	DIR_SPEC = OUT_DISK + "[000000]" + U_DIR + ".DIR"
$	
$ GET_DIR_SIZE:
$	
$	@GET_LBN 'DIR_SPEC'
$
$	SHO SYM S_FILE_LBN
$	SHO SYM S_FILE_SIZE
$
$	IF S_FILE_LBN .LT. LOWER_DIR_LIMIT 	.OR. -
	   S_FILE_LBN .GT. UPPER_DIR_LIMIT	THEN GOTO FILE_FULL
$
$ FIRST_OFFENSE:
$
$	! HANDLE DIR ACP FAIL:
$
$	DIR_SIZE = F$FILE_ATTRIBUTES( "''DIR_SPEC'","ALQ")
$
$	LBN = LAST_LBN
$	
$	F_SIZE = F_SIZE + DIR_SIZE + S_CLUSTER ! CLUSTER GLOBAL FROM DISK_REO
$	
$	LBN = LBN + F_SIZE
$	
$	DSZ = OUT_DISK_FREE - F_SIZE
$
$	WTO "* ''FILE'	''OUT_DISK_FREE'	''F_SIZE'	''DSZ	''LBN'"
$	
$	SH DEV 'OUT_DISK'
$	
$	DEL 'P3';*
$
$	SH DEV 'OUT_DISK'
$	
$	@GEN_DUMMY DUMMY.FDL 'DSZ' 'LBN' 'P3'
$
$	SH DEV 'OUT_DISK'
$	
$	BACKUP 'FILE' 'OUT_DISK'[*...]/OWN=ORIG
$
$	GOSUB ADD_CNT
$
$	SH DEV 'OUT_DISK'
$
$	@GEN_DUMMY2 DUMMY.FDL 'DIR_SIZE' 'OUT_DISK'[000000]DUMMY_DIR.FILL
$
$	SH DEV 'OUT_DISK'
$	
$	DEL 'P3';*
$
$	SH DEV 'OUT_DISK'
$	
$	SET NOVER
$	
$	GOSUB SAVE_DIR_INFO
$	
$	ON ERROR THEN GOTO DIR_FAIL
$
$	GOTO COP_LOOP
$	
$ FILE_FULL:	! WILL DUMMY ENTIRE FREE SPACE BECAUSE I DO NOT KNOW IF THE
$		! DIR WILL ALWAYS BE LOCATED AT THE LOW END. 
$	
$	! CHECK FOR SECOND OFFENSE. IF THE DIR DID NOT MOVE HANDLE AS FIRST:
$	
$	IF DIR_SPEC   .EQS. LAST_DIR_SPEC .AND. -
	   S_FILE_LBN .EQ.  LAST_DIR_LBN  THEN GOTO FIRST_OFFENSE
$	
$	SH DEV 'OUT_DISK'
$	
$	T_BLK = F$GETDVI("''OUT_DISK'", "FREEBLOCKS")
$
$	SHO SYM T_BLK
$
$	@GEN_DUMMY3 	DUMMY3.FDL			-
			'T_BLK'				-
			'OUT_DISK'[000000]DUMMY_DIR.FILL
$
$	SH DEV 'OUT_DISK'
$	
$	@GET_LBN 'P3'
$	
$	SHO SYM S_FILE_LBN
$	SHO SYM S_FILE_SIZE
$	
$	LBN = S_FILE_LBN
$	
$	DEL 'P3';*
$
$	SH DEV 'OUT_DISK'
$	
$	SET NOVER
$	
$	GOSUB SAVE_DIR_INFO
$	
$	ON ERROR THEN GOTO DIR_FAIL
$
$	GOTO SPACE_OUT
$	
$ CLOSE_LOG_FILE:
$
$	CLOSE ZONE_DATA
$
$	SHO SYM COPY_FILE_CNT
$
$	SHO SYM COPY_FILE_SIZE_CNT
$
$ 	SET VER
$
$ 	EXIT 1
$
$ CHECH_OUT_SIZE:
$
$ 	OUT_F_SIZE = -
     F$FILE_ATTRIBUTES("''OUT_DISK'''F$EXTRACT(F$LOCATE("[",FILE), 99, FILE)'",-
     "ALQ")
$	
$	IF F_SIZE .EQ. OUT_F_SIZE THEN RETURN
$	
$	LBN = LBN + (OUT_F_SIZE - F_SIZE)
$	
$SHO SYM OUT_F_SIZE
$	
$	RETURN
$
$ SAVE_DIR_INFO:
$
$	@GET_LBN 'DIR_SPEC'
$
$	SHO SYM S_FILE_LBN
$	SHO SYM S_FILE_SIZE
$
$	LAST_DIR_LBN  = S_FILE_LBN
$
$	LAST_DIR_SPEC = DIR_SPEC
$
$	RETURN
$
$ ADD_CNT:
$
$	COPY_FILE_CNT = COPY_FILE_CNT + 1
$
$	COPY_FILE_SIZE_CNT = COPY_FILE_SIZE_CNT + F_SIZE
$
$	RETURN
$
$ BOMBER:
$
$	SET VERIFY
$
$	EXIT 2
$
