/*******************************************************************************
*	BATCHACC is run in SYSLOGIN.COM for batch jobs only.  It determines
*  the account number of the submitting process at the time of submission, then 
*  it sets the account of the batch job to that of the submitting parent.
*
*	This was necessary since the default action taken by the system is to
*  set the batch job account to the default stored in the UAF.  NOTE;
*
*		- The users should realize that the batch job will take on the
*		  account # that they are using at the time of submission.
*
*		- Some accumulation of time will still be accrued to the account
*		  # in the UAF, since part of SYSLOGIN.COM is executed before
*		  BATCHACC.
*
*		- BATCHACC assumes that the account number of the parent is
*		  valid, and does not perform any validity checking.
*
*******************************************************************************/
#include <ssdef.h>
#include <descrip.h>

#define	NBRITEMS		2	/* number of getqui items req.	*/
#define ACCTSZ			8	/* size of account field (may change) */
#define	QUI$_SEARCH_FLAGS	76	/* item 1			*/
#define	QUI$_ACCOUNT_NAME	1	/* item 2			*/
#define	QUI$M_SEARCH_THIS_JOB	16
#define	QUI$_DISPLAY_JOB	5
#define JBC$_NORMAL		262145

int	status;
char	parent_acc[9];

main()
{
	/* Find account number of parent at time of submission */
	do_qui();

	/* Set account number in this process */
	status = setcharge(parent_acc);

	exit(status);
}

do_qui()
{
int	flags;
int	sys$getqui();		/* getqui system service returns an int	*/
short	return_len;
int	iosb[2];

struct	ITEM
{
	short	len;		/* buffer length			*/
	short	item_code;	/* information item requested		*/
	char	*buf;		/* return buffer			*/
	int	*rlen;		/* return length			*/
};

struct
{
	struct	ITEM items[NBRITEMS];
	int	endoflist;
} itemlist				=
					{
	/* search flags	*/	4 , QUI$_SEARCH_FLAGS	,&flags ,0
	/* account name	*/	,8 ,QUI$_ACCOUNT_NAME	,parent_acc ,&return_len
					};


	itemlist.endoflist = 0;
	flags = QUI$M_SEARCH_THIS_JOB;

	status = sys$getquiw(0,QUI$_DISPLAY_JOB,0,&itemlist,&iosb,0,0);
	if (iosb[0] != JBC$_NORMAL)
		exit(iosb[0]);

	/* Make sure that the result is null terminated */
	if ((return_len > 0) && (return_len <= ACCTSZ))
		parent_acc[return_len] = 0;

	return(SS$_NORMAL);
}

setcharge(chargecode)
	char *chargecode;
	{
	struct dsc$descriptor acctdesc;	/* for passing to setacct() */
	char acctno[ACCTSZ];		/* holds blank-filled chargecode */
	register int i;			/* loop index */

	/*
	 *	Copy and blank fill the chargecode.
	 */
	strcpy(acctno,chargecode);
	for (i=strlen(chargecode); i<ACCTSZ; i++)
		acctno[i] = ' ';
	/*
	 *	Build the descriptor.
	 */
	acctdesc.dsc$w_length = ACCTSZ;
	acctdesc.dsc$b_dtype = DSC$K_DTYPE_T;
	acctdesc.dsc$b_class = DSC$K_CLASS_S;
	acctdesc.dsc$a_pointer = &acctno[0];

	/*
	 *	Call setacct to do the actual work.
	 */
	return(setaccount(&acctdesc));
}
