/*******************************************************************************
*	LIMITER controls the number of users signed onto a single usercode,
*  and also provides a "barometer" of system performance.
*
*	If users are signed on multiple times, and this count exceeds the
*  UAF field MAXDETACH, then the user may optionally be signed off.  MAXDETACH
*  is used for this purpose since we don't use it for anything else.  In report
*  mode, LIMITER only informs the user that they are signed on n terminals
*  already, and they should reconsider their needs.
*
*	The performance barometer is simply the number of processes in the
*  compute queue, excluding the NULL process.  Each number (up to 10) is given
*  a rating, with 10 and over being the worst case.
*******************************************************************************/
# include	<jpidef.h>
# include	<ssdef.h>
# include	<stdio.h>
# include	<desc.h>
# include	<ctype.h>

# define	SCH$C_COM	12
# define	SCH$C_COMO	13
# define	TRUE		 1
# define	FALSE		 0
# define	LIMIT		TRUE

main()
{
	int	com_count, state, census, mode, owner, max_detach;
	int	prime_time = FALSE;
	int	pidcontext, counter, day_of_week, bin_time[2];
	short	status, minute_of_day, num_time[7];
	char	username[13], my_username[13], my_terminal[8];
	char	plural[] = "s";

struct	ITEM
{
	short	len;			/* buffer length */
	short	item_code;		/* information item requested */
	char	*buf;			/* return buffer */
	int	*rlen;			/* return length */
};

struct
{
	struct	ITEM items[3];
	int	endoflist;
} itemlist1				=
					{
		/* detached qta	*/	4 ,JPI$_MAXDETACH,&max_detach	,0
		/* username	*/	,12,JPI$_USERNAME,my_username	,0
		/* terminal	*/	,7,JPI$_TERMINAL,my_terminal	,0
					};

struct
{
	struct	ITEM items[4];
	int	endoflist;
} itemlist2				=
					{
		/* owner pid	*/	4 ,JPI$_OWNER	,&owner		,0
		/* job mode	*/	,4 ,JPI$_MODE,&mode	 	,0
		/* state	*/	,4 ,JPI$_STATE,&state	 	,0
		/* username	*/	,12,JPI$_USERNAME,username	,0
					};


/* First, find out who I am */
	itemlist1.endoflist = 0;
	status = sys$getjpiw(0,0,0,&itemlist1,0,0,0);
	if (status != SS$_NORMAL)
		{
		printf ("GETJPI status of %d for me\n",status);
		exit(status);
		}
	my_username[12] = 0;

/* Next, find out if this is prime time */
	sys$gettim(&bin_time);
	sys$numtim(&num_time,&bin_time);
	lib$day_of_week(&bin_time,&day_of_week);
	minute_of_day = (60 * num_time[3]) + num_time[4];
	if ((day_of_week < 6) &&
	    (minute_of_day > 449) &&
	    (minute_of_day < 976))
		prime_time = TRUE;

/* Next, look through all processes */
	itemlist2.endoflist = status = census = com_count = 0;
	for (pidcontext = -1;status != SS$_NOMOREPROC;)
		{
		status = sys$getjpiw(1,&pidcontext,0,&itemlist2,0,0,0);
		if (status != SS$_NORMAL)
			continue;
		username[12] = 0;

		if ((state == SCH$C_COM) ||
		    (state == SCH$C_COMO))
			com_count++;

		if ((mode == JPI$K_INTERACTIVE) &&
		    (owner == 0)		&&
		    (strcmp(username,my_username) == 0))
			census++;
		}

/* At minimum, chew out the user, else blow this greedy user off the system */
	if (census > 1)
		{
		if (census == 2)
			plural[0] = ' ';
		printf("\n****************************");
		printf("************************************\n");
		printf("NOTE: ");
		printf("This usercode is already signed on %d",census - 1);
		printf(" other terminal%s\n", plural);

/* If limiting is enabled, this is prime time, limiting is enabled for this user
   and this isn't OPA0: or usercode SYSTEM, then log them off! */
		if ((LIMIT == TRUE) &&
		    (prime_time == TRUE) &&
		    (max_detach > 0) &&
		    (census > max_detach) &&
		    (strncmp(my_terminal,"OPA0",4) != 0) &&
		    (strncmp(my_username,"SYSTEM",6) != 0))
			{
			printf("Your process is being deleted.\n");
			printf("****************************");
			printf("************************************\n");
			sleep(1);	/* allow time for message to sink in */
			sys$delprc(0,0);
			}

		printf("Please evaluate your needs and weigh them against \n");
		printf("the needs of the other users.\n");
		printf("****************************");
		printf("************************************\n");
		}

	printf("\nThe system performance rating is: %d ",com_count - 1);
	switch(com_count - 1)
		{
		case 0:
			printf("(Idle)\n");
			break;
		case 1:
			printf("(Almost idle)\n");
			break;
		case 2:
			printf("(Almost idle)\n");
			break;
		case 3:
			printf("(Lightly loaded)\n");
			break;
		case 4:
			printf("(Moderately loaded)\n");
			break;
		case 5:
			printf("(Well loaded)\n");
			break;
		case 6:
			printf("(Well loaded)\n");
			break;
		case 7:
			printf("(Heavily loaded)\n");
			break;
		case 8:
			printf("(Slightly overloaded)\n");
			break;
		case 9:
			printf("(Overloaded)\n");
			break;
		case 10:
			printf("(Extremely overloaded)\n");
			break;
		default:
			printf("(Dead in the water)\n");
			break;
		}

	exit(SS$_NORMAL);
}
