PROGRAM local_print (INPUT, OUTPUT, infile);

CONST
   nul            = chr(00);
   linefeed       = chr(10);
   formfeed       = chr(12);
   carriage       = chr(13);
   ctrlo          = chr(15);
   ctrlr          = chr(18);
   esc            = chr(27);
   prsm1          = chr(29);
   prsm2          = chr(30);
   prsm3          = chr(31);
   prsm4          = chr(01);
   prsm5          = chr(02);

   size_filespec  = 64;
   size_buffer    = 1024;
   size_control   = 5;
   size_files     = 50;

   sys_tparse       = %X00158284;
   sys_normal       = %X00000001;

   null_file      = '                                                                ';
   prompt_file    = 'Filespec: ';


TYPE
   ubyte           = 0..255;
   word            = -32768..32767;
   string_filespec = PACKED ARRAY [1..size_filespec] OF char;
   array_files     =        ARRAY [1..size_files]    OF string_filespec;
   buffer_qio      = PACKED ARRAY [1..size_buffer]   OF char;
   buffer_control  = PACKED ARRAY [1..size_control]  OF char;

VAR

   nltr       : boolean := false;
   emph       : boolean := false;
   bold       : boolean := false;
   dbls       : boolean := false;
   comp       : boolean := false;
   width      : boolean := false;
   npage      : boolean := false;
   prntr      : boolean := false;
   prism      : boolean := false;
   la50       : boolean := false;
   la100      : boolean := false;
   epson      : boolean := false;
   star       : boolean := false;
   ibmp       : boolean := false;

   bufflen    ,
   loopcnt    ,
   lastspace  ,
   length_str ,
   prnt_width ,
   size_page  ,
   return     ,
   prnt_style ,
   curpage    ,
   parse      ,
   scrn_width : integer;

   width_str   : packed array [1..4] of char;

   channel    : word;
   infile     : text;
   inchar     : char;
   err_code   : unsigned;
   line       : integer;
   count      : integer;
   max_files  : integer;
   files      : array_files;
   iobuffer   : buffer_qio;
   la_elite_on     : buffer_control := (esc, '[', '2', 'w', nul);
   la_dblw_on      : buffer_control := (esc, '[', '5', 'w', nul);
   la_dblh_on      : buffer_control := (esc, '[', '5', 'z', nul);
   la_wide_on      : buffer_control := (esc, '[', '8', 'w', nul);
   la_high_on      : buffer_control := (esc, '[', '6', 'z', nul);
   la_bold_on      : buffer_control := (esc, '[', '1', 'm', nul);
   la_bold_off     : buffer_control := (esc, '[', '2', '2', 'm');
   la_ltr_on       : buffer_control := (esc, '[', '2', '"', 'z');
   la_ltr_off      : buffer_control := (esc, '[', '0', '"', 'z');
   comphorz        : buffer_control := (esc, '[', '4', 'w', nul);
   normhorz        : buffer_control := (esc, '[', '0', 'w', nul);
   compvert        : buffer_control := (esc, '[', '2', 'z', nul);
   normvert        : buffer_control := (esc, '[', '0', 'z', nul);
   clmns_132       : buffer_control := (esc, '[', '?', '3', 'h');
   clmns_80        : buffer_control := (esc, '[', '?', '3', 'l');
   star_dbls_on    : buffer_control := (esc, 'G', nul, nul, nul);
   star_dbls_off   : buffer_control := (esc, 'H', nul, nul, nul);
   star_emph_on    : buffer_control := (esc, 'E', nul, nul, nul);
   star_emph_off   : buffer_control := (esc, 'F', nul, nul, nul);
   star_nltr_on    : buffer_control := (esc, '4', nul, nul, nul);
   star_nltr_off   : buffer_control := (esc, '5', nul, nul, nul);
   star_132_on     : buffer_control := (ctrlo, nul, nul, nul, nul);
   star_132_off    : buffer_control := (ctrlr, nul, nul, nul, nul);
   star_elite_on   : buffer_control := (esc, 'B', '2', nul, nul);
   star_dblw_on    : buffer_control := (esc, 'W', '1', nul, nul);
   star_dblw_off   : buffer_control := (esc, 'W', '0', nul, nul);
   ibmp_elite_on   : buffer_control := (esc, ':', nul, nul, nul);
   ibmp_dblw_on    : buffer_control := (esc, 'W', '1', nul, nul);
   ibmp_dblw_off   : buffer_control := (esc, 'W', '0', nul, nul);
   ibmp_132_on     : buffer_control := (ctrlo, nul, nul, nul, nul);
   ibmp_132_off    : buffer_control := (ctrlr, nul, nul, nul, nul);
   ibmp_emph_on    : buffer_control := (esc, 'E', nul, nul, nul);
   ibmp_emph_off   : buffer_control := (esc, 'F', nul, nul, nul);
   ibmp_nltr_on    : buffer_control := (esc, 'G', nul, nul, nul);
   ibmp_nltr_off   : buffer_control := (esc, 'H', nul, nul, nul);
   prsm_elite_on   : buffer_control := (prsm2, nul, nul, nul, nul);
   prsm_emph_on    : buffer_control := (esc, 'R', '2', '$', prsm1);
   prsm_emph_off   : buffer_control := (esc, 'R', '1', '$', prsm1);
   prsm_132_on     : buffer_control := (prsm3, nul, nul, nul, nul);
   prsm_132_off    : buffer_control := (prsm1, nul, nul, nul, nul);
   prsm_dblw_on    : buffer_control := (prsm4, nul, nul, nul, nul);
   prsm_dblw_off   : buffer_control := (prsm5, nul, nul, nul, nul);
   epsn_elite_on   : buffer_control := (esc, 'M', nul, nul, nul);
   epsn_dblw_on    : buffer_control := (esc, 'W', '1', nul, nul);
   epsn_dblw_off   : buffer_control := (esc, 'W', '0', nul, nul);
   epsn_132_on     : buffer_control := (ctrlo, nul, nul, nul, nul);
   epsn_132_off    : buffer_control := (ctrlr, nul, nul, nul, nul);
   epsn_emph_on    : buffer_control := (esc, 'E', nul, nul, nul);
   epsn_emph_off   : buffer_control := (esc, 'F', nul, nul, nul);
   epsn_nltr_on    : buffer_control := (esc, 'x', '1', nul, nul);
   epsn_nltr_off   : buffer_control := (esc, 'x', '0', nul, nul);
   epsn_dbls_on    : buffer_control := (esc, 'G', nul, nul, nul);
   epsn_dbls_off   : buffer_control := (esc, 'H', nul, nul, nul);
   start_ctrl      : buffer_control := (esc, '[', '5', 'i', nul);
   end_ctrl        : buffer_control := (esc, '[', '4', 'i', nul);
   pagebreak       : buffer_control := (formfeed, nul, nul, nul, nul);

FUNCTION cli$present (
  %STDESCR name   : PACKED ARRAY [l1..u1 : INTEGER] OF CHAR) : INTEGER;
  EXTERN;

FUNCTION cli$get_value (
  %STDESCR name   : PACKED ARRAY [l1..u1 : INTEGER] OF CHAR;
           retbuf : PACKED ARRAY [l2..u2 : INTEGER] OF CHAR) : INTEGER;
  EXTERN;

FUNCTION load_number_parse (
  %STDESCR name   : PACKED ARRAY [l1..u1 : INTEGER] OF CHAR;
           count  : INTEGER) : INTEGER;
  EXTERN;

(*	Check to see if command language interpretter option is selected.  *)

FUNCTION check_cli(
          string  : PACKED ARRAY [l1..u1 : INTEGER] OF CHAR) : BOOLEAN;

  BEGIN
  return := cli$present (string);
  IF ODD(return) THEN
    check_cli := TRUE
  ELSE
    check_cli := FALSE;
  END;

(*	Perform checks of command language interpretter.	*)

PROCEDURE perform_checks;

  BEGIN

(*	Check to see if emphasized type is required on printout		*)
     IF check_cli( 'EMPH' ) THEN emph := true;

(*	Check to see if BOLD type is required on printout		*)
     IF check_cli( 'BOLD' ) THEN bold := true;

(*	Check to see if double-strike is required on printout		*)
     IF check_cli( 'DOUBLE' ) THEN dbls := true;

(*	Check to see if letter quality type is required on printout	*)
     IF check_cli( 'NLTR' ) THEN nltr := true;

(*	Check to see if printer is LA50 matrix printer look alike.	*)
     IF check_cli( 'LA50' ) THEN la50 := true;

(*	Check to see if printer is LA100 matrix printer look alike.	*)
     IF check_cli( 'LA100' ) THEN la100 := true;
     IF la100 THEN prntr := true;

(*	Check to see if printer is IBM matrix printer look alike.	*)
     IF check_cli( 'IBM' ) THEN ibmp := true;
     IF ibmp THEN prntr := true;

(*	Check to see if printer is EPSON matrix printer look alike.	*)
     IF check_cli( 'EPSON' ) THEN epson := true;
     IF epson THEN prntr := true;

(*	Check to see if printer is PRISM matrix printer look alike.	*)
     IF check_cli( 'PRISM' ) THEN prism := true;
     IF prism THEN prntr := true;

(*	Check to see if printer is STAR matrix printer look alike.	*)
     IF check_cli( 'STAR' ) THEN star := true;
     IF star THEN prntr := true;

(*	Check to see if compression of print is requested.	*)
     IF check_cli( 'COMPR' ) THEN comp := true;
     IF comp THEN prnt_width := 132
     ELSE prnt_width := 80;

(*	Check to see if printer should page before starting to print.	*)
     IF check_cli( 'PAGE' ) THEN npage := true;

  END;

PROCEDURE check_width;

  BEGIN
  prnt_style := 10;
  width := false;
  return := cli$present ('WIDTH');
  IF ODD(return) THEN
    BEGIN
    return := cli$get_value ('WIDTH', width_str);
    IF ODD(return) THEN
      BEGIN
        parse := load_number_parse (width_str, prnt_width);
        width := true;
        IF NOT ODD(parse) THEN
           BEGIN
              scrn_width := 132;
              prnt_width := 80;
           END
        ELSE
           IF PRNT_WIDTH <=80 THEN SCRN_WIDTH := 80
           ELSE SCRN_WIDTH := 132;
        prnt_style := trunc((prnt_width * 17.0) / 132.0);
      END
    ELSE
      BEGIN
         scrn_width := 80;
      END;    
    END
  END;

(************************************************************************)
(*			     GET_REQUESTS				*)
(************************************************************************)

PROCEDURE get_requests;

   VAR
      temp_file : string_filespec;
      i         : integer;

   BEGIN
   FOR i := 1 TO size_files DO files [i] := null_file;

   max_files := 0;
   write (prompt_file);
   WHILE (not eoln) and (max_files < size_files) DO
     BEGIN
     readln (temp_file);
     max_files := max_files + 1;
     files[max_files] := temp_file;
     write (prompt_file)
     END;
   readln
   END;


(************************************************************************)
(*			  ENTER_PRINT_CTRL				*)
(************************************************************************)

PROCEDURE enter_print_ctrl;

VAR

   i   : integer;

   PROCEDURE enter_dec_ctrl;
     BEGIN

       write( start_ctrl:size_control);
       if not(comp) and width then
          BEGIN
             if prnt_style <= 5 then
                BEGIN
                   write( la_dblw_on:size_control);
                   write( la_dblh_on:size_control);
                   size_page := 30;
                END;
             if (prnt_style > 5) and (prnt_style <= 9) then
                BEGIN
                   write( la_wide_on:size_control);
                   write( la_high_on:size_control);
                   size_page := 40;
                END;
             if (prnt_style > 10) and (prnt_style <= 13) then
                   write( la_elite_on:size_control);
             if (prnt_style > 13) then
                BEGIN
                   write( compvert:size_control);
                   write( comphorz:size_control);
                   size_page := 80;
                END;
          END
       ELSE
          if comp then
             BEGIN
               size_page := 80;
               write( compvert:size_control);
               write( comphorz:size_control);
             END;

       if nltr then emph := true;       
       if bold and not(emph) then write( la_bold_on:size_control);
       if not(bold) and emph then write( la_ltr_on:size_control);
       if bold and emph then write( la_ltr_on:size_control);

     END;

   PROCEDURE enter_ibm_ctrl;
     BEGIN

       write( start_ctrl:size_control);
       if emph then bold := true;
       if dbls then nltr := true;
       if comp or width then
         BEGIN
            if not(comp) then
               BEGIN
                 if prnt_style > 13 then
                    write( ibmp_132_on:size_control);
                 if (prnt_style > 10) and (prnt_style <= 13) then
                    write( ibmp_elite_on:size_control);
                 if (prnt_style > 5) and (prnt_style <= 9) then
                    BEGIN
                       write( ibmp_132_on:size_control);
                       write( ibmp_dblw_on:size_control);
                    END;
                 if prnt_style <= 5 then
                    write( ibmp_dblw_on:size_control);
               END
            ELSE
               write( ibmp_132_on:size_control);
            if bold then write( ibmp_emph_on:size_control);
         END
       ELSE
         BEGIN
            if not(bold) and nltr then write( ibmp_nltr_on:size_control);
            if bold and not(nltr) then write( ibmp_emph_on:size_control);
            if bold and nltr then write( ibmp_nltr_on:size_control);
         END;

     END;

   PROCEDURE enter_star_ctrl;
     BEGIN

       write( start_ctrl:size_control);
       if bold then emph := true;
       if comp or width then
         BEGIN
            if not(comp) then
               BEGIN
                 if prnt_style > 13 then
                    write( star_132_on:size_control);
                 if (prnt_style > 10) and (prnt_style <= 13) then
                    write( star_elite_on:size_control);
                 if (prnt_style > 5) and (prnt_style <= 9) then
                    BEGIN
                       write( star_132_on:size_control);
                       write( star_dblw_on:size_control);
                    END;
                 if prnt_style <= 5 then
                    write( star_dblw_on:size_control);
               END
            ELSE
                  write( star_132_on:size_control);
            if dbls then write( star_dbls_on:size_control);
         END
       ELSE
         BEGIN
           if nltr and not(emph) then write( star_nltr_on:size_control);
           if not(nltr) and emph then write( star_emph_on:size_control);
           if emph and nltr      then write( star_nltr_on:size_control);
         END;

     END;

   PROCEDURE enter_prsm_ctrl;
     BEGIN

       write( start_ctrl:size_control);
       if comp or width then
            if not(comp) then
               BEGIN
                 if prnt_style > 13 then
                    write( prsm_132_on:size_control);
                 if (prnt_style > 10) and (prnt_style <= 13) then
                    write( prsm_elite_on:size_control);
                 if (prnt_style > 5) and (prnt_style <= 9) then
                    BEGIN
                       write( prsm_132_on:size_control);
                       write( prsm_dblw_on:size_control);
                    END;
                 if prnt_style <= 5 then
                    write( prsm_dblw_on:size_control);
               END
            ELSE
                  write( prsm_132_on:size_control);
     END;

   PROCEDURE enter_epson_ctrl;
     BEGIN

       write( start_ctrl:size_control);
       if emph then bold := true;
       if comp or width then
         BEGIN
            if not(comp) then
               BEGIN
                 if prnt_style > 13 then
                    write( epsn_132_on:size_control);
                 if (prnt_style > 10) and (prnt_style <= 13) then
                    write( epsn_elite_on:size_control);
                 if (prnt_style > 5) and (prnt_style <= 9) then
                    BEGIN
                       write( epsn_132_on:size_control);
                       write( epsn_dblw_on:size_control);
                    END;
                 if prnt_style <= 5 then
                    write( epsn_dblw_on:size_control);
               END
            ELSE
               write( epsn_132_on:size_control);
            if dbls then write( epsn_dbls_on:size_control);
            if bold then write( epsn_emph_on:size_control);
          END
       ELSE
         BEGIN
            if not(nltr) and (bold or dbls) then
               BEGIN
		  if bold then write( epsn_emph_on:size_control);
                  if dbls then write( epsn_dbls_on:size_control);
               END;
            if nltr then write( epsn_nltr_on:size_control);
         END;

     END;

   BEGIN

      if (scrn_width > 80) or comp THEN write( clmns_132:size_control);
      size_page := 60;
      WRITELN;
      writeln( ' FILES TO BE PRINTED:  ');
      WRITELN;
      FOR i := 1 TO max_files DO
            writeln( '       ', files[i]);

      IF ibmp                    THEN enter_ibm_ctrl;
      IF star                    THEN enter_star_ctrl;
      IF epson                   THEN enter_epson_ctrl;
      IF prism                   THEN enter_prsm_ctrl;
      IF la50                    THEN enter_dec_ctrl;
      IF la100                   THEN enter_dec_ctrl;

   END;

(************************************************************************)
(*			  EXIT_PRINT_CTRL				*)
(************************************************************************)

PROCEDURE exit_print_ctrl;

   PROCEDURE exit_dec_ctrl;
     BEGIN

       if bold and not(emph) then write( la_bold_off:size_control);
       if not(bold) and emph then write( la_ltr_off:size_control);
       if bold and emph then write( la_ltr_off:size_control);

       IF comp or (scrn_width=132) THEN
         BEGIN
           write( normvert:size_control);
           write( normhorz:size_control);
         END;

       writeln( end_ctrl:size_control);

     END;

   PROCEDURE exit_ibm_ctrl;
     BEGIN

       if bold and not(nltr) then write( ibmp_emph_off:size_control);
       if not(bold) and nltr then write( ibmp_nltr_off:size_control);
       if bold and nltr then write( ibmp_nltr_off:size_control);
       if prnt_style < 10 then write( ibmp_dblw_off:size_control);

       if comp or (scrn_width=132) then
            write( ibmp_132_off:size_control);

       writeln( end_ctrl:size_control);

     END;

   PROCEDURE exit_star_ctrl;
     BEGIN

       if emph and not(nltr) then write( star_emph_off:size_control);
       if not(bold) and nltr then write( star_nltr_off:size_control);
       if bold and nltr then write( star_nltr_off:size_control);
       if prnt_style < 10 then write( star_dblw_off:size_control);

       if comp or (scrn_width=132) then
            write( star_132_off:size_control);

       if dbls then write( star_dbls_off:size_control);

       writeln( end_ctrl:size_control);

     END;

   PROCEDURE exit_prsm_ctrl;
     BEGIN

       if prnt_style < 10 then write( prsm_dblw_off:size_control);

       if comp or (scrn_width=132) then
            write( prsm_132_off:size_control);

       writeln( end_ctrl:size_control);

     END;



   PROCEDURE exit_epson_ctrl;
     BEGIN

       if prnt_style < 10 then write( epsn_dblw_off:size_control);

       if comp or (scrn_width=132) then
          BEGIN
            write( epsn_132_off:size_control);
            if bold then write( epsn_emph_off:size_control);
            if dbls then write( epsn_dbls_off:size_control);
          END
       ELSE
          BEGIN
            if not(nltr) and (bold or dbls) then
               BEGIN
                  if bold then write( epsn_emph_off:size_control);
                  if dbls then write( epsn_dbls_off:size_control);
               END;
            if nltr then write( epsn_nltr_off:size_control);
          END;

       writeln( end_ctrl:size_control);

     END;

   BEGIN

      IF ibmp                    THEN exit_ibm_ctrl;
      IF star                    THEN exit_star_ctrl;
      IF epson                   THEN exit_epson_ctrl;
      IF prism                   THEN exit_prsm_ctrl;
      IF la50                    THEN exit_dec_ctrl;
      IF la100                   THEN exit_dec_ctrl;

      if (scrn_width > 80) or comp THEN write( clmns_80:size_control);

   END;

(************************************************************************)
(*			      PAGE_BREAK				*)
(************************************************************************)

PROCEDURE page_break;

   BEGIN
   write( pagebreak:size_control);
   line := 0;
   curpage := curpage + 1;
   END;


(************************************************************************)
(*			     DUMP_BUFFER				*)
(************************************************************************)

PROCEDURE dump_buffer;

   BEGIN
   IF line > size_page THEN page_break;
   writeln( iobuffer:count);
   line  := line  + 1;
   count := 0
   END;


(************************************************************************)
(*			     OVERFLOW_PRINT     			*)
(************************************************************************)

PROCEDURE overflow_print;

VAR

   loc_width     : integer;

   BEGIN

      bufflen    := count;
      loc_width  := prnt_width;

      IF ((line = 0) and (curpage = 1)) THEN writeln;

      WHILE (bufflen > 0) DO
      BEGIN

         loopcnt    := 1;
         lastspace  := 0;

         IF bufflen <= loc_width Then loc_width := bufflen;
         
         WHILE (loopcnt <= loc_width) DO
            IF iobuffer[loopcnt] = ' ' THEN
               BEGIN
                  lastspace := loopcnt;
                  loopcnt := loopcnt + 1;
               END
           ELSE
               loopcnt := loopcnt + 1;

         IF lastspace = 0 THEN lastspace := loc_width;
         IF bufflen = loc_width THEN lastspace := loc_width;
         IF lastspace < (loc_width/4) THEN lastspace := loc_width - 4;
         count := lastspace;

         dump_buffer;

         lastspace := lastspace + 1;
      
         count := 0;

         IF lastspace < bufflen THEN
         BEGIN
            FOR loopcnt := lastspace TO bufflen DO
            BEGIN
               count := count + 1;
               iobuffer[count] := iobuffer[loopcnt];
            END;
            write( '_c ' );
         END;

         bufflen := count;

      END;

   END;


(************************************************************************)
(*			      PRINT_FILE				*)
(************************************************************************)

PROCEDURE print_file (curr : integer);

BEGIN
   OPEN  (infile, files[curr], HISTORY := OLD, error := continue );
   if status(infile) = 0 then
   BEGIN
      RESET (infile);

      line  := 0;
      count := 0;

      WHILE NOT EOF (infile) DO
         BEGIN
         WHILE NOT EOLN (infile) DO
            BEGIN
            READ  (infile, inchar); 
            CASE inchar OF
               formfeed: BEGIN
                            IF ((count > (prnt_width-5)) and (line = 0) and 
                                (curpage = 1)) or ((count > prnt_width) and 
                                (line <> 0)) THEN overflow_print
                            ELSE dump_buffer;
                            page_break;
                         END;

               linefeed: BEGIN
                            IF ((count > (prnt_width-5)) and (line = 0) and 
                                (curpage = 1)) or ((count > prnt_width) and 
                                (line <> 0)) THEN overflow_print
                            ELSE dump_buffer;
                            writeln;
                         END;

               OTHERWISE BEGIN
                            count := count + 1;
                            iobuffer [count] := inchar
                         END;
               END;
            END;

         READLN (infile);
         IF ((count > (prnt_width-5)) and (line = 0) and 
             (curpage = 1)) or ((count > prnt_width) and 
             (line <> 0)) THEN overflow_print
         ELSE dump_buffer;
         END;

      CLOSE (infile);

   END
   ELSE
      writeln('   %ERROR - ',files[curr], ' cannot be opened by system.');

   page_break;
   END;


(************************************************************************)
(*			    PROCESS_REQUESTS				*)
(************************************************************************)

PROCEDURE process_requests;

   VAR
      i : integer;

   BEGIN
   writeln (' ');
   enter_print_ctrl;
   if npage then page_break;
   FOR i := 1 TO max_files DO
      BEGIN
         curpage := 1;
         print_file (i);
      END;
   exit_print_ctrl;
   END;


(************************************************************************)
(*			  Main body of program				*)
(************************************************************************)

BEGIN
  return := cli$present ('FILESPEC');
  IF ODD(return) THEN
    BEGIN
       return := cli$get_value ('FILESPEC', files[1]);
       max_files := 1;
    END
  ELSE
    get_requests;

  perform_checks;
  if NOT(prntr) THEN LA50 := TRUE;
  check_width;

  writeln( prnt_width );

  process_requests;

END.
