[INHERIT ('QIOLIB.PEN')] PROGRAM load (INPUT, OUTPUT, error);

(*									*)
(* This program is designed to read data from any tape that has been 	*)
(* mounted "FOREIGN" or "NOLABEL".  It does this by means of logical/	*)
(* virtual reads to the tape.  The output file is a FILES-11 structured	*)
(* file; this file may be another tape.  				*)
(*									*)
(* Another important feature of this program is that it uses the 	*)
(* "COMMAND LANGUAGE DEFINITION" facility of VMS Version 3.2.  This 	*)
(* improves the user-friendliness of the program.  A great		*)
(* deal of effort was made to make this program behave like a VMS	*)
(* utility.  The "CLD" facility is descibed in the "VAX/VMS UTILITY	*)
(* REFERENCE MANUAL".  In addition to "CLD", this program uses 		*)
(* "LIB$TPARSE" to parse the data received from DCL.  The state tables	*)
(* and macros for TPARSE are found in the file "PARSE.MAR".		*)
(*									*)
(* There are four basic sections to this program.			*)
(*									*)
(*	1) Process the data received from DCL through CLI. This section *)
(* 	   receives data about the command line and sets the correct	*)
(*	   controlling flags.  						*)
(*									*)
(*	2) Initialize the input/output ports.  This section assigns a	*)
(*	   channel to the tape drive, rewinds it, skips the correct	*)
(*  	   number of files and records, and then opens the output file.	*)
(*	   A failure at any point in this process aborts the image.	*)
(*									*)
(*	3) Process the data.  This procedure reads the tape and writes	*)
(*	   the output file.						*)
(*									*)
(*	4) Image rundown.  If any files need to be closed they are, and	*)
(*	   the tape drive is deassigned.				*)
(*									*)
(* All tape input is done through a utility called NTRAN.  This set	*)
(* of subprograms performs virtual (logical if the tape is FOREIGN) I/O	*)
(* operations on the tape drive.  These subprograms should be in the	*)
(* systems engineering object library. 					*)
(*									*)

CONST
  (* SYSTEM error codes *)

  sys_tparse  = %X00158284;
  sys_normal  = %X00000001;
  sys_eof     = %X00000870;
  sys_eot     = %X00000878;

  (* LOAD error/symbol codes *)

  load_recovrflw = -1;
  load_ambcount  = -2;
  load_badendrec = -3;
  load_maxsize   = 32767;
  load_maxcount  = 200000;

TYPE
  string_buff = PACKED ARRAY [1..load_maxsize] OF CHAR;

VAR
  error       : TEXT;
  dump        : TEXT;

  record_str  : PACKED ARRAY [1..32] OF CHAR;
  size_str    : PACKED ARRAY [1..10] OF CHAR;
  skip_str    : PACKED ARRAY [1..10] OF CHAR;
  input_dev   : PACKED ARRAY [1..25] OF CHAR;
  filespec    : PACKED ARRAY [1..64] OF CHAR;
  Abuffer     ,
  buffer      : string_buff;

  icb         : PACKED ARRAY [1..6] OF word;

  return      : INTEGER;
  parse       : INTEGER;
  rec_max     : INTEGER;
  rec_start   : INTEGER;
  rec_end     : INTEGER;
  rec_count   : INTEGER;
  swp_count   : INTEGER;
  siz_count   : INTEGER;
  skp_count   : INTEGER;
  counter     : INTEGER;
  transfer    : INTEGER;

  ebcdic      ,
  rwnd        ,
  swap        : BOOLEAN;
  skip        : BOOLEAN;
  recr        : BOOLEAN;
  size        : BOOLEAN;

(* This section declares all the external subprograms that are not 	*)
(* defined in any of the ENVIRONMENT files.  If a subprogram other then *)
(* this is referenced then it was defined by one of the *.PEN files	*)

FUNCTION lib$tra_ebc_asc (
  %DESCR buffer  : string_buff;
  %DESCR Abuffer : string_buff) : INTEGER;
  EXTERN;

FUNCTION cli$present (
  %STDESCR name   : PACKED ARRAY [l1..u1 : INTEGER] OF CHAR) : INTEGER;
  EXTERN;

FUNCTION cli$get_value (
  %STDESCR name   : PACKED ARRAY [l1..u1 : INTEGER] OF CHAR;
           retbuf : PACKED ARRAY [l2..u2 : INTEGER] OF CHAR) : INTEGER;
  EXTERN;

FUNCTION load_record_parse (
  %STDESCR name   : PACKED ARRAY [l1..u1 : INTEGER] OF CHAR;
           start  : INTEGER;
           rend   : INTEGER;
           count  : INTEGER) : INTEGER;
  EXTERN;

FUNCTION load_number_parse (
  %STDESCR name   : PACKED ARRAY [l1..u1 : INTEGER] OF CHAR;
           count  : INTEGER) : INTEGER;
  EXTERN;

PROCEDURE sys$exit (
  %IMMED   code     : INTEGER);
  EXTERN;


(*									*)
(* This procedure is used for error trapping.  Every identifiable error	*)
(* condition is given an error code and is trapped when control passes	*)
(* to this procedure.  All "LOAD" specific errors are given negative	*)
(* numbers so as not to interfere with any systems errors.  This error	*)
(* codes are defined in the CONST section of this program.		*)
(*									*)
(* The first argument is the error code and the second argument is any  *)
(* text string that may be required for some conditions.		*)
(*									*)

PROCEDURE load_error (
  number : INTEGER;
  string : PACKED ARRAY [l1..u1 : INTEGER] OF CHAR);

  VAR
    flag   : BOOLEAN;

  BEGIN
  flag := FALSE;

  OPEN (error, FILE_NAME := 'SYS$ERROR');
  REWRITE (error);

  IF number = sys_tparse THEN
    BEGIN
    flag := TRUE;
    WRITELN (error, 'ERROR - parsing string ',string);
    sys$exit (sys_tparse);
    END;
  
  IF number = sys_eof THEN
    BEGIN
    flag := TRUE;
    WRITELN (error, 'INFORMATION - end of file encountered');
    END;
  
  IF number = sys_eot THEN
    BEGIN
    flag := TRUE;
    WRITELN (error, 'WARNING - end of tape encountered');
    END;
  
  IF number = load_recovrflw THEN
    BEGIN
    flag := TRUE;
    WRITELN (error, 'ERROR - \RECORD\ qualifier specifies too large a file');
    sys$exit (sys_normal);
    END;

  IF number = load_ambcount THEN
    BEGIN
    flag := TRUE;
    WRITELN (error, 'ERROR - ambiguous information in \RECORD\ qualifier');
    sys$exit (sys_normal);
    END;

  IF number = load_badendrec THEN
    BEGIN
    flag := TRUE;
    WRITELN (error, 'ERROR - invalid \END:\ in \RECORD\ qualifier');
    sys$exit (sys_normal);
    END;

  IF NOT flag THEN
    BEGIN
    WRITELN (error, 'ERROR - system error encountered');
    sys$exit (number);
    END;

  CLOSE (error);
  END;


(*									*)
(* This procedure queries DCL to see if the \SWAP\ option was specified *)
(* on the command line.  If it was the 'SWAP' flay is set.		*)
(*									*)

PROCEDURE check_swap;

  BEGIN
  return := cli$present ('SWAP');
  IF ODD(return) THEN
    BEGIN
    swap := TRUE;
    END
  ELSE
    BEGIN
    swap := FALSE;
    END;
  END;


(*									*)
(* This procedure queries DCL to see if the \EBCDIC\ option was 	*)
(* specified on the command line.  If it was the 'EBCDIC' flag is set.	*)
(*									*)

PROCEDURE check_ebcdic;

  BEGIN
  return := cli$present ('EBCDIC');
  IF ODD(return) THEN
    BEGIN
    ebcdic := TRUE;
    END
  ELSE
    BEGIN
    ebcdic := FALSE;
    END;
  END;


(*									*)
(* This procedure queries DCL to see if the \NOREWIND\ option was 	*)
(* specified on the command line.  If it was the 'rwnd' flag is set.	*)
(*									*)

PROCEDURE check_rewind;

  BEGIN
  return := cli$present ('NOREWIND');
  IF ODD(return) THEN
    BEGIN
    rwnd := FALSE;
    END
  ELSE
    BEGIN
    rwnd := TRUE;
    END;
  END;


(*									*)
(* This procedure queries DCL to see if the \SKIP\ option was 		*)
(* specified.  If it was the skip flag is set and the value associated  *)
(* with this option is fetched into the variable 'SKP_STRING'.  This 	*)
(* string is then parsed by TPARSE to check to see if the string 	*)
(* represents a decimal number.  If it is, the number is converted to 	*)
(* binary and its value is returned to the variable SKP_COUNT. 		*)
(*									*)

PROCEDURE check_skip;

  BEGIN
  return := cli$present ('SKIP');
  IF ODD(return) THEN
    BEGIN
    skip := TRUE;
    return := cli$get_value ('SKIP', skip_str);
    IF ODD(return) THEN
      BEGIN
      parse := load_number_parse (skip_str, skp_count);
      IF NOT ODD(parse) THEN load_error (parse, skip_str);
      END;
    END

  ELSE
    BEGIN
    skip := FALSE;
    END;
  END;


(*									*)
(* This procedure queries DCL to see if the \SIZE\ option was 		*)
(* specified.  If it was the size flag is set and the value associated  *)
(* with this option is fetched into the variable 'SIZ_STRING'.  This 	*)
(* string is then parsed by TPARSE to check to see if the string 	*)
(* represents a decimal number.  If it is, the number is converted to 	*)
(* binary and its value is returned to the variable SIZ_COUNT. 		*)
(*									*)

PROCEDURE check_size;

  BEGIN
  siz_count := load_maxsize;
  return := cli$present ('SIZE');
  IF ODD(return) THEN
    BEGIN
    size := TRUE;
    return := cli$get_value ('SIZE', size_str);
    IF ODD(return) THEN
      BEGIN
      parse := load_number_parse (size_str, siz_count);
      IF NOT ODD(parse) THEN load_error (parse, size_str);
      END
    ELSE
      BEGIN
      siz_count := load_maxsize
      END;    
    END

  ELSE
    BEGIN
    size := FALSE;
    END;
  END;


(*									*)
(* This procedure queries DCL to see if the \RECORD\ option was 	*)
(* specified.  If it was the RECR flag is set and the value associated  *)
(* with this option is fetched into the variable 'RECORD_STRING'. This 	*)
(* string is then parsed by TPARSE to check to see if the string 	*)
(* conforms to the syntax required by this option (this syntax is	*)
(* documented in the file PARSE.MAR).  Since this option may be a list	*)
(* of values the fetch/parse operation is continued untill all  values	*)
(* are returned.  The input strings are converted into integers and 	*)
(* are placed in one of the following variables: REC_START, REC_END, or *)
(* REC_COUNT.  These variables represent: 1) REC_START, the record	*)
(* on the input tape at which to start processing, 2) REC_END, the 	*)
(* on the input tape at which to stop processing, and 3) REC_COUNT,	*)
(* the number of records to process.  Since REC_END and REC_COUNT may	*)
(* contain conflicting data and the number of records may exceed the	*)
(* MAXCOUNT limit, this procedure checks the validity of the input 	*)
(* data.								*)
(*									*)

PROCEDURE check_record;

  BEGIN
  rec_start := 0;
  rec_end   := 0;
  rec_count := 0;

  return := cli$present ('RECORD');
  IF ODD(return) THEN
    BEGIN
    recr := TRUE;

    return := cli$get_value ('RECORD', record_str);
    WHILE ODD(return) DO
      BEGIN
      parse  := load_record_parse (record_str, rec_start, rec_end, rec_count);
      IF NOT ODD(parse) THEN load_error (parse, record_str);
      return := cli$get_value ('RECORD', record_str);
      END;

    IF rec_start > load_maxcount THEN
      BEGIN
      load_error (load_recovrflw, 'N/A');
      END;
      
    IF (rec_end <> 0) AND (rec_count <> 0) THEN
      BEGIN
      load_error (load_ambcount, 'N/A');
      END;

    IF rec_end <> 0 THEN
      BEGIN
      rec_end := 0;
      rec_count := rec_end - rec_start + 1;
      IF rec_count < 0 THEN load_error (load_badendrec, 'N/A');
      END
    ELSE
      BEGIN
      IF rec_count = 0 THEN rec_count := load_maxcount - rec_start
      END;

    END

  ELSE
    BEGIN
    recr := FALSE;
    rec_count := load_maxcount;
    END;
  END;


(*									*)
(* This procedure accepts the name of the input tape device from DCL.	*)
(*									*)

PROCEDURE accept_device;

  BEGIN
  return := cli$get_value ('DEVICE', input_dev);
  END;


(*									*)
(* This procedure accepts the name of the output filespec from DCL.	*)
(*									*)

PROCEDURE accept_filespec;

  BEGIN
  return := cli$get_value ('FILE', filespec);
  END;


(*									*)
(* This procedure controls the acceptance of input data from DCL.	*)
(* The actual processing is do in the sub-procedures call here.		*)
(*									*)

PROCEDURE process_command_line;

  BEGIN
  check_swap;
  check_skip;
  check_size;
  check_record;
  check_ebcdic;
  check_rewind;
  accept_device;
  accept_filespec;
  END;


(*									*)
(* This procedures initializes the tape device for input and the 	*)
(* filespec for output.  The following steps are taken to effect this   *)
(*									*)
(*	1) The input I/O channel for the tape drive is obtained and	*)
(*	   stored in the first element of 'icb'.			*)
(*									*)
(*	2) The tape is set at the beginning of the first block.		*)
(*									*)
(*	3) The appropriate number of files on the input device are	*)
(*	   skipped (if required).					*)
(*									*)
(*	4) The appropriate number of records of the current file are	*)
(*	   skipped (if required).					*)
(*									*)
(*	5) If the maximum size of the output record was not specified	*)
(*	   then this size is set to 132.				*)
(*									*)
(*	6) The output FILES-11 file is opened with a maximum record	*)
(*	   size of 'SIZ_COUNT'.  The file is then prepared for output   *)
(*									*)

PROCEDURE initialize_input_output;

  BEGIN
  return := ntran_assign (icb, input_dev);
  IF NOT ODD(return) THEN load_error (return, 'N/A');

  IF rwnd THEN 
     BEGIN
       return := ntran_rewind (icb);
       IF NOT ODD(return) THEN load_error (return, 'N/A');
     END;

  IF skip THEN
    BEGIN
    return := ntran_skipfile (icb, skp_count);
    IF NOT ODD(return) THEN load_error (return, 'N/A');
    END;

  IF rec_start > 1 THEN
    BEGIN
    return := ntran_skiprec (icb, (rec_start-1));
    IF NOT ODD(return) THEN load_error (return, 'N/A');
    END;

  OPEN (dump, FILE_NAME := filespec,
              RECORD_LENGTH := siz_count,
              ORGANIZATION := SEQUENTIAL);
  REWRITE (dump);
  END;


(*									*)
(* This procedure swaps the high and low byte in each word of the	*)
(* I/O buffer.	This operation is required if the 'SWAP' flag has	*)
(* been set by the user.						*)
(*									*)

PROCEDURE buffer_swap;  (* not implemented *)

  BEGIN
  END;

(*									*)
(* This procedure translates EBCDIC input data into ASCII output.	*)
(* The input data is translated from array 'buffer' into array          *)
(* 'Abuffer'.  Array 'Abuffer' is then assigned back to 'buffer'.	*)
(*									*)

PROCEDURE buffer_trans;  

  BEGIN
     return := lib$tra_ebc_asc(buffer,Abuffer);
     buffer := Abuffer
  END;


(*									*)
(* This procedure actually transfers the data from the input tape	*)
(* device to the FILES-11 output file (this may also be a tape).	*)
(* The main body of this section is a loop that is executed once for	*)
(* each successful tape read operation.  This looping continues until 	*)
(* either a read error occurs (EOF or EOT) or a REC_COUNT number of 	*)
(* records have been read.  For each pass through the loop the 		*)
(* following occurs.							*)
(*									*)
(*	1) The high and low bytes of each word are swapped (if		*)
(*	   required)							*)
(*									*)
(*	2) The output buffer is truncated to SIZ_COUNT if it is too	*)
(*	   large, otherwise it remains unchanged.			*)
(*									*)
(*	3) The buffer is written to the output file and the counter	*)
(*	   is incremented by one.					*)
(*									*)
(*	4) If there are still records to be read then a read is done.	*)
(*									*)

PROCEDURE process_data;

  BEGIN
  rec_max := 0;
  counter := 0;
  return := ntran_read (icb, load_maxsize, buffer);
  WHILE ODD(return) AND (return <> sys_eof) DO
    BEGIN
    IF swap THEN buffer_swap;
    IF ebcdic THEN buffer_trans;
    transfer := cvt_w_l(icb[4]);
    IF transfer > siz_count THEN transfer := siz_count;
    WRITELN (dump, buffer:transfer);
    IF transfer > rec_max THEN rec_max := transfer;
    counter := counter + 1;
    IF counter < rec_count THEN
      return := ntran_read (icb, load_maxsize, buffer);
    END;

  IF NOT ODD(return) THEN load_error (return, 'N/A');
  WRITELN ('INFORMATION - Maximum record size: ', rec_max:5);
  WRITELN ('INFORMATION - Records transfered:  ', counter:7);
  END;


(*									*)
(* This procedure preforms any image rundown operations that may be	*)
(* required.								*)
(*									*)

PROCEDURE image_rundown;

  BEGIN
  return := ntran_deassign (icb);
  CLOSE (dump);
  END;


(*									*)
(* This is the main body of the program.				*)
(*									*)

BEGIN
process_command_line;
initialize_input_output;
process_data;
image_rundown;
END.
