	INTEGER*4 FUNCTION NTRAN(N,ICB,NCOUNT,BUFFER)
C
C******************************************************************************
C
C General purpose magnetic tape handler
C
C The general format for the call is:
C
C 	return_status = NTRAN (code, icb, count, buffer) ! General call
C	return_status = NTRAN_ASSIGN (icb, device)       ! Assign channel
C	return_status = NTRAN_REWIND (icb)               ! Rewind tape
C       return_status = NTRAN_WRITEOF (icb)		 ! Write EOF
C	return_status = NTRAN_SETMODE			 ! NOT IN USE
C	return_status = NTRAN_READ (icb, count, buffer)  ! Read virtual block
C	return_status = NTRAN_WRITE (icb, count, buffer) ! Write virtual block
C	return_status = NTRAN_SKIPREC (icb, count)       ! Skip 'count' records
C	return_status = NTRAN_SKIPFILE (icb, count)      ! Skip 'count' files
C	return_status = NTRAN_DEASSIGN (icb) 		 ! Deassign channel
C
C These arguments are defines as follows:
C
C   code      - The address of a longword integer containing the NTRAN 
C		operation code.  These codes are as follows.
C		  1 = Assign unit
C		  2 = Rewind unit
C		  3 = Write end of file
C		  4 = Set density/parity
C		  5 = Read physical record
C		  6 = Write physical record
C		  7 = Skip records
C		  8 = Skip files
C		  9 = Deassign unit
C
C   icb       - The address of a 6 word (INTEGER*2) information array 
C		returned by ntran.  These items are set by system service 
C		routines called within ntran.  The various fields within the
C		ICB are defined as follows:
C
C		  ICB(1) - Channel number (set by an NTRAN assign call)
C		  ICB(2) - NOT USED
C     		  ICB(3) - I/O return status for normal completion
C		  ICB(4) - Number of bytes transferred, or
C			   Number of records/files skipped before end of file
C			   tape detected
C		  ICB(5) - low byte of device-dependent code
C		  ICB(6) - high byte of device-dependent code
C
C		The last four bytes of ICB are the I/O status block.  This
C		data structure is described in detail in the "VAX/VMS I/O
C		User's Guide".
C
C   count     - Address of a longword containing the maximum number of 
C		bytes, records, or files to process.
C
C   buffer    - Address of the Input/output buffer.  For 'code' equal to
C		'1' this buffer contains the device name.
C
C   device    - Address of the buffer containing the device name.
C
C
C   NOTE:
C     This return value of this function is the QIO return value.  The
C     calling program must check this value to determine the success of 
C     of this function.  The ICB can be used to determine device dependent
C     information.
C
C   Program Development History:
C
C	Original Written by: Richard Hill LEMSCO/JSC Houston
C
C	Documented and Revised by: Paul Schweizer HSD/JSC Houston
C			           2-MAY-1983
C
C	Additional entry points by: Paul Schweizer HSD/JSC Houston
C				    4-MAY-1983
C
C******************************************************************************
C
C
      EXTERNAL		IO$_REWIND,
     *			IO$_WRITEOF,
     *			IO$_READVBLK,
     *			IO$_WRITEVBLK,
     *			IO$_SKIPFILE,
     *	                IO$_SKIPRECORD,
     *			IO$_SETMODE
C
      PARAMETER     	SS$_NORMAL    = '00000001'X,
     *			SS$_BADPARAM  = '00000014'X
C
      LOGICAL*4         BUFFER
C
      INTEGER*4		SYS$QIOW,
     *			SYS$ASSIGN,
     * 			SYS$DEASSIGN
C
      INTEGER*2 	ICB(6)
C
      CHARACTER*5 	DEVICE
C
      IF (N .EQ. 1) THEN
        GOTO 100
      ELSE IF (N .EQ. 2) THEN
        GOTO 150
      ELSE IF (N .EQ. 3) THEN
        GOTO 200
      ELSE IF (N .EQ. 4) THEN
        GOTO 250
      ELSE IF (N .EQ. 5) THEN
        GOTO 300
      ELSE IF (N .EQ. 6) THEN
        GOTO 350
      ELSE IF (N .EQ. 7) THEN
        GOTO 400
      ELSE IF (N .EQ. 8) THEN
        GOTO 450
      ELSE IF (N .EQ. 9) THEN
        GOTO 500
      ELSE
        NTRAN = SS$_BADPARAM
      END IF
      RETURN
C
C     +-------------+
C     | ASSIGN UNIT |
C     +-------------+
C
      ENTRY NTRAN_ASSIGN (ICB, BUFFER)
C
  100 ENCODE(5,1005,DEVICE) BUFFER
      NTRAN = SYS$ASSIGN(DEVICE,ICB(1),,)
      ICB(3) = 1
      RETURN
C
C     +-------------+
C     | REWIND UNIT |
C     +-------------+
C
      ENTRY NTRAN_REWIND (ICB)
C
  150 NTRAN = SYS$QIOW(,%VAL(ICB(1)),IO$_REWIND,ICB(3),,,,,,,,)
      IF (NTRAN) NTRAN = ICB(3)
      RETURN
C
C     +-------------------+
C     | WRITE END OF FILE |
C     +-------------------+
C
      ENTRY NTRAN_WRITEOF (ICB)
C
  200 NTRAN = SYS$QIOW(,%VAL(ICB(1)),IO$_WRITEOF,ICB(3),,,,,,,,)
      IF (NTRAN) NTRAN = ICB(3)
      RETURN
C
C     +--------------------+
C     | SET DENSITY/PARITY |
C     +--------------------+
C
      ENTRY NTRAN_SETMODE ()
C
  250 TYPE 1010,N
      NTRAN  = 1
      RETURN
C
C     +-------------+
C     | READ RECORD |
C     +-------------+
C
      ENTRY NTRAN_READ (ICB, NCOUNT, BUFFER)
C
  300 NTRAN = SYS$QIOW(,%VAL(ICB(1)),IO$_READVBLK,ICB(3),,,
     *        BUFFER,%VAL(NCOUNT),,,,)
      IF (NTRAN) NTRAN = ICB(3)
      RETURN
C
C     +--------------+
C     | WRITE RECORD |
C     +--------------+
C
      ENTRY NTRAN_WRITE (ICB, NCOUNT, BUFFER)
C
  350 NTRAN = SYS$QIOW(,%VAL(ICB(1)),IO$_WRITEVBLK,ICB(3),,,
     *        BUFFER,%VAL(NCOUNT),,,,)
      IF (NTRAN) NTRAN = ICB(3)
      RETURN
C
C     +----------------+
C     | SKIP N RECORDS |
C     +----------------+
C
      ENTRY NTRAN_SKIPREC (ICB, NCOUNT)
C
  400 NTRAN = SYS$QIOW(,%VAL(ICB(1)),IO$_SKIPRECORD,ICB(3),,,
     *        %VAL(NCOUNT),,,,,)
      IF (NTRAN) NTRAN = ICB(3)
      RETURN
C
C     +--------------+
C     | SKIP N FILES |
C     +--------------+
C
      ENTRY NTRAN_SKIPFILE (ICB, NCOUNT)
C
  450 NTRAN = SYS$QIOW(,%VAL(ICB(1)),IO$_SKIPFILE,ICB(3),,,
     *        %VAL(NCOUNT),,,,,)
      IF (NTRAN) NTRAN = ICB(3)
      RETURN
C
C     +---------------+
C     | DEASSIGN UNIT |
C     +---------------+
C
      ENTRY NTRAN_DEASSIGN (ICB, NCOUNT, BUFFER)
C
  500 NTRAN = SYS$DASSGN(ICB(1))
      RETURN
C
C
 1005 FORMAT (A4':')
 1010 FORMAT (' NTRAN code 'I3,' has not been implemented')
      END
