MODULE pascal_vax_utilities (INPUT, OUTPUT);

CONST
   nul       = chr(0);
   bel       = chr(7);
   cr        = chr(13);
   esc       = chr(27);
   size_time = 8;


TYPE
  string_time  = PACKED ARRAY [1..size_time] OF char;


VAR
   ocelot_assign  : [STATIC] boolean := false;
   ocelot_chan    : [STATIC] integer;



FUNCTION sys$assign (%STDESCR devnam : PACKED ARRAY [l1..u1 : integer] OF char;
                     %REF     chan   : integer;
                     %REF     acmode : integer := %IMMED 0;
                     %STDESCR mdxnam : PACKED ARRAY [l2..u2 : integer] OF char
                                       := %IMMED 0) : unsigned; EXTERN;

FUNCTION sys$qiow   (%IMMED   efn    : integer  := %IMMED 0;
                     %IMMED   chan   : integer;
                     %IMMED   func   : unsigned;
                     %REF     iosb   : integer  := %IMMED 0;
                     %IMMED   astdar : integer  := %IMMED 0;
                     %IMMED   astprm : integer  := %IMMED 0;
                     %REF     p1_buf : PACKED ARRAY [l1..u1 : integer] OF char;
                     %IMMED   p2_siz : integer;
                     %IMMED   p3_tim : integer  := %IMMED 0;
                     %IMMED   p4_msk : unsigned := %IMMED 0;
                     %IMMED   p5     : integer  := %IMMED 0;
                     %IMMED   p6     : integer  := %IMMED 0) : unsigned; EXTERN;



(************************************************************************)
(*				  GETCHAR				*)
(************************************************************************)

[GLOBAL] FUNCTION getchar : char;

   CONST
      io$_ttyreadall = 58;
      io$m_noecho    = 64;
      ttyreadall     = uor (io$_ttyreadall, io$m_noecho);
      char_nul       = chr(0);
      size_inbuf     = 8;

   TYPE
      buffer_in      = PACKED ARRAY [1..size_inbuf] OF char;

   VAR
      inbuf  : buffer_in;
      stat   : unsigned;

   BEGIN
   IF not ocelot_assign THEN 
      BEGIN
      stat := sys$assign ('SYS$COMMAND', ocelot_chan);
      ocelot_assign := true
      END;

   stat := sys$qiow ( , ocelot_chan, ttyreadall, , , , inbuf, 1, , , , );
   IF odd (stat) THEN
      getchar := inbuf[1]
   ELSE
      getchar := char_nul
   END;


(************************************************************************)
(*			          PUTCHAR				*)
(************************************************************************)

[GLOBAL] PROCEDURE putchar (a : char);

   CONST
      io$_writelblk  = 32;
      io$m_noformat  = 256;
      writelblk      = uor (io$_writelblk, io$m_noformat);
      size_outbuf    = 8;

   TYPE
      buffer_out     = PACKED ARRAY [1..size_outbuf] OF char;

   VAR
      outbuf : buffer_out;
      stat   : unsigned;

   BEGIN
   IF not ocelot_assign THEN 
      BEGIN
      stat := sys$assign ('SYS$COMMAND', ocelot_chan);
      ocelot_assign := true
      END;

   outbuf[1] := a;
   stat := sys$qiow ( , ocelot_chan, writelblk, , , , outbuf, 1, , , , )
   END;


(************************************************************************)
(*			          PUTLINE				*)
(************************************************************************)

[GLOBAL] PROCEDURE putline (a : PACKED ARRAY [l1..u1 : integer] OF char);

   CONST
      io$_writelblk  = 32;
      io$m_noformat  = 256;
      writelblk      = uor (io$_writelblk, io$m_noformat);
      size_outbuf    = 8;

   TYPE
      buffer_out     = PACKED ARRAY [1..size_outbuf] OF char;

   VAR
      len    : integer;
      stat   : unsigned;

   BEGIN
   IF not ocelot_assign THEN 
      BEGIN
      stat := sys$assign ('SYS$COMMAND', ocelot_chan);
      ocelot_assign := true
      END;

   len := u1 - l1 + 1;
   stat := sys$qiow ( , ocelot_chan, writelblk, , , , a, len, , , , )
   END;


(************************************************************************)
(*			      RESET_SCREEN				*)
(************************************************************************)

[GLOBAL] PROCEDURE reset_screen;

  BEGIN
  putchar (esc);
  putline ('[1;24r'(27)'[J')
  END;


(************************************************************************)
(*			      SAVE_CURSOR				*)
(************************************************************************)

[GLOBAL] PROCEDURE save_cursor;

  BEGIN
  putchar (esc);
  putchar ('7')
  END;


(************************************************************************)
(*			    RESTORE_CURSOR				*)
(************************************************************************)

[GLOBAL] PROCEDURE restore_cursor;

  BEGIN
  putchar (esc);
  putchar ('8')
  END;


(************************************************************************)
(*			      CONTINUE					*)
(************************************************************************)

[GLOBAL] PROCEDURE continue;

  BEGIN
  writeln;
  writeln;
  write  ('Press <RETURN> to continue: ');
  readln
  END;


(************************************************************************)
(*			      CVT_JTIME					*)
(************************************************************************)

[GLOBAL] FUNCTION cvt_jtime (intime : integer) : string_time;

   CONST
      delimiter = ':';
      size_int2 = 2;

   TYPE
      string_int2 = PACKED ARRAY [1..size_int2] OF char;

   VAR
      temp  : VARYING [size_time] OF char;
      jtime : string_time;
      hrs   : integer;
      min   : integer;
      sec   : integer;
      i     : integer;


(************************************************************************)

   FUNCTION int2 (i : integer) : string_int2;

      VAR
         temp : string_int2;

      BEGIN
      i       := i mod 100;
      temp[1] := chr((i div 10) + 48);
      temp[2] := chr((i mod 10) + 48);
      int2    := temp
      END;


(************************************************************************)

   BEGIN
   hrs := intime div 3600;
   min := (intime - hrs*3600) div 60;
   sec := intime mod 60;

   writev (temp, int2(hrs), delimiter, int2(min), delimiter, int2(sec));
   FOR i := 1 to size_time DO jtime[i] := temp[i];
   cvt_jtime := jtime
   END;


(************************************************************************)
(*			      CVT_ATIME					*)
(************************************************************************)

[GLOBAL] FUNCTION cvt_atime (intime : string_time) : integer;

   CONST
      delim      = ':';
      nul        = chr(0);
      size_slot  = 3;

   TYPE
      type_ptr   = 1..size_time;
      type_slot  = 1..size_slot;
      type_time  = (lim, num, skp, eos);
      array_time = ARRAY [type_slot] OF integer;

   VAR
      time  : array_time;
      mode  : type_time;
      slot  : type_slot;
      ptr   : integer;
      sec   : integer;
      curr  : char;


(************************************************************************)

   PROCEDURE next_char;

      BEGIN
      curr := nul;
      IF ptr <= size_time THEN
         BEGIN
         curr := intime [ptr];
         ptr  := ptr + 1
         END;

      IF      curr = delim THEN mode := lim
      ELSE IF curr = ' '   THEN mode := skp
      ELSE IF curr in ['0'..'9']  THEN mode := num
      ELSE mode := eos
      END;



(************************************************************************)

   BEGIN
   FOR slot := 1 TO size_slot DO time [slot] := 0;
   slot := 1;
   ptr  := 1;
   next_char;

   WHILE mode <> eos DO
      CASE mode OF
         lim: BEGIN
              IF slot < size_time THEN slot := slot + 1;
              next_char
              END;

         num: BEGIN
              WHILE mode = num DO
                 BEGIN
                 time[slot] := time[slot]*10 + (int(curr) - 48);
                 next_char
                 END;
              END;

         skp: BEGIN
              next_char
              END;

         END;

   sec := (time[1] mod 48) * 3600;
   sec := (time[2] mod 60) * 60 + sec;
   sec := (time[3] mod 60) + sec;
   cvt_atime := sec
   END;

END.
