[INHERIT ('TAP_SYSTEM:QIOLIB.PEN')] PROGRAM load (INPUT, OUTPUT, DUMP);

(************************************************************************)
(*									*)
(* This program is designed to be a general tape data manipulation	*)
(* routine.  It requires two tapes mounted "FOREIGN" or "NOLABEL".	*)
(*									*)
(* All tape input/output is done through a utility called NTRAN.  	*)
(* This set of subprograms performs virtual (logical for FOREIGN tapes) *)
(* I/O operations on the tape drive.  These subprograms should be in the*)
(* systems engineering object library. 					*)
(*									*)
(************************************************************************)

CONST
  (* SYSTEM error codes *)

  esc         = CHR(27);

  sys_tparse  = %X00158284;
  sys_normal  = %X00000001;
  sys_eof     = %X00000870;
  sys_eot     = %X00000878;

  (* LOAD error/symbol codes *)

  load_recovrflw = -1;
  load_ambcount  = -2;
  load_badendrec = -3;
  load_maxsize   = 32767;
  load_maxcount  = 200000;

TYPE

  com_string = PACKED ARRAY [1..5] OF CHAR;
  cb_string  = PACKED ARRAY [1..6] OF WORD;
  buf_string = PACKED ARRAY [1..load_maxsize] OF CHAR;


VAR
  dump        : TEXT;

  record_str  : PACKED ARRAY [1..32] OF CHAR;
  size_str    : PACKED ARRAY [1..10] OF CHAR;
  skip_str    : PACKED ARRAY [1..10] OF CHAR;
  input_dev   ,
  output_dev  : com_string;
  filespec    : PACKED ARRAY [1..32] OF CHAR;
  Abuffer     : buf_string;
  Ebuffer     : buf_string;
  buffer      : buf_string;

  oucb        ,
  icb         ,
  wrkcb       : cb_string;
  cmnd2       : com_string;

  return2     ,
  in_type     ,
  out_type    ,
  return      : INTEGER;
  parse       : INTEGER;
  rec_max     : INTEGER;
  rec_min     : INTEGER;
  rec_start   : INTEGER;
  rec_end     : INTEGER;
  rec_count   : INTEGER;
  skp_count   : INTEGER;
  counter     : INTEGER;
  transfer    : INTEGER;

(* This section declares all the external subprograms that are not 	*)
(* defined in any of the ENVIRONMENT files.  If a subprogram other than *)
(* this is referenced then it was defined by one of the *.PEN files	*)

PROCEDURE sys$exit (
  %IMMED   code     : INTEGER);
  EXTERN;

PROCEDURE lib$tra_asc_ebc(
  %DESCR  buffer   : buf_string;
  %DESCR  Ebuffer  : buf_string);
  EXTERN;

PROCEDURE lib$tra_ebc_asc(
  %DESCR  buffer   : buf_string;
  %DESCR  Abuffer  : buf_string);
  EXTERN;

PROCEDURE lib$spawn(
  %STDESCR  Abuffer  : packed array [l1..u1 : integer] of char);
  EXTERN;

PROCEDURE lib$get_symbol(
  %STDESCR  Symbol  : packed array [l1..u1 : integer] of char;
  %STDESCR  Abuffer : packed array [l2..u2 : integer] of char);
  EXTERN;

(*									*)
(* This procedure is used for error trapping.  Every identifiable error	*)
(* condition is given an error code and is trapped when control passes	*)
(* to this procedure.  All "LOAD" specific errors are given negative	*)
(* numbers so as not to interfere with any systems errors.  This error	*)
(* codes are defined in the CONST section of this program.		*)
(*									*)
(* The first argument is the error code and the second argument is any  *)
(* text string that may be required for some conditions.		*)
(*									*)

PROCEDURE load_error (
  number : INTEGER;
  string : PACKED ARRAY [l1..u1 : INTEGER] OF CHAR);

  VAR
    flag   : BOOLEAN;

  BEGIN
  flag := FALSE;

  IF number = sys_eof THEN
    BEGIN
    flag := TRUE;
    WRITELN ('INFORMATION - end of file encountered');
    END;
  
  IF number = sys_eot THEN
    BEGIN
    flag := TRUE;
    WRITELN ('WARNING - end of tape encountered');
    END;
  
  IF number = load_recovrflw THEN
    BEGIN
    flag := TRUE;
    WRITELN ('ERROR - \RECORD\ qualifier specifies too large a file');
    sys$exit (sys_normal);
    END;

  IF number = load_ambcount THEN
    BEGIN
    flag := TRUE;
    WRITELN ('ERROR - ambiguous information in \RECORD\ qualifier');
    sys$exit (sys_normal);
    END;

  IF number = load_badendrec THEN
    BEGIN
    flag := TRUE;
    WRITELN ('ERROR - invalid \END:\ in \RECORD\ qualifier');
    END;

  IF NOT flag THEN
    BEGIN
    WRITELN ('ERROR - system error encountered');
    END;

  END;


(*									*)
(* This procedures initializes the tape device for input and the 	*)
(* filespec for output.  The following steps are taken to effect this   *)
(*									*)
(*	1) The input I/O channel for the read tape is obtained and	*)
(*	   stored in the first element of 'icb'.			*)
(*									*)
(*	2) The read tape is set at the beginning of the first block.	*)
(*									*)
(*	3) The output I/O channel for a 2nd drive is obtained and	*)
(*	   stored in the first element of 'oucb'.			*)
(*									*)
(*	4) The 2nd tape is set at the beginning of the first block.	*)
(*									*)

PROCEDURE initialize_input_output;

  BEGIN

  reset_screen;
  WRITELN(esc, '[1;14H', esc, '#3SET DEVICES');
  WRITELN(esc, '[2;14H', esc, '#4SET DEVICES');
  WRITELN;	
  WRITELN('Valid Commands:  Mxxn:   EXIT');
  WRITELN('      Command EXIT for input device, will cause program to stop.');
  WRITELN;
  input_dev := '     ';
  output_dev := '     ';
  writeln('   Attempting to fetch symbols INTAPE and OUTAPE');
  lib$get_symbol('INTAPE',input_dev);
  lib$get_symbol('OUTAPE',output_dev);
  return := 0;
  WHILE (NOT ODD(return)) AND
        (substr(input_dev,1,4) <> 'EXIT') AND
		(substr(input_dev,1,4) <> 'exit') DO
     BEGIN
       IF ((substr(input_dev,1,4) <> 'EXIT') AND
		(substr(input_dev,1,4) <> 'exit')) THEN
          return := ntran_assign (icb, input_dev);
       IF NOT ODD(return) then
          Begin
             input_dev := '     ';
	     WRITELN;
             WRITE(' Name of INPUT Tape Device    :  >>  ');
             IF NOT EOLN THEN READLN(input_dev) ELSE READLN;
             WRITELN;
             return := 0;
          end;
     END;

  IF ((substr(input_dev,1,4) = 'EXIT') OR 
		(substr(input_dev,1,4) = 'exit')) THEN sys$exit(return);

  writeln;
  writeln('   INPUT_DEVICE has been defined and channel assigned');
  writeln;

  return := ntran_rewind (icb);
  IF NOT ODD(return) THEN load_error (return, 'N/A');

  return := 0;
  WHILE (NOT ODD(return)) AND
        (substr(output_dev,1,4) <> 'EXIT') AND
		(substr(output_dev,1,4) <> 'exit') DO
     BEGIN
       IF ((substr(output_dev,1,4) <> 'EXIT') AND
		(substr(output_dev,1,4) <> 'exit')) THEN
          return := ntran_assign (oucb, output_dev);
       IF NOT ODD(return) then
          Begin
             output_dev := '     ';
             WRITELN;
             WRITE(' Name of OUTPUT Tape Device    :  >>  ');
             IF NOT EOLN THEN READLN(output_dev) ELSE READLN;
             WRITELN;
             return := 0;
          end;
     END;
  IF ((substr(output_dev,1,4) <> 'EXIT') AND
		(substr(output_dev,1,4) <> 'exit'))
  THEN BEGIN
         return := ntran_rewind (oucb);
         IF NOT ODD(return) THEN load_error (return, 'N/A');
         writeln;
         writeln('   OUTPUT_DEVICE has been defined and channel assigned');
         writeln;
       END;

  filespec := 'UNKNOWN.DAT                     ';

  END;

{	This module spawns off a sub-process to allow the user to use	}
{	the TAPCTRL help library.					}

PROCEDURE CTRLHELP( cmd : com_string);

VAR
  QUES : CHAR;

BEGIN

RESET_SCREEN;

WRITELN(esc, '[1;18H', esc, '#3HELP');
WRITELN(esc, '[2;18H', esc, '#4HELP');

LIB$SPAWN('$ HELP/LIBR=SYS$HELP:TAPCTRL TAPCTRL');

RESET_SCREEN;

END;

Procedure set_top(cmd : com_string);

BEGIN
  reset_screen;
  WRITE('Command Level:  ', cmd:5);
  IF (substr(cmd,1,2) = 'IN') 
  THEN WRITE('     ':5, input_dev:5)
  ELSE WRITE('     ':5, output_dev:5);
  IF (substr(cmd,5,1) = 'E') 
  THEN WRITELN('     ':5, 'Data format is EBCDIC')
  ELSE WRITELN('     ':5, 'Data format is ASCII');
  WRITELN('Valid Commands:  EXIT    REWND   SKIPF   SKIPR   READ    LOADF');
  WRITELN('                 CLEAR   WEOF    LABEL   FEOF');
  IF (substr(cmd,1,2) = 'IN') or (substr(cmd,1,2) = 'in') 
  THEN WRITELN('                 WRITE   COPYR   COPYF')
  ELSE WRITELN;
  WRITELN(esc, '[5;15r', esc, '[6;1H');

END;

{	This module represents the guts of the actual work being	}
{	performed by this program.  Each different command is		}
{	represented by a section of code associated with a command	}
{	location variable.  The command location variable finds		}
{	its code through the use of a PASCAL CASE statement.		}
{	Certain base information is alway stored on the top five rows   }
{	of the screen.  What mode (IN or OUT), the mode data format,	}
{	and valid commands for that mode.				}

PROCEDURE CONTROL( wrkcb : cb_string;
                   cmd   : com_string);

TYPE
  EVAL_TYPE = RECORD
    CASE INTEGER OF
      1: ( BUF : PACKED ARRAY [1..8] OF CHAR);
      2: ( NUM : ARRAY [1..2] OF REAL );
      3: ( NIT : ARRAY [1..2] OF INTEGER);
  END;

VAR
  eval     : eval_type;
  prv_cmd  : com_string;
  new_label: PACKED ARRAY [1..6] OF CHAR;
  rec_num  ,
  siz_count,
  loc      ,
  show     ,
  index    : integer;

BEGIN

  rec_num := 0;
  cmnd2 := 'REWND';
  prv_cmd := cmnd2;
  set_top(cmd);
  save_cursor;
  WRITELN;
  WRITE(cmnd2:5,' >>  ');
  cmnd2 := '     ';
  IF NOT EOLN THEN READLN(cmnd2) ELSE READLN;
  WRITELN;
  IF cmnd2 = '     ' THEN cmnd2 := prv_cmd;

  WHILE (substr(cmnd2,1,2) <> 'EX') and (substr(cmnd2,1,2) <> 'ex') DO
     BEGIN

{	here we define the locator code for the command issued.		}

     loc := 0;
     IF (substr(cmnd2,1,2) = 'CL') or 
	(substr(cmnd2,1,2) = 'cl')	then loc := 10;
     IF (substr(cmnd2,1,2) = 'LA') or 
	(substr(cmnd2,1,2) = 'la')	then loc := 12;
     IF (substr(cmnd2,1,2) = 'WE') or 
	(substr(cmnd2,1,2) = 'we')	then loc := 11;
     IF (substr(cmnd2,1,2) = 'FE') or 
	(substr(cmnd2,1,2) = 'fe')	then loc := 13;
     IF (substr(cmnd2,1,3) = 'REW') or 
	(substr(cmnd2,1,3) = 'rew')	then loc := 1;
     IF (substr(cmnd2,1,3) = 'REA') or 
	(substr(cmnd2,1,3) = 'rea')	then loc := 6;
     IF (substr(cmnd2,1,2) = 'HE') or 
	(substr(cmnd2,1,2) = 'he')	then loc := 9;
     IF (substr(cmnd2,1,5) = 'SKIPF') or 
	(substr(cmnd2,1,5) = 'skipf')	then loc := 2;
     IF (substr(cmnd2,1,5) = 'SKIPR') or 
	(substr(cmnd2,1,5) = 'skipr')	then loc := 3;
     IF ((substr(cmnd2,1,5) = 'COPYF') or 
	(substr(cmnd2,1,5) = 'copyf')) and
	(wrkcb[1] <> oucb[1])  		then loc := 4;
     IF ((substr(cmnd2,1,2) = 'LO') or 
	(substr(cmnd2,1,2) = 'lo')) and
	(wrkcb[1] <> oucb[1])  		then loc := 8;
     IF ((substr(cmnd2,1,2) = 'WR') or 
	(substr(cmnd2,1,2) = 'wr')) and
	(wrkcb[1] <> oucb[1])  		then loc := 7;
     IF ((substr(cmnd2,1,5) = 'COPYR') or 
	(substr(cmnd2,1,5) = 'copyr')) and
	(wrkcb[1] <> oucb[1])  		then loc := 5;

{	Now lets find the code to be executed.				}

     CASE LOC OF 
	1 : BEGIN		{ REWIND }
               return := ntran_rewind (wrkcb);
               rec_num := 0;
               IF NOT ODD(return) THEN load_error (return, 'N/A');
            END;

       10 : BEGIN		{ CLEAR }
               WRITELN(esc, '[5;16r', esc, '[6;1H', esc, '[J');
               save_cursor;
            END;

        2 : BEGIN		{ SKIPF }
               skp_count := 0;
               WRITE('How many files to skip?  ');
               IF NOT EOLN THEN READLN(skp_count) ELSE READLN;
               return := ntran_skipfile (wrkcb, skp_count);
               rec_num := 0;
               IF NOT ODD(return) THEN load_error (return, 'N/A');
            END;

        3 : BEGIN		{ SKIPR }
               rec_start := 0;
               WRITE('How many records to skip?  ');
               IF NOT EOLN THEN READLN(rec_start) ELSE READLN;
               rec_num := rec_num + rec_start;
               return := ntran_skiprec (wrkcb, (rec_start-1));
               IF NOT ODD(return) THEN load_error (return, 'N/A');
            END;

        4 : BEGIN		{ COPYF }
             index := 0;
             rec_max := 0;
             rec_min := load_maxsize;
             return := ntran_read (wrkcb, load_maxsize, buffer);
             return2 := return;
             WHILE (return <> sys_eof) and ODD(return) and ODD(return2)  DO
               BEGIN
		 if out_type <> in_type then
		    begin
			if out_type = 1 then
			   lib$tra_asc_ebc(buffer, Ebuffer)
			else lib$tra_ebc_asc(buffer, Ebuffer);
			buffer := Ebuffer;
		    end;
                 index := index + 1;
                 transfer := cvt_w_l(wrkcb[4]);
                 IF transfer > rec_max THEN rec_max := transfer;
                 IF transfer < rec_min THEN rec_min := transfer;
                 return2 := ntran_write (oucb, transfer, buffer);
                 return := ntran_read (wrkcb, load_maxsize, buffer);
               END;
               IF return = sys_eof THEN return := ntran_writeof( oucb )
               ELSE IF NOT ODD(return) THEN load_error (return, 'N/A')
                    ELSE IF NOT ODD(return2) THEN load_error (return2, 'N/A');
               WRITELN('Information - ', index:6, ' Records Transfered.');
               WRITELN('Information - ', rec_max:6, ' Max Record Size. ');
               WRITELN('Information - ', rec_min:6, ' Min Record Size. ');
             END;

        8 : BEGIN		{ LOADF }
             siz_count := load_maxsize;
             WRITE('Enter name of file  >>  ');
             IF NOT EOLN THEN READLN(filespec) ELSE READLN;
             OPEN (dump, FILE_NAME := filespec,
                         RECORD_LENGTH := siz_count,
                         ORGANIZATION := SEQUENTIAL);
             REWRITE (dump);
             index := 0;
             rec_max := 0;
             rec_min := load_maxsize;
             return := ntran_read (wrkcb, load_maxsize, buffer);
             WHILE (return <> sys_eof) and ODD(return) DO
               BEGIN
                 index := index + 1;
		 if out_type <> in_type then
		    begin
			if out_type = 1 then
			   lib$tra_asc_ebc(buffer, Ebuffer)
			else lib$tra_ebc_asc(buffer, Ebuffer);
			buffer := Ebuffer;
		    end;
                 transfer := cvt_w_l(wrkcb[4]);
                 IF transfer > rec_max THEN rec_max := transfer;
                 IF transfer < rec_min THEN rec_min := transfer;
                 WRITELN (dump, buffer:transfer);
                 return := ntran_read (wrkcb, load_maxsize, buffer);
               END;
               IF NOT ODD(return) THEN load_error (return, 'N/A');
               WRITELN('Information - ', index:6, ' Records Transfered.');
               WRITELN('Information - ', rec_max:6, ' Max Record Size. ');
               WRITELN('Information - ', rec_min:6, ' Min Record Size. ');
               rec_num := 0;
               CLOSE (dump);
             END;

        5 : BEGIN			{ COPYR }
               rec_max := 0;
               index := 0;
               rec_min := load_maxsize;
               rec_start := 0;
               WRITE('How many records to copy?  ');
               IF NOT EOLN THEN READLN(rec_start) ELSE READLN;
               return := ntran_read (wrkcb, load_maxsize, buffer);
               return2 := return;
               WHILE (return <> sys_eof) and ODD(return) and ODD(return2)
                                         and (index < rec_start) DO
                BEGIN
                  index := index + 1;
		  if out_type <> in_type then
		    begin
			if out_type = 1 then
			   lib$tra_asc_ebc(buffer, Ebuffer)
			else lib$tra_ebc_asc(buffer, Ebuffer);
			buffer := Ebuffer;
		    end;
                  rec_num := rec_num + 1;
                  transfer := cvt_w_l(wrkcb[4]);
                  IF transfer > rec_max THEN rec_max := transfer;
                  IF transfer < rec_min THEN rec_min := transfer;
                  return2 := ntran_write (oucb, transfer, buffer);
                  return := ntran_read (wrkcb, load_maxsize, buffer);
                END;
               IF return = sys_eof THEN return := ntran_writeof( oucb )
               ELSE IF NOT ODD(return) THEN load_error (return, 'N/A')
                    ELSE IF NOT ODD(return2) THEN load_error (return2, 'N/A');
               WRITELN('Information - ', index:6, ' Records Transfered.');
               WRITELN('Information - ', rec_max:6, ' Max Record Size. ');
               WRITELN('Information - ', rec_min:6, ' Min Record Size. ');
               WRITELN('Information - ', rec_num:6, ' Record Number.');
               IF return = sys_eof THEN rec_num := 0
            END;

        6 : BEGIN			{ READ }
              return := ntran_read (wrkcb, load_maxsize, buffer);
              IF NOT ODD(return) THEN load_error (return, 'N/A')
              ELSE BEGIN
                rec_num := rec_num + 1;
                transfer := cvt_w_l(wrkcb[4]);
                IF transfer > 26 THEN show := 26
                ELSE show := transfer;
                save_cursor;
                IF show > 0 THEN
                  BEGIN
	          if ((in_type = 1) and ((substr(cmd,1,2) = 'IN')
				    or (substr(cmd,1,2) = 'in')))
			then lib$tra_ebc_asc(buffer, Abuffer)
		  else Abuffer := buffer;
	          if (out_type = 1) and ((substr(cmd,1,2) = 'OU')
				    or (substr(cmd,1,2) = 'ou'))
			then lib$tra_ebc_asc(buffer, Abuffer)
		  else Abuffer := buffer;
                  WRITELN(esc, '[16;1H', esc, '[J');
                  FOR index := 1 to show DO
                     IF (Abuffer[index] <= CHR(30)) or (Abuffer[index] >= CHR(105))
                     THEN WRITE( '  ~':3 )
                     ELSE WRITE(  '  ':2, Abuffer[index]:1 );
                  writeln;
                  FOR index := 1 to show DO
                     WRITE( ord(Abuffer[index]):3 );
                  writeln;
                  FOR index := (show+1) to (show*2) DO
                     IF (Abuffer[index] <= CHR(30)) or (Abuffer[index] >= CHR(105))
                     THEN WRITE( '  ~':3 )
                     ELSE WRITE(  '  ':2, Abuffer[index]:1 );
                  writeln;
                  FOR index := (show+1) to (show*2) DO
                     WRITE( ord(Abuffer[index]):3 );
                  writeln;
                  writeln;
                  eval.buf[1] := Abuffer[1];      eval.buf[2] := Abuffer[2];
                  eval.buf[3] := Abuffer[3];      eval.buf[4] := Abuffer[4];
                  eval.buf[5] := Abuffer[5];      eval.buf[6] := Abuffer[6];
                  eval.buf[7] := Abuffer[7];      eval.buf[8] := Abuffer[8];
                  WRITE('REAL1 = ', eval.num[1] );
		  WRITELN(' INTEGER1 = ', eval.nit[1]:12 );
                  WRITE('REAL2 = ', eval.num[2] );
		  WRITELN(' INTEGER2 = ', eval.nit[2]:12 );
                  END;
                restore_cursor;
                WRITELN('Information - ', transfer:6, ' Record Size. ', rec_num:6);
              END;
            END;

        7 : BEGIN			{ WRITE }
              IF out_type <> in_type then
		    begin
			if out_type = 1 then
			   lib$tra_asc_ebc(buffer, Ebuffer)
			else lib$tra_ebc_asc(buffer, Ebuffer);
			buffer := Ebuffer;
		    end;
              IF return <> sys_eof THEN
                 return := ntran_write (oucb, transfer, buffer)
              ELSE return := ntran_writeof( oucb );
              IF NOT ODD(return) THEN load_error (return, 'N/A');
            END;

        9 : BEGIN			{ HELP }
              ctrlhelp(cmd);
	      set_top(cmd);
              save_cursor;
            END;

       11 : BEGIN			{ WEOF }
	       IF (in_type <> 0) or (out_type <> 0) then
	          WRITELN('Error - Use only on ASCII format devices.')
	       else
		  begin
                    return := ntran_writeof( wrkcb );
	            WRITELN('EOF has been written to current (Command Level) device');
		  end;
            END;

       12 : BEGIN			{ LABEL }
	       IF (in_type <> 0) or (out_type <> 0) then
	          WRITELN('Error - Use only on ASCII format devices.')
	       else
		  begin
		    WRITELN('Warning - ', cmd:5, 'device must be labeled ');
		    return := ntran_rewind (wrkcb);
		    return := ntran_read (wrkcb, load_maxsize, buffer);
		    transfer := cvt_w_l(wrkcb[4]);
		    WRITELN('The old tape label is  >>  ', substr(buffer,5,6):6 );
		    WRITE('Enter the 6-Character label desired.  >>  ');
		    IF NOT EOLN THEN READLN( new_label ) ELSE READLN;
		    FOR index := 1 TO 6 DO
		    buffer[index+4] := new_label[index];
		    return := ntran_rewind (wrkcb);
		    return := ntran_write (wrkcb, transfer, buffer)
		  end;
            END;

       13 : BEGIN			{ FEOF }
	       IF substr(cmd,5,1) = 'E' then
	          WRITELN('Error - Use only on ASCII format device.')
	       else
		  begin
	          WRITELN('Warning - Tape may not have EOF before EOT.');
	          WHILE return <> sys_eof DO
                     BEGIN
                       return := ntran_read (wrkcb, load_maxsize, buffer);
                       rec_num := rec_num + 1;
                     END;
                  WRITELN('INFORMATION - ', rec_num:6, ' Record Number.');
		  end;
            END;

        OTHERWISE
           WRITELN('Error - Invalid Command - ',cmnd2:5 );
           cmnd2 := '     ';
           WRITELN;

      END;

      IF cmnd2 <> '     ' THEN prv_cmd := cmnd2;
      WRITE(cmnd2:5,' >>  ');
      cmnd2 := '     ';
      IF NOT EOLN THEN READLN(cmnd2) ELSE READLN;
      IF cmnd2 = '     ' THEN cmnd2 := prv_cmd;
      WRITELN;

   END

END;

PROCEDURE datatype(cmd : com_string);

VAR
   loc_get   : char;

   BEGIN

     WRITELN('****** Setting INPUT/OUTPUT data type');
     WRITELN;
     WRITELN('****** Enter <I> for input device, or');
     WRITELN('****** Enter <O> for output device.  Default is <I>.');
     WRITE('***** >>  ');
     IF NOT EOLN THEN READLN(loc_get) ELSE READLN;
     WRITELN;
     IF (loc_get <> 'O') and (loc_get <> 'o') then loc_get := 'I'
     else loc_get := 'O';
     IF ((substr(cmd,1,1) = 'E') or (substr(cmd,1,1) = 'e')) and
	 (loc_get = 'I') then in_type := 1;
     IF ((substr(cmd,1,1) = 'A') or (substr(cmd,1,1) = 'a')) and
	 (loc_get = 'I') then in_type := 0;
     IF ((substr(cmd,1,1) = 'E') or (substr(cmd,1,1) = 'e')) and
	 (loc_get = 'O') then out_type := 1;
     IF ((substr(cmd,1,1) = 'A') or (substr(cmd,1,1) = 'a')) and
	 (loc_get = 'O') then out_type := 0;

   END;

PROCEDURE accept_commands;

VAR
  loc       : integer;
  command   : com_string;

  BEGIN

  reset_screen;
  in_type := 0;
  out_type := 0;
  WRITELN(esc, '[1;12H', esc, '#3SET READ DEVICE');
  WRITELN(esc, '[2;12H', esc, '#4SET READ DEVICE');
  WRITELN('Valid Commands are:   IN    OUT    EXIT   HELP   ASCII   EBCDI');
  WRITELN;
  WRITELN(esc, '[5;19r', esc, '[6;1H');
  command := 'IN   ';
  WRITE(command:5 ,'>>  ');
  IF NOT EOF THEN READLN(command) ELSE sys$exit(0);
  WRITELN;
  WHILE ((substr(command,1,2) <> 'EX') AND (substr(command,1,2) <> 'ex')) DO 
     BEGIN
       WHILE (substr(command,1,1) <> 'I') and 
             (substr(command,1,1) <> 'i') and 
	     (substr(command,1,1) <> 'O') and
	     (substr(command,1,1) <> 'o') and
             (substr(command,1,1) <> 'H') and 
             (substr(command,1,1) <> 'h') and
             (substr(command,1,1) <> 'A') and 
             (substr(command,1,1) <> 'a') and
             (substr(command,1,2) <> 'EB') and 
             (substr(command,1,2) <> 'eb') and
             (substr(command,1,2) <> 'EX') and 
             (substr(command,1,2) <> 'ex') DO 
          BEGIN
            WRITE(command:5 ,'>>  ');
            command := '     ';
            IF NOT EOF THEN READLN(command) ELSE sys$exit(1);
            WRITELN;
          END;

       save_cursor;

       loc := 0;
       IF (substr(command,1,1)  = 'I') or 
          (substr(command,1,1)  = 'i') then 
	  begin
             loc := 1;
	     if in_type = 0 then command := 'IN  A'
	     else command := 'IN  E'
	  end;
       IF (substr(command,1,1)  = 'O') or 
          (substr(command,1,1)  = 'o') then 
	  begin
             loc := 2;
	     if out_type = 0 then command := 'OUT A'
	     else command := 'OUT E'
	  end;
       IF (substr(command,1,2)  = 'EX') or 
          (substr(command,1,2)  = 'ex') then
	  begin
             loc := 3;
	     command := 'EXIT '
	  end;
       IF (substr(command,1,1)  = 'H') or 
          (substr(command,1,1)  = 'h') then
	  begin
             loc := 4;
	     command := 'HELP '
	  end;

       IF ((substr(command,1,2)  = 'EB')  or 
           (substr(command,1,2)  = 'eb')) then 
	  begin
             loc := 5;
	     command := 'EBCDI'
	  end;
       IF ((substr(command,1,1)  = 'A')   or 
           (substr(command,1,1)  = 'a'))  then 
	  begin
             loc := 5;
	     command := 'ASCII'
	  end;

       CASE LOC OF
            1 : BEGIN
                  wrkcb := icb;
                  control(wrkcb, command);
                END;


            2 : BEGIN
                  wrkcb := oucb;
                  control(wrkcb, command);
                END;


            3 : sys$exit(1);

	    4:  ctrlhelp(command);

	    5:  datatype(command);

       END;

       reset_screen;
       WRITELN(esc, '[1;12H', esc, '#3SET READ DEVICE');
       WRITELN(esc, '[2;12H', esc, '#4SET READ DEVICE');
       WRITELN('Valid Commands are:   IN    OUT    EXIT   HELP   ASCII   EBCDI');
       WRITELN;
       WRITELN(esc, '[5;19r', esc, '[6;1H');
       WRITE(command:5 ,'>>  ');
       command := '     ';
       IF NOT EOF THEN READLN(command) ELSE sys$exit(1);
       WRITELN;
       save_cursor
     END;

  END;

PROCEDURE image_rundown;

  BEGIN
  reset_screen;
  return := ntran_deassign (icb);
  return := ntran_deassign (oucb);
  END;


(*									*)
(* This is the main body of the program.				*)
(*									*)

BEGIN
initialize_input_output;
accept_commands;
image_rundown;
END.
